' -----------------------------------------------------
' Phantasie II
' Towns
' -----------------------------------------------------
' Decoded and commented by Franck Charlet
' -----------------------------------------------------

init_town:
    dim races(16)
    dim I%(114)
    dim items_list(180)

    S$ = "+--------------------------------------+"

    runes_list(0) = "AR"
    runes_list(1) = "ER"
    runes_list(2) = "FR"
    runes_list(3) = "WT"
    runes_list(4) = "GOD"
    SA = 16385
    town_data_addr = 2048
    XA = 1990
    def get_roster_character_address(X) = SA + 170 * WG + X
    def fn Q(X) = 2 * get_character_data(X) + 30
    def fn W(X) = peek(SA + 170 * WG + X)
    GoldPot = SA + 1199
    def get_character_data(X) = peek(SA + 170 * PM + X)
    def get_party_gold(X) = peek(GoldPot + 1) + 256 * peek(GoldPot)          ' Read 16 bit
    def character_data_address(X) = SA + 170 * PM + X
    def fn D(X) = (2 ^ X * int(MF / 2 ^ X)) <> (2 ^ (X - 1) * int(MF / 2 ^ (X - 1)))

    ' Read the 180 items names from the file
    &read "ITEM"
    for X = 1 to 180
        input items_list(X)
    next
    &end

    ' Read the 6 classes data
    for X = 0 to 5
        read classes(X)
    next

    ' Read the 17 races data (16 + 1 empty entry)
    for X = 0 to 16
        read races(X)
    next
    data "THIEF", "MONK", "RANGER", "FIGHTER", "PRIEST", "WIZARD"
    data "HUMAN", "DWARF", "ELF", "HALFLING", "GNOME", "PIXIE", "SPRITE", "OGRE", "GNOLL", "TROLL", "KOBOLD", "ORC", "GOBLIN", "LIZARD MAN", "MINOTAUR", "UNDEAD",

    status_list(0) = "DEAD"
    status_list(1) = "OKAY"
    origin_land(0) = "GELNOR"
    origin_land(1) = "FERRONRAH"
    origin_land(2) = "NETHERWORLD"
    text
    home
    ' Load characters roster file at 0x2021 / 8225
    &recall "CHAR2"
    ' Retrieve the value of the key which was pressed during the main menu (SA + 1196 = 17581)
    if peek(SA + 1196) > 0 then
        goto recall_saved_game
    end if
    cur_characters = peek(SA + CHARACTERS_IN_PARTY)
    town = peek(SA + 1191)

enter_town:

    for PM = 0 to 6
        gosub get_character_name
        poke character_data_address(CHARACTER_CURRENT_TOWN), town
    next
    gosub divide_gold_and_xp
    poke 0xC010, 0

    ' Modify age
    for PM = 0 to cur_characters
        DM = peek(SA + 1194) / 7
        LC = character_data_address(CHARACTER_AGE_HI)
        gosub modify_16bit_value
    next

    ' clear:
    ' 1194/1195 = DAYS OUT
    ' 1197/1198 = EXPERIENCE EARNED
    ' 1199/1200 = GOLD EARNED
    for X = SA + 1194 to SA + 1200
        poke X, 0
    next

    ' Handle characters aging
    DM = 0
    for PM = 0 to cur_characters
        if get_character_data(CHARACTER_AGE_HI) < .7 * get_character_data(CHARACTER_LIFESPAN) then
            goto lbl_580
        end if
        if get_character_data(CHARACTER_AGE_HI) >= get_character_data(CHARACTER_LIFESPAN) then
            goto lbl_560
        end if
        if get_character_data(CHARACTER_ALREADY_AGED) > 0 then
            goto lbl_580
        end if
        DM = 1
        print character_names(PM) " HAS AGED"
        poke 0xC010, 0
        get A$
        poke character_data_address(CHARACTER_ALREADY_AGED), 1

lbl_560:

        ' 0.8f or 0
        DM = sgn(DM) * .8
        if DM = 0 then
            print character_names(PM) " HAS DIED"
            poke 0xC010, 0
            poke character_data_address(CHARACTER_STATUS), CHARACTER_DEAD
            get A$
        end if
        ' Reduce character stats to 80% of their former values
        for X = CHARACTER_STRENGTH to 7
            poke character_data_address(X), DM * get_character_data(X)
        next
        gosub calc_character_shield_armor_weapon_points

lbl_580:

    next PM
    goto lbl_800

' -----------------------------------------------------
divide_gold_and_xp:

    home
    print "--DIVIDING GOLD AND EXPERIENCE POINTS--"
    print
    ' Avoid a divide by zero
    TS = .00001
    if cur_characters < 0 then
        return
    end if
    for PM = 0 to cur_characters
        print "SHARES FOR " character_names(PM) ;

lbl_620:

        get A$
        ' Store share for that character
        S%(PM) = val(A$)
        if S%(PM) < 0 or S%(PM) > 3 then
            goto lbl_620
        end if
        ' Can't have 0 share for a character who's alive & awake
        ' (we'll only display characters that fit with 1 share minimim)
        if S%(PM) = 0 and get_character_data(CHARACTER_STATUS) = CHARACTER_OKAY then
            goto lbl_620
        end if
        print " " S%(PM)
        TS = TS + S%(PM)
    next
    ' TS = total share
    ' Retrieve the gained experience and divide it
    EX = 10 * (256 * peek(SA + 1197) + peek(SA + 1198)) / TS
    GL = get_party_gold(0) / TS
    print
    print "EACH SHARE IS " int(EX) " XP AND " int(GL) " GOLD"
    for PM = 0 to cur_characters
        DM = S%(PM) * GL
        LC = character_data_address(CHARACTER_GOLD_HI)
        gosub modify_16bit_value

        DM = S%(PM) * EX + 2 ^ 16 * get_character_data(CHARACTER_EXP_HI) + 256 * get_character_data(CHARACTER_EXP_MID) + get_character_data(CHARACTER_EXP_LO)
        ' 8388608 (0x800000)
        if DM > 2 ^ 23 then
            DM = 2 ^ 23
        end if
        ' Store the 24 bit value
        poke character_data_address(CHARACTER_EXP_HI), DM / 2 ^ 16
        DM = DM - 2 ^ 16 * get_character_data(CHARACTER_EXP_HI)
        poke character_data_address(CHARACTER_EXP_MID), DM / 256
        poke character_data_address(CHARACTER_EXP_LO), DM - 256 * get_character_data(CHARACTER_EXP_MID)
    next
    for PS = 1 to 2000
    next
    poke SA + 1199, 0
    poke SA + 1200, 0
    return

' -----------------------------------------------------
recall_saved_game:

    if peek(SA + 1196) = 2 then
        goto set_starting_town_from_character
    end if
    town = TOWN_PIPPACOTT           ' Start new game in PIPPACOTT
    goto start_new_game

' -----------------------------------------------------
set_starting_town_from_character:

    gosub display_guild_members
    input "WHICH CHARACTER? " ; A
    ' Look like there's room for 37 characters in roster
    if A < 1 or A > 37 then
        goto set_starting_town_from_character
    end if
    PM = A - 49
    if get_character_data(CHARACTER_AVAILABLE) = 0 then
        goto set_starting_town_from_character
    end if
    ' Set the current town according to the one of the selected character
    town = get_character_data(CHARACTER_CURRENT_TOWN)
    home

' -----------------------------------------------------
start_new_game:

    cur_characters = -1
    ' Clear characters pool data
    for PM = 0 to 6
        poke character_data_address(CHARACTER_STATUS), CHARACTER_DEAD
        poke character_data_address(CHARACTER_AVAILABLE), 0
    next

lbl_800:

    ' Load the town data at 0x801 2049
    &recall "TWN2" + str$(town), 2049
    ' Read town's name (20 chars)
    for X = town_data_addr + (TOWN_NAME + 1) to town_data_addr + (TOWN_NAME + 20)
        town_name = town_name + chr$(peek(X))
    next
    if peek(SA + 1196) > 0 then
        htab 10
    end if
    vtab 15
    print "WELCOME TO " town_name
    CF = peek(town_data_addr + (TOWN_PRIZES_MULTIPLIER + 1)) / 100

    &recall "TINFO"             ' Load at 0x8fc / 2300
    if peek(SA + 1196) > 0 then
        goto lbl_890
    end if

    ' Clear 18 (from 0 to 17 included) random values in the prizes list
    ' among the first 110 values (most probably 109)
    ' (maybe it's used to randomize the market a bit ?)
    for X = 0 to 17
        poke town_data_addr + rnd(1) * 111, 0
    next
    ' Set 8 (or 9 ?) values
    for X = 0 to 8
        poke town_data_addr + peek(town_data_addr + (TOWN_STOCK_RANDOMIZER + 1) + rnd(1) * peek(SA + PARTY_GLOBAL_LEVEL) * 3), 1
    next

    ' ???
    if peek(SA + 1269) > 0 then
        home
        NW = 1
        goto lbl_8130
    end if

lbl_890:

    ' Clear the passed value
    poke SA + 1196, 0
    goto display_main_town_screen

' -----------------------------------------------------
display_current_equip_summary:

    text
    home
    print S$
    vtab 2
    htab 8
    print town_name " (" town ")"
    print
    cur_characters = -1

    for PM = 0 to 5
        if get_character_data(CHARACTER_AVAILABLE) = 0 then
            goto lbl_1004
        end if
        cur_characters = cur_characters + 1
        WG = cur_characters
        if WG = PM then
            goto lbl_1004
        end if
        character_names(cur_characters) = character_names(PM)
        LB = get_roster_character_address(CHARACTER_STRENGTH)
        AL = character_data_address(CHARACTER_STRENGTH)
        gosub copy_roster_character
        poke character_data_address(CHARACTER_AVAILABLE), 0
        poke character_data_address(CHARACTER_STATUS), CHARACTER_DEAD
        SC = 0

lbl_1004:

    next

    ' Display it
    for PM = 0 to 6
        if get_character_data(CHARACTER_AVAILABLE) = 0 then
            goto lbl_1015
        end if
        print PM + 1 " " character_names(PM) ;
        htab 19
        print " " left$(races(get_character_data(CHARACTER_RACE)), 7) tab(28) classes(get_character_data(CHARACTER_CLASS)) tab(36) status_list(get_character_data(CHARACTER_STATUS))

lbl_1015:

    next
    vtab 11
    print S$
    poke 34, 11
    return

' -----------------------------------------------------
enter_armory:

    if cur_characters = -1 then
        goto display_town_main_menu
    end if
    X = 0
    Y = 1
    gosub ask_character_shopping
    goto display_shop

ask_character_shopping:

    home
    vtab 23
    print "WHO WILL SHOP" ;
    get A$
    print
    PM = val(A$) - 1
    if PM < 0 or PM > cur_characters then
        goto ask_character_shopping
    end if
    return

display_shop:

    text
    home
    print S$

lbl_1108:

    WG = 2 * get_character_data(CHARACTER_STRENGTH) + get_character_data(CHARACTER_DEXTERITY)
    goto lbl_1235

enter_armory_items_from_start:

    X = 0
    Y = 1

enter_armory_items:

    print
    poke 34, 1
    poke 35, 17
    vtab 17

lbl_1115:

    U = 0
    ' Not available in this town
    if peek(town_data_addr + Y) < 1 then
        goto next_item
    end if
    ' Calc the price
    CT = int(peek(XA + Y + 399) ^ 2 / 5 * CF + 1)
    print Y ; tab(5) ; items_list(Y) ; tab(20) ; CT " GP" ;
    X = X + 1
    if Y < 21 and get_character_data(CHARACTER_STRENGTH) >= Y then
        U = 1
        DM = Y
    end if
    if Y > 20 and Y < 31 and get_character_data(CHARACTER_STRENGTH) >= Y * 2 - 40 then
        U = 1
        DM = Y - 20
    end if
    if Y > 30 and Y < 41 and get_character_data(CHARACTER_STRENGTH) >= Y * 2 - 64 then
        U = 1
        DM = Y - 29
    end if
    if Y > 40 and Y < 71 and WG >= Y * 2 - 80 then
        U = 1
        DM = int(Y / 2 - 20)
    end if
    if Y > 70 and Y < 101 and WG >= Y * 2 - 140 then
        U = 1
        DM = int(Y * .75 - 50)
    end if
    if Y > 100 then
        print
        goto next_item
    end if
    if U = 1 then
        print tab(26) "  " DM " PTS"
        goto next_item
    end if
    print tab(26) "  UNUSABLE"

next_item:

    Y = Y + 1
    if Y < 181 and X < 14 then
        goto lbl_1115
    end if

lbl_1235:

    poke 35, 24
    vtab 18
    print S$

' -----------------------------------------------------
enter_armory_menu:

    poke 34, 18
    home
    print tab(7) "--" left$(character_names(PM), 15) " " get_party_gold(0) " GOLD--"
    print "L)IST ITEMS          C)ONTINUE LIST"
    print "B)UY ITEM          A)NOTHER SHOPPER"
    print tab(24) "E)XIT ARMORY"
    gosub command_prompt
    if A$ = "L" then
        goto enter_armory_items_from_start
    end if
    if A$ = "E" then
        gosub calc_character_shield_armor_weapon_points
        goto display_main_town_screen
    end if
    if A$ = "A" then
        gosub calc_character_shield_armor_weapon_points
        home
        gosub ask_character_shopping
        goto lbl_1108
    end if
    if A$ = "C" then
        goto continue_enter_armory_items
    end if
    if A$ <> "B" then
        goto enter_armory_menu
    end if

    ' Buy an item at the armory
    vtab 23
    htab 1
    input "BUY WHICH ITEM " ; A
    if A < 1 or A > 180 then
        goto enter_armory_menu
    end if
    home
    vtab 23
    ' Price of the item
    CT = int(peek(XA + A + 399) ^ 2 / 5 * CF + 1)
    if peek(town_data_addr + A) < 1 then
        print "THAT ITEM IS OUT OF STOCK"
        goto pause_armory_display
    end if
    if get_party_gold(0) < CT then
        print "YOU CAN NOT AFFORD IT"
        goto pause_armory_display
    end if
    IT = 0
    ' Get the first available slot
    for J = CHARACTER_ITEM_1 to CHARACTER_ITEM_9
        if get_character_data(J) > 0 then
            goto lbl_1320
        end if
        IT = J

lbl_1320:

    next J
    if IT = 0 then
        print "YOU HAVE NINE ITEMS ALREADY"
        goto pause_armory_display
    end if
    poke character_data_address(IT), A

    ' Pay for the item.
    LC = GoldPot
    DM = -1 * CT
    gosub modify_16bit_value

    ' Decrease the amount of stock available for this item
    poke town_data_addr + A, peek(town_data_addr + A) - 1
    goto enter_armory_menu

continue_enter_armory_items:

    X = 0
    if Y > 180 then
        Y = 1
    end if
    goto enter_armory_items

pause_armory_display:

    print
    for PS = 1 to 2000
    next
    goto enter_armory_menu

' -----------------------------------------------------
calc_character_values:

    if VL = 15 then
        return
    end if
    poke character_data_address(CHARACTER_SKILL_DEFAULT_ATTACK), fn Q(CHARACTER_STRENGTH)
    poke character_data_address(CHARACTER_SKILL_DEFAULT_PARRY), fn Q(CHARACTER_DEXTERITY)
    poke character_data_address(CHARACTER_SKILL_DEFAULT_FIND_ITEM), fn Q(CHARACTER_INTELLIGENCE)
    poke character_data_address(CHARACTER_SKILL_DEFAULT_SPOT_TRAP), fn Q(CHARACTER_INTELLIGENCE)
    poke character_data_address(CHARACTER_SKILL_DEFAULT_DISARM_TRAP), fn Q(CHARACTER_DEXTERITY)
    poke character_data_address(CHARACTER_SKILL_DEFAULT_LISTEN), fn Q(CHARACTER_INTELLIGENCE)
    poke character_data_address(CHARACTER_SKILL_DEFAULT_PICK_LOCK), fn Q(CHARACTER_DEXTERITY)
    poke character_data_address(CHARACTER_SKILL_DEFAULT_SWIM), fn Q(CHARACTER_DEXTERITY)
    poke character_data_address(CHARACTER_SKILL_DEFAULT_TOSS_ROCK), VL * 2 + 
                                                                    15 * (get_character_data(CHARACTER_RACE) = RACE_DWARF) +
                                                                    30 * (get_character_data(CHARACTER_RACE) = RACE_GNOME) +
                                                                    get_character_data(CHARACTER_DEXTERITY) * 4 +
                                                                    get_character_data(CHARACTER_STRENGTH) + 10

    ' VL = character level
    ' CL = character class
    poke character_data_address(CHARACTER_MAX_CLASS_MAGIC), peek(XA + 1050 + CL * 15 + VL)

    ' Set character skills (0 to 8)
    for X = 0 to 9
        A = (get_character_data(CHARACTER_SKILL_ATTACK + 2 * X) +
             peek(XA + 1140 + CL * 150 + VL * 10 + X)) / 2 +
             get_character_data(CHARACTER_SKILL_DEFAULT_ATTACK + 2 * X) - 49.5
        if A < 1 then
            A = 1
        end if
        if A > 255 then
            A = 255
        end if
        poke character_data_address(CHARACTER_SKILL_ATTACK + 2 * X), A
    next X

    ' Calculate hit points
    X = peek(XA + 310 + CL * 15 + VL) * (get_character_data(CHARACTER_CONSTITUTION) + 5) / 16
    WG = 0
    for J = 1 to X
        WG = WG + int(rnd(1) * 3 + 1)
    next J
    if WG <= get_character_data(CHARACTER_MAX_HIT_POINTS) then
        WG = get_character_data(CHARACTER_MAX_HIT_POINTS) + 1
    end if
    if WG > 255 then
        WG = 255
    end if
    poke character_data_address(CHARACTER_MAX_HIT_POINTS), WG

    
    for X = 0 to 59
        if peek(XA + 690 + CL * 60 + X) <> VL + 1 then
            goto lbl_1590
        end if
        ' Make it learnable
        poke character_data_address(X + CHARACTER_SPELL_HEALING_1), 2

lbl_1590:

    next
    poke character_data_address(CHARACTER_LEVEL), VL + 1
    return

' -----------------------------------------------------
modify_16bit_value:

    DM = DM + peek(LC) * 256 + peek(LC + 1)
    if DM < 0 then
        DM = 0
    end if
    if DM > 65535 then
        DM = 65535
    end if
    poke LC, DM / 256
    poke LC + 1, DM - 256 * peek(LC)
    return

' -----------------------------------------------------
create_new_guild_member:

    home
    print tab(11) "--NEW CHARACTER--"
    poke 34, 12

restart_character_creation:

    PM = 0
    home
    for WG = -48 to -9
        if fn W(CHARACTER_AVAILABLE) <> 0 then
            goto lbl_2207
        end if
        if PM = 0 then
            PM = WG
        end if

lbl_2207:

    next WG
    if PM = 0 then
        print
        print "NO ROOM"
        for PS = 1 to 2000
        next
        goto display_adventurers_guild_menu
    end if
    print
    for X = 0 to 4
        print X + 1, races(X)
    next
    print 6, "RANDOM CREATURE"
    J = 0
    print
    print "PICK RACE" ;
    get A$
    RC = val(A$)
    ' 6 possible choices
    if RC < 6 then
        goto no_random_race
    end if
    ' Pick a random race from RACE_SPRITE to RACE_UNDEAD
    RC = RACE_SPRITE + int(rnd(1) * 10)
    ' Check if there's already a character of the same race in the roster
    for WG = -48 to -9
        J = J + (RC - 1 = fn W(CHARACTER_RACE))
    next
    if J > 0 then
        RC = int(1 + rnd(1) * 5)
    end if

no_random_race:

    if RC < RACE_HUMAN then
        goto restart_character_creation
    end if
    home
    print races(RC - 1)
    print
    for X = 0 to 5
        print X + 1, classes(X)
    next
    print
    print "PICK CLASS" ;
    get A$
    CL = val(A$)
    ' Random races can only be thiefs or fighters
    
    if RC < 6 or CL = CLASS_THIEF or CL = CLASS_FIGHTER then
        goto lbl_2240
    end if
    goto restart_character_creation

lbl_2240:

    CL = CL - 1
    RC = RC - 1
    if CL < CLASS_THIEF or CL > CLASS_WIZARD then
        goto restart_character_creation
    end if
    ' Clean all character data
    for X = 0 to 169
        poke character_data_address(X), 0
    next
    'poke character_data_address(55), 1
    poke character_data_address(CHARACTER_AVAILABLE), PM + 49

    ' Calc character base values
    for X = 0 to 4
        Y = int(rnd(1) ^ .6 * 6) + int(rnd(1) ^ .6 * 6) + int(rnd(1) ^ .6 * 6) + 3
        Y = Y + peek(XA + 660 + CL * 5 + X) - 10
        if Y > 18 then
            Y = 18
        end if
        Y = Y + peek(XA + 585 + RC * 5 + X) - 10
        if Y < 3 then
            Y = 3
        end if
        poke character_data_address(X + 1), Y
    next

    home
    vtab 16
    print "STR " get_character_data(CHARACTER_STRENGTH) "  INT " get_character_data(CHARACTER_INTELLIGENCE) "  DEX " get_character_data(CHARACTER_DEXTERITY) "  CON " get_character_data(CHARACTER_CONSTITUTION) "  CHA " get_character_data(CHARACTER_CHARISMA)
    print
    print "KEEP THIS CHARACTER? " ;
    get A$
    if A$ = "N" then
        poke character_data_address(CHARACTER_AVAILABLE), 0
        goto display_equipment_and_return_guild_menu
    end if
    print "YES"
    gosub ask_character_name
    goto create_character

' -----------------------------------------------------
ask_character_name:

    print
    print "INPUT NAME (MAXIMUM 20 CHARACTERS)"
    X = 131
    print

loop_enter_character_name:

    get A$
    print A$ ;
    Y = asc(A$)
    ' Store the character into the name field
    poke character_data_address(X), Y * (Y <> 13)
    ' Done
    if Y = 13 then
        return
    end if
    ' Delete a character
    if Y = 8 then
        X = X - 1
        goto loop_enter_character_name
    end if
    if Y = 21 then
        goto loop_enter_character_name
    end if
    X = X + 1
    if X > 150 then
        return
    end if
    goto loop_enter_character_name

' -----------------------------------------------------
create_character:

    VL = 0
    gosub calc_character_values

    ' Select the shape that will be associated with the character
    if RC > RACE_SPRITE then
        poke character_data_address(CHARACTER_PICTURE), 26
        goto lbl_2360
    end if
    poke character_data_address(CHARACTER_PICTURE), 25
    if RC <> RACE_HUMAN and RC <> RACE_ELF then
        goto lbl_2360
    end if
    poke character_data_address(CHARACTER_PICTURE), 19
    if CL = CLASS_MONK or CL > CLASS_FIGHTER then
        poke character_data_address(CHARACTER_PICTURE), 27
    end if

lbl_2360:

    poke character_data_address(CHARACTER_STATUS), CHARACTER_OKAY
    ' Add 0x100/256 gold pieces
    poke character_data_address(CHARACTER_GOLD_HI), 1
    poke character_data_address(CHARACTER_CURRENT_TOWN), town
    poke character_data_address(CHARACTER_ORIGIN), 1 + (town > TOWN_FERRON)
    poke character_data_address(CHARACTER_ITEM_1), 4        ' SM SHIELD
    poke character_data_address(CHARACTER_ITEM_2), 48       ' CLUB
    poke character_data_address(CHARACTER_ITEM_3), 22       ' ROBES
    gosub calc_character_shield_armor_weapon_points

    ' Monks & wizard have FIREFLASH1 learned by default
    if CL = CLASS_MONK or CL = CLASS_WIZARD then
        poke character_data_address(CHARACTER_SPELL_FIREFLASH_1), 1
    end if
    ' HEALING1 is available to rangers & priest by default
    if CL = CLASS_RANGER or CL = CLASS_PRIEST then
        poke character_data_address(CHARACTER_SPELL_HEALING_1), 1
    end if

    poke character_data_address(CHARACTER_LUCK), 8 + rnd(1) * 6
    poke character_data_address(CHARACTER_CURRENT_HIT_POINTS), get_character_data(CHARACTER_MAX_HIT_POINTS)
    poke character_data_address(CHARACTER_MAX_MAGIC), 0.75f * get_character_data(CHARACTER_MAX_CLASS_MAGIC)
    poke character_data_address(CHARACTER_CURRENT_MAGIC), get_character_data(CHARACTER_MAX_MAGIC)
    poke character_data_address(CHARACTER_RACE), RC
    poke character_data_address(CHARACTER_CLASS), CL

    AG = get_character_data(CHARACTER_LUCK)
    if RC > RACE_SPRITE then
        AG = AG / 2.5f
    end if
    if RC = RACE_DWARF or RC = RACE_HALFLING or RC = RACE_GNOME then
        AG = AG * 2.5f
    end if
    if RC = RACE_ELF then
        AG = AG * 19
    end if
    poke character_data_address(CHARACTER_LIFESPAN), AG + .5

    ' Set random age for character
    AG = 260 + AG * 256 * (.1 + rnd(1) / 40)
    DM = AG
    LC = character_data_address(CHARACTER_AGE_HI)
    gosub modify_16bit_value

    ' Display all stats on screen
    gosub display_character_stats
    goto display_equipment_and_return_guild_menu

' -----------------------------------------------------
display_guild_members:

    text
    home
    print tab(11) "--GUILD MEMBERS--"
    print "#   NAME  CLASS TOWN #  NAME  CLASS TOWN"
    ' 40 characters max in char2 file
    for X = 1 to 40
        ' Start at -48
        PM = X - 49
        if get_character_data(CHARACTER_AVAILABLE) = 0 then
            goto lbl_2740
        end if
        Y = 20 * (X > 20)
        vtab X + 2 - Y
        htab Y + 1
        print X tab(Y + 4) ;
        ' Print character name
        for J = CHARACTER_NAME to 137
            print chr$(get_character_data(J)) ;
        next
        ' Race / Class / Status
        print tab(Y + 13) left$(classes(get_character_data(CHARACTER_CLASS)), 2) tab(Y + 18) get_character_data(CHARACTER_CURRENT_TOWN)

lbl_2740:

    next X
    vtab 23
    return

' -----------------------------------------------------
ask_character_training:

    home
    vtab 23
    print "WHO WANTS TRAINING" ;
    get A$
    PM = val(A$) - 1
    if PM < 0 or PM > cur_characters or get_character_data(CHARACTER_STATUS) = CHARACTER_DEAD then
        goto display_adventurers_guild_menu
    end if
    print
    print
    if get_character_data(CHARACTER_LEVEL) > 14 then
        print "NO ONE IS WISE ENOUGH TO TRAIN YOU"
        for PS = 1 to 4000
        next
        goto display_adventurers_guild_menu
    end if

    ' Get 24 bit value
    EX = get_character_data(CHARACTER_EXP_HI) * 2 ^ 16 +
         get_character_data(CHARACTER_EXP_MID) * 256 +
         get_character_data(CHARACTER_EXP_LO)

    J = (get_character_data(CHARACTER_CLASS) = CLASS_THIEF) *
        (2 * get_character_data(CHARACTER_DEXTERITY) +
         get_character_data(CHARACTER_STRENGTH)
        ) / 19.8 +
        (get_character_data(CHARACTER_CLASS) = CLASS_MONK) *
        (get_character_data(CHARACTER_STRENGTH) +
         get_character_data(CHARACTER_INTELLIGENCE) +
         get_character_data(CHARACTER_DEXTERITY)
        ) / 49.5 +
        (get_character_data(CHARACTER_CLASS) = CLASS_RANGER) *
        (get_character_data(CHARACTER_STRENGTH) +
         get_character_data(CHARACTER_INTELLIGENCE) +
         get_character_data(CHARACTER_CHARISMA)
        ) / 52.8

    J = J + (get_character_data(CHARACTER_CLASS) = CLASS_FIGHTER) *
            (get_character_data(CHARACTER_DEXTERITY) + 2 *
             get_character_data(CHARACTER_STRENGTH)
            ) / 33 +
            (get_character_data(CHARACTER_CLASS) = CLASS_PRIEST) *
            (get_character_data(CHARACTER_STRENGTH) + 2 *
             get_character_data(CHARACTER_INTELLIGENCE)
            ) / 36.3 +
            (get_character_data(CHARACTER_CLASS) = CLASS_WIZARD) *
            (2 * get_character_data(CHARACTER_INTELLIGENCE) +
             get_character_data(CHARACTER_LUCK)
            ) / 42.9

    if J = 0 then
        J = .00001
    end if

    XX = 1000 * (-.8 + 1.8 ^ get_character_data(CHARACTER_LEVEL)) / J
    EX = int(EX - XX)

    N$ = town_name + " TEMPLE"
    if get_character_data(CHARACTER_CLASS) <> CLASS_PRIEST then
        N$ = classes(get_character_data(CHARACTER_CLASS)) + "'S GUILD"
    end if
    home
    print
    print tab(8) "--" N$ "--"
    print
    print
    if EX > 0 then
        goto lbl_2900
    end if
    print character_names(PM) " YOU NEED " EX * -1
    print "EXPERIENCE POINTS TO GO UP A LEVEL" ;
    get A$
    goto display_adventurers_guild_menu

lbl_2900:

    GL = CF * (20 - get_character_data(CHARACTER_CHARISMA)) ^ 2 *
               get_character_data(CHARACTER_LEVEL) ^ 2.2
    ' Max cost for training
    if GL > 15000 then
        GL = 15000
    end if
    print "IT WILL COST " int(GL) " GOLD PIECES. OKAY?" ;

lbl_2906:

    get A$
    if A$ = "N" then
        goto display_adventurers_guild_menu
    end if
    if A$ <> "Y" then
        goto lbl_2906
    end if
    if get_party_gold(0) < GL then
        print
        print "YOU CAN NOT AFFORD IT"
        for PS = 1 to 2500
        next
        goto display_adventurers_guild_menu
    end if

    ' Take the gold away
    LC = GoldPot
    DM = -1 * GL
    gosub modify_16bit_value

    RC = get_character_data(CHARACTER_RACE)
    CL = get_character_data(CHARACTER_CLASS)
    VL = get_character_data(CHARACTER_LEVEL)
    gosub calc_character_values
    goto display_adventurers_guild_menu

' -----------------------------------------------------
ask_character_to_learn_spell:

    home
    vtab 23
    print "WHO WILL LEARN SPELLS" ;
    get A$
    PM = val(A$) - 1
    if PM < 0 or PM > cur_characters or get_character_data(CHARACTER_STATUS) = CHARACTER_DEAD then
        goto display_adventurers_guild_menu
    end if
    TS = get_character_data(CHARACTER_LEVEL) *
         get_character_data(CHARACTER_INTELLIGENCE) ^ 1.177 / 90 *
         (get_character_data(CHARACTER_CLASS) + 1 - 3 *
         (get_character_data(CHARACTER_CLASS) = CLASS_FIGHTER))
    J = 0
    for X = CHARACTER_SPELL_HEALING_1 to CHARACTER_SPELL_DIVINE_Ic
        if get_character_data(X) = 1 then
            J = J + 1
        end if
    next X
    if J > TS then
        print
        print
        print "YOU CAN NOT LEARN ANY MORE"
        for PS = 1 to 3000
        next
        goto display_adventurers_guild_menu
    end if
    home
    print tab(5) "--" left$(character_names(PM), 15) " LEARNING SPELLS--"
    print
    print "YOU CAN LEARN " int(TS - J + 1) " MORE SPELLS"
    CT = int(CF * 9000 / get_character_data(CHARACTER_CHARISMA) / get_character_data(CHARACTER_INTELLIGENCE) * get_character_data(CHARACTER_LEVEL) / (9 - get_character_data(CHARACTER_CLASS)))
    print "THE COST IS " CT " GOLD PIECES"
    print "FOR EACH TRY TO LEARN A SPELL"
    print
    print "POSSIBLE SPELLS: " ;
    ' Show the possible spells a character can learn
    for X = 1 to 60
        if get_character_data(X + (CHARACTER_SPELL_HEALING_1 - 1)) > 1 then
            print " #" X ;
        end if
    next
    print
    print
    input "WHICH SPELL (0 FOR NONE)? " ; A
    print
    if A < 1 or A > 60 then
        goto display_adventurers_guild_menu
    end if
    ' Already learned
    if get_character_data(60 + A) < 2 then
        goto display_adventurers_guild_menu
    end if
    if get_party_gold(0) < CT then
        print "YOU CAN NOT AFFORD IT"
        for PS = 1 to 3000
        next
        goto display_adventurers_guild_menu
    end if

    ' Take the gold away
    LC = GoldPot
    DM = -1 * CT
    gosub modify_16bit_value

    ' It takes time to learn a spell
    DM = 2
    LC = character_data_address(CHARACTER_AGE_HI)
    gosub modify_16bit_value

    ' And it may not even be learned successfully
    if rnd(1) * 100 < get_character_data(CHARACTER_INTELLIGENCE) * 5 then
        goto spell_learning_success
    end if
    print
    print "YOU FAILED"
    goto pause_learning_display

spell_learning_success:

    print
    print "YOU LEARNED IT"
    ' Learned
    poke character_data_address(60 + A), 1

pause_learning_display:

    for PS = 1 to 2000
    next
    goto ask_character_to_learn_spell

' -----------------------------------------------------
display_adventurers_guild_menu:

    home
    print tab(9) "--ADVENTURERS GUILD--"
    print
    print "PARTY MEMBERS:  S)PELLS  T)RAINING"
    print tab(17) ; "A)DD     D)ROP"
    print
    print "GUILD MEMBERS:  L)IST    N)EW MEMBER"
    print tab(17) ; "R)ENAME  P)URGE"
    print
    print tab(17) "E)XIT GUILD"
    gosub command_prompt
    on CM goto ask_character_to_add,
               display_adventurers_guild_menu,
               display_adventurers_guild_menu,
               ask_character_to_drop,
               display_town_main_menu,
               display_adventurers_guild_menu,
               display_adventurers_guild_menu,
               display_adventurers_guild_menu,
               display_adventurers_guild_menu,
               display_adventurers_guild_menu,
               display_adventurers_guild_menu,
               list_guild_members,
               display_adventurers_guild_menu,
               create_new_guild_member,
               display_adventurers_guild_menu,
               ask_character_to_purge,
               display_adventurers_guild_menu,
               ask_character_to_rename,
               ask_character_to_learn_spell,
               ask_character_training
    goto display_adventurers_guild_menu

' -----------------------------------------------------
list_guild_members:

    gosub display_guild_members
    get A$
    goto display_equipment_and_return_guild_menu

' -----------------------------------------------------
ask_character_to_add:

    home
    vtab 20
    input "ADD WHICH GUILD MEMBER? " ; J
    if J < 1 or J > 40 then
        goto display_adventurers_guild_menu
    end if
    PM = -1
    WG = J - 49
    if J > 37 then
        PM = 6
        goto lbl_3290
    end if
    ' Check our equip
    for WG = 0 to 5
        ' Check if it has already been recruited
        if fn W(CHARACTER_AVAILABLE) = J then
            PM = 100
        end if
        if fn W(CHARACTER_AVAILABLE) > 0 then
            goto lbl_3260
        end if
        if PM > -1 then
            goto lbl_3260
        end if
        PM = WG

lbl_3260:

    next WG
    if PM = -1 then
        print
        print "NO ROOM"
        for PS = 1 to 2000
        next
        goto display_adventurers_guild_menu
    end if
    if PM = 100 then
        print
        print "ALREADY HERE"
        for PS = 1 to 2000
        next
        goto display_adventurers_guild_menu
    end if
    WG = J - 49
    if fn W(CHARACTER_AVAILABLE) = 0 then
        print
        print "WHO?"
        for PS = 1 to 2000
        next
        goto display_adventurers_guild_menu
    end if
    DM = 0
    ' Check if character is in this town
    if fn W(CHARACTER_CURRENT_TOWN) = town then
        goto lbl_3290
    end if
    print
    print "NOT IN " town_name ". IT WILL TAKE"
    print "TWO MONTHS TO GET HERE OKAY? " ;
    get A$
    if A$ = "N" then
        goto display_adventurers_guild_menu
    end if
    DM = 4

lbl_3290:

    LB = character_data_address(CHARACTER_STRENGTH)
    AL = get_roster_character_address(CHARACTER_STRENGTH)
    gosub copy_roster_character
    gosub get_character_name
    ' Equip full (?)
    if PM = 6 then
        goto display_equipment_and_return_guild_menu
    end if
    poke character_data_address(CHARACTER_CURRENT_TOWN), town
    LC = character_data_address(CHARACTER_AGE_HI)
    gosub modify_16bit_value

display_equipment_and_return_guild_menu:

    gosub display_current_equip_summary
    goto display_adventurers_guild_menu

' -----------------------------------------------------
ask_character_to_drop:

    home
    vtab 23
    print "DROP WHOM " ;
    get A$
    PM = val(A$) - 1
    if PM = 6 then
        goto lbl_3515
    end if
    if PM < 0 or PM > cur_characters then
        goto display_adventurers_guild_menu
    end if
    gosub sub_3520

lbl_3515:

    poke character_data_address(CHARACTER_AVAILABLE), 0
    poke character_data_address(CHARACTER_STATUS), CHARACTER_DEAD
    goto display_equipment_and_return_guild_menu

sub_3520:

    WG = get_character_data(CHARACTER_AVAILABLE) - 49
    AL = character_data_address(0)
    LB = get_roster_character_address(0)
    gosub copy_roster_character
    return

get_character_name:

    character_names(PM) = ""
    ' Get character name from roster
    for X = CHARACTER_NAME to 150
        character_names(PM) = character_names(PM) + chr$(get_character_data(X))
    next
    return

' -----------------------------------------------------
divide_and_save:

    if get_party_gold(0) > 0 then
        gosub divide_gold_and_xp
    end if
    home
    vtab 23
    print "SAVING THE GAME"
    gosub save_game
    goto display_main_town_screen

save_game:

    for PM = 0 to 5
        if get_character_data(CHARACTER_AVAILABLE) > 0 then
            gosub sub_3520
        end if
    next
    &del "CHAR2"
    &store "CHAR2", 8225, 6800
    &del "TWN2" + str$(town)
    &store "TWN2" + str$(town), 2049, 250
    return

' -----------------------------------------------------
ask_character_to_purge:

    home
    vtab 23
    input "PURGE WHICH GUILD MEMBER? " ; A
    if A < 1 or A > 37 then
        goto display_adventurers_guild_menu
    end if
    PM = A - 49
    poke character_data_address(CHARACTER_AVAILABLE), 0
    poke character_data_address(CHARACTER_RACE), RACE_HUMAN
    for PM = 1 to 6
        if get_character_data(CHARACTER_AVAILABLE) <> A then
            goto lbl_4420
        end if
        poke character_data_address(CHARACTER_AVAILABLE), 0
        poke character_data_address(CHARACTER_STATUS), CHARACTER_DEAD

lbl_4420:

    next PM
    goto display_equipment_and_return_guild_menu

' -----------------------------------------------------
display_mystic_stuff:

    home
    print
    print tab(8) "--- " town_name " MYSTIC ---"
    print
    print
    print "THE WISE OLD MYSTIC PEERS INTO"
    print "HIS CRYSTAL BALL AND SPEAKS:"
    print
    print tab(5) "'YOUR SCORE IS " ;
    SC = 0
    Y = 0
    Q = 0
    Z = 0
    for X = 1 to 8
        I%(X) = 0
    next
    if cur_characters = -1 then
        goto lbl_4653
    end if
    for PM = 0 to cur_characters
        ' Any of the character have an ORB ?
        for J = CHARACTER_ITEM_1 to CHARACTER_ITEM_9
            Y = Y + (get_character_data(J) = 150)
        next
        Z = Z + (get_character_data(CHARACTER_WAND_OF_NIKADEMUS) > 99)
        if get_character_data(CHARACTER_SCORE) > 150 then
            Q = 5
        end if
        ' Check the runes bits
        for J = 1 to 8
            MF = get_character_data(CHARACTER_BEAST_RUNES)
            I%(J) = I%(J) + fn D(J)
        next
    next
    ' Sgn(X) = -1 if X < 0 / 0 if X = 0 / 1 if X > 0
    ' obviously it's only either 0 or 1 here
    for X = 1 to 8
        SC = SC + 10 * sgn(I%(X))
    next
    SC = SC + 20 * sgn(Y)

lbl_4653:

    adventurers_type(0) = "NOVICE ADVENTURERS"
    adventurers_type(1) = "EXPERIENCED ADVENTURERS"
    adventurers_type(3) = "READY TO DESTROY THE ORB"
    adventurers_type(2) = "NOT YET " + adventurers_type(3)
    adventurers_type(4) = "IN NEED OF THE ORB"
    adventurers_type(5) = "HEROES OF " + origin_land(1)

    ' Print the result on screen
    print SC "'"
    print
    Q = Q + (SC > 10) + (SC > 60) + (SC > 90)
    if SC > 50 and Y = 0 then
        Q = 4
    end if
    if Q > 5 then
        Q = 5
    end if

    print "YOU ARE " adventurers_type(Q)
    if Z > 0 then
        print
        print "ALSO HEROES OF " origin_land(0)
    end if
    for PM = 0 to cur_characters
        if get_character_data(CHARACTER_SCORE) < SC then
            poke character_data_address(CHARACTER_SCORE), SC
        end if
    next
    get A$
    goto display_town_main_menu

' -----------------------------------------------------
spell_transportation:

    text
    home
    vtab 7
    print "THE PARTY IS SUDDENLY ENGULFED          IN BLACKNESS..."
    print
    gosub save_game

lbl_4707:

    vtab 22
    input "TRANSPORT TO WHICH TOWN? " ; new_town
    new_town = int(new_town)
    if new_town < TOWN_PIPPACOTT or new_town > TOWN_DARK_ABYSS then
        home
        goto lbl_4707
    end if
    ' Can't transport to certain towns
    if (town < TOWN_BLACK_MOOR and new_town > TOWN_FERRON) or
       ((town = TOWN_BLACK_MOOR or town = TOWN_DEATHPORT) and
       (new_town < TOWN_BLACK_MOOR or new_town > TOWN_DEATHPORT)) or
       (town > TOWN_DEATHPORT and new_town < TOWN_DEMONIAC) then
        print "SPELL FAILED"
        for PS = 1 to 1500
        next
        goto display_main_town_screen
    end if
    town_name = ""
    town = new_town
    goto enter_town

ask_character_to_rename:

    home
    vtab 18
    input "CHANGE WHICH NAME (1-37) ?" ; PM
    if PM < 1 or PM > 37 then
        goto display_adventurers_guild_menu
    end if
    PM = PM - 49
    for X = CHARACTER_NAME to CHARACTER_NAME + 19
        poke character_data_address(X), 0
    next
    print
    gosub ask_character_name
    goto display_equipment_and_return_guild_menu

display_main_town_screen:

    gosub display_current_equip_summary

' -----------------------------------------------------
display_town_main_menu:

    home
    print
    print " GO TO:  A)RMORY   G)UILD   M)YSTIC"
    print tab(10) "IN)N   B)ANK   L)EAVE TOWN"
    print
    print tab(10) "C)AST SPELL"
    print tab(10) "U)SE ITEMS"
    print tab(10) "D)ISTRIBUTE AND SELL ITEMS"
    print tab(10) "I)NSPECT PARTY MEMBER"
    print tab(10) "S)AVE GAME"

unknown_main_menu_cmd:

    gosub command_prompt
    on CM goto enter_armory,
               display_bank_menu,
               ask_character_to_cast_spell,
               distribute_and_sell_items,
               unknown_main_menu_cmd,
               unknown_main_menu_cmd,
               display_adventurers_guild_menu,
               unknown_main_menu_cmd,
               ask_character_to_inspect,
               unknown_main_menu_cmd,
               unknown_main_menu_cmd,
               leave_town,
               display_mystic_stuff,
               display_inn_infos,
               unknown_main_menu_cmd,
               unknown_main_menu_cmd,
               unknown_main_menu_cmd,
               unknown_main_menu_cmd,
               divide_and_save,
               unknown_main_menu_cmd,
               use_item
    goto unknown_main_menu_cmd

' -----------------------------------------------------
display_character_items:

    text
    home
    print "   -- ITEMS OF " character_names(PM) " --"
    print S$
    vtab 3
    if get_character_data(CHARACTER_WAND_OF_NIKADEMUS) > 99 then
        print 0, "WAND OF NIK(USELESS NOW)" ;
    end if
    for X = CHARACTER_ITEM_1 to CHARACTER_ITEM_9
        Y = get_character_data(X)
        print X - 151, items_list(Y)
    next X
    print S$
    poke 34, 13
    return

' -----------------------------------------------------
calc_character_shield_armor_weapon_points:

    R = CHARACTER_ITEM_9
    poke character_data_address(CHARACTER_SHIELD_POINT_VALUE), 0
    poke character_data_address(CHARACTER_ARMOR_POINT_VALUE), 0
    poke character_data_address(CHARACTER_WEAPON_POINT_VALUE), 0

sub_5512:

    TM = get_character_data(CHARACTER_STRENGTH)
    WG = 2 * TM + get_character_data(CHARACTER_DEXTERITY)

    for Z = CHARACTER_ITEM_1 to R
        X = get_character_data(Z)
        if X = 0 or X > 100 then
            goto lbl_5570
        end if
        if X < 21 and X > get_character_data(CHARACTER_SHIELD_POINT_VALUE) and TM >= X then
            poke character_data_address(CHARACTER_SHIELD_POINT_VALUE), X
            poke character_data_address(CHARACTER_ITEM_SHIELD_TYPE), X
            goto lbl_5570
        end if
        if X < 31 and TM >= (2 * X - 40) and X - 20 > get_character_data(CHARACTER_ARMOR_POINT_VALUE) then
            poke character_data_address(CHARACTER_ARMOR_POINT_VALUE), X - 20
            poke character_data_address(CHARACTER_ITEM_ARMOR_TYPE), X
            goto lbl_5570
        end if
        if X < 41 and TM >= (2 * X - 64) and X - 29 > get_character_data(CHARACTER_ARMOR_POINT_VALUE) then
            poke character_data_address(CHARACTER_ARMOR_POINT_VALUE), X - 29
            poke character_data_address(CHARACTER_ITEM_ARMOR_TYPE), X
            goto lbl_5570
        end if
        if X < 71 and WG >= (2 * X - 80) and (X / 2 - 20) > get_character_data(CHARACTER_WEAPON_POINT_VALUE) then
            poke character_data_address(CHARACTER_WEAPON_POINT_VALUE), (X / 2 - 20)
            poke character_data_address(CHARACTER_ITEM_WEAPON_TYPE), X
            goto lbl_5570
        end if
        if WG >= (2 * X - 140) and (.75 * X - 50) > get_character_data(CHARACTER_WEAPON_POINT_VALUE) then
            poke character_data_address(CHARACTER_WEAPON_POINT_VALUE), (.75 * X - 50)
            poke character_data_address(CHARACTER_ITEM_WEAPON_TYPE), X
        end if

lbl_5570:

    next
    return

' -----------------------------------------------------
leave_town:

    home
    vtab 23
    if cur_characters < 0 then
        print "YOU MUST ASSEMBLE A GROUP"
        goto wait_3_seconds
    end if
    print "LEAVING " town_name
    gosub save_game
    SC = 0
    for PM = 0 to cur_characters
        if get_character_data(CHARACTER_SCORE) > SC then
            SC = get_character_data(CHARACTER_SCORE)
        end if
        poke character_data_address(CHARACTER_SKILL_WEAPON),
             get_character_data(CHARACTER_STRENGTH) / 6 + get_character_data(CHARACTER_LEVEL) *
             (abs(3.5 - get_character_data(CHARACTER_CLASS)) / 11)
    next PM
    DM = 0
    J = 0
    poke SA + PARTY_GLOBAL_SCORE, SC
    for PM = 0 to cur_characters
        DM = DM + get_character_data(CHARACTER_LEVEL)
        J = J + 1
    next PM
    poke SA + PARTY_GLOBAL_LEVEL, DM / J + .5
    poke SA + CHARACTERS_IN_PARTY, cur_characters
    ' Save coordinates (OUTTx number + X/Y coordinates)
    for X = 0 to 2
        poke SA + X + 1191, peek(town_data_addr + X + (TOWN_COORDINATES + 1))
    next
    call 62450
    &run "OUTDOORS"

' -----------------------------------------------------
display_bank_title:

    home
    print
    print tab(16) "--BANK--"
    print
    print "GOLD PIECES CARRIED: " get_party_gold(0)
    return

' -----------------------------------------------------
display_bank_menu:

    if cur_characters < 0 then
        goto display_town_main_menu
    end if
    gosub display_bank_title
    print
    print "W)ITHDRAW GOLD PIECES"
    print "D)EPOSIT CARRIED GOLD"
    print "E)XIT BANK"
    gosub command_prompt
    if A$ = "D" then
        gosub divide_gold_and_xp
        goto display_bank_menu
    end if
    if A$ = "W" then
        goto withdraw_gold
    end if
    goto display_town_main_menu

' -----------------------------------------------------
withdraw_gold:

    for PM = 0 to cur_characters

lbl_6225:

        MN = get_character_data(CHARACTER_GOLD_HI) * 256 + get_character_data(CHARACTER_GOLD_LO)
        gosub display_bank_title
        print
        print left$(character_names(PM), 9) " HAS " MN " GOLD PIECES IN BANK"
        if MN + get_party_gold(0) > 65000 then
            print "MAXIMUM WITHDRAWAL IS " 65000 - get_party_gold(0)
            print
        end if
        ' Select the amount to withdraw
        input "WITHDRAWAL = " ; A
        if A < 0 or A > MN then
            home
            goto lbl_6225
        end if
        if get_party_gold(0) + A > 65000 then
            A = 65000 - get_party_gold(0)
        end if
        LC = character_data_address(CHARACTER_GOLD_HI)
        DM = -A
        gosub modify_16bit_value
        DM = A
        LC = GoldPot
        gosub modify_16bit_value
        ' Stop the process if the pot reached 65000 gold pieces
        if get_party_gold(0) = 65000 then
            PM = cur_characters
        end if
    next PM
    goto display_bank_menu

' -----------------------------------------------------
' -----------------------------------------------------
' -----------------------------------------------------
use_amulet:

    print
    print "E)XAMINE, W)EAR OR L)EAVE" ;

lbl_6310:

    get A$
    if A$ = "L" then
        goto display_main_town_screen
    end if
    if A$ = "W" then
        print
        print "LOOKS NICE!"
        goto ask_item_number
    end if
    if A$ <> "E" then
        goto lbl_6310
    end if

    ' Examine item
    print
    print
    print "THE WRITING SEEMS MEANINGLESS BUT YOU"
    print "CAN'T STOP READING..."
    for PM = 0 to cur_characters
        MF = get_character_data(CHARACTER_AMULETS)
        ' Get bits from the amulets
        L = fn D(IM - 140)
        MF = get_character_data(CHARACTER_BEAST_RUNES)
        poke character_data_address(CHARACTER_BEAST_RUNES), MF + 2 ^ (IM - 141) * (1 - fn D(IM - 140)) * L
    next
    print
    print "YOU ARE CHANGED"
    for PS = 1 to 7000
    next
    goto display_main_town_screen

' -----------------------------------------------------
use_item:

    home
    vtab 23
    print "WHO WILL USE " ;
    get A$
    PM = val(A$) - 1
    if PM < 0 or PM > cur_characters then
        goto display_town_main_menu
    end if
    gosub display_character_items

ask_item_number:

    print
    print "USE WHAT " ;
    get A$
    IT = val(A$)
    if IT < 1 or IT > 9 then
        goto display_main_town_screen
    end if
    print IT
    IM = get_character_data(IT + (CHARACTER_ITEM_1 - 1))
    if IM > 100 and IM < 121 then
        goto use_magic_item
    end if
    if IM > 140 and IM < 149 then
        goto use_amulet
    end if
    if IM < 141 and IM > 119 then
        goto read_scroll
    end if
    ' Use the FILMON WAND
    if IM = 149 and get_character_data(CHARACTER_SCORE) <> 200 then
        goto lbl_6770
    end if
    if IM = 149 then
        poke character_data_address(IT + (CHARACTER_ITEM_1 - 1)), 0
        poke character_data_address(CHARACTER_SCORE), 250
        print "YOU ARE IMPROVED"
        for X = CHARACTER_STRENGTH to CHARACTER_CHARISMA
            Y = get_character_data(X)
            poke character_data_address(X), Y + (Y < 20) + (Y < 9) + (Y < 18) * (get_character_data(CHARACTER_LUCK) > 11)
        next
        goto wait_3_seconds
    end if

lbl_6770:

    print "UNUSABLE"
    goto ask_item_number

' -----------------------------------------------------
use_magic_item:

    print "ON WHOM " ;
    get A$
    WG = val(A$) - 1
    if WG < 0 or WG > cur_characters then
        goto display_main_town_screen
    end if
    poke character_data_address(IT + (CHARACTER_ITEM_1 - 1)), 0
    print WG + 1
    if IM < 111 then
        goto use_healing_item
    end if
    DM = IM * 3 - 330 + fn W(CHARACTER_CURRENT_MAGIC)
    if DM > fn W(CHARACTER_MAX_MAGIC) then
        DM = fn W(CHARACTER_MAX_MAGIC)
    end if
    print character_names(WG) " RECEIVES MAGIC"
    poke get_roster_character_address(CHARACTER_CURRENT_MAGIC), DM
    goto ask_item_number

' -----------------------------------------------------
use_healing_item:

    DM = (IM - 100) ^ 2 + fn W(CHARACTER_CURRENT_HIT_POINTS)
    if DM > fn W(CHARACTER_MAX_HIT_POINTS) then
        DM = fn W(CHARACTER_MAX_HIT_POINTS)
    end if
    print character_names(WG) " IS HEALED"
    poke get_roster_character_address(CHARACTER_CURRENT_HIT_POINTS), DM
    goto ask_item_number

wait_3_seconds:

    for PS = 1 to 3000
    next
    goto display_main_town_screen

' -----------------------------------------------------
ask_character_to_cast_spell:

    home
    vtab 17
    print "WHO WILL CAST THE SPELL " ;
    get A$
    PM = val(A$) - 1
    if PM < 0 or PM > cur_characters then
        goto display_town_main_menu
    end if
    if get_character_data(CHARACTER_STATUS) = CHARACTER_DEAD then
        goto display_town_main_menu
    end if
    print PM + 1
    print
    input "WHICH SPELL? " ; WS
    if WS < 1 or WS > 60 then
        goto display_town_main_menu
    end if
    print
    if get_character_data(WS + 60) <> 1 then
        print "YOU DO NOT KNOW THAT SPELL"
        goto wait_3_seconds
    end if
    DM = WS - 4 * int((WS - 1) / 4)
    if get_character_data(CHARACTER_CURRENT_MAGIC) < DM then
        print "YOU DO NOT HAVE ENOUGH MAGIC"
        goto wait_3_seconds
    end if

    ' Cast the spell
    poke character_data_address(CHARACTER_CURRENT_MAGIC), get_character_data(CHARACTER_CURRENT_MAGIC) - DM

    ' Resurrect all dead characters
    if WS = (CHARACTER_SPELL_RESURRECTION - 60) then
        goto spell_resurrection
    end if
    if WS <= (CHARACTER_SPELL_HEALING_4 - 60) then
        goto ask_spell_target
    end if
    if WS = (CHARACTER_SPELL_TRANSPORTATION - 60) then
        goto spell_transportation
    end if
    if WS > (CHARACTER_SPELL_DIVINE_II - 60) then
        print "THAT SPELL WILL NOT WORK IN TOWN"
    end if
    if WS = (CHARACTER_SPELL_DIVINE_II - 60) then
        print "THAT SPELL WILL NOT WORK IN THIS PLANE"
    end if
    goto wait_3_seconds

' -----------------------------------------------------
ask_spell_target:

    print "ON WHOM" ;

lbl_7015:

    get A$
    WG = val(A$) - 1
    if WG < 0 or WG > cur_characters then
        goto lbl_7015
    end if
    print " " WG + 1
    ' W works on WG
    DM = int(rnd(1) * (4 + 6 * TM * TM)) + fn W(CHARACTER_CURRENT_HIT_POINTS)
    if DM > fn W(CHARACTER_MAX_HIT_POINTS) then
        DM = fn W(CHARACTER_MAX_HIT_POINTS)
    end if
    print
    print character_names(WG) " IS HEALED"
    poke get_roster_character_address(CHARACTER_CURRENT_HIT_POINTS), DM
    goto wait_3_seconds

sub_7100:

    ST = fn W(CHARACTER_STRENGTH)
    SS = 2 * ST + fn W(CHARACTER_DEXTERITY)
    W = I%(Q)
    US = (W < 21) * (ST >= W) or (W > 20 and W < 31) * (ST >= W * 2 - 40) or (W > 30 and W < 41) * (ST >= W * 2 - 64)
    US = US or (W > 40 and W < 71) * (SS >= W * 2 - 80) or (W > 70) * (SS >= W * 2 - 140)
    return

' -----------------------------------------------------
display_party_brief_summary:

    vtab 1
    print "# NAME       CLASS  WEAPON ARMOR SHIELD"
    for PM = 0 to cur_characters
        print PM + 1 " " left$((character_names(PM)), 9) tab(13) classes(get_character_data(CHARACTER_CLASS)) tab(23) get_character_data(CHARACTER_WEAPON_POINT_VALUE) " " tab(30) get_character_data(CHARACTER_ARMOR_POINT_VALUE) " " tab(36) get_character_data(CHARACTER_SHIELD_POINT_VALUE) " "
    next
    vtab 8
    print S$
    return

' -----------------------------------------------------
display_inn_infos:

    home
    print tab(8) "--THE " town_name " INN--"
    print
    for PM = 0 to cur_characters
        if get_character_data(CHARACTER_STATUS) = CHARACTER_DEAD then
            goto lbl_7310
        end if
        print character_names(PM) " IS RESTING. "
        ' Did character use magic ?
        if get_character_data(CHARACTER_USED_MAGIC) = 0 then
            goto lbl_7260
        end if
        poke character_data_address(CHARACTER_USED_MAGIC), 0
        if rnd(1) * 100 > 20 * (get_character_data(CHARACTER_MAX_CLASS_MAGIC) - get_character_data(CHARACTER_MAX_MAGIC)) then
            goto lbl_7260
        end if
        poke character_data_address(CHARACTER_MAX_MAGIC), get_character_data(CHARACTER_MAX_MAGIC) + 1
        print tab(10) "MAGIC INCREASES."

lbl_7260:

        poke character_data_address(CHARACTER_CURRENT_MAGIC), get_character_data(CHARACTER_MAX_MAGIC)
        poke character_data_address(CHARACTER_CURRENT_HIT_POINTS), get_character_data(CHARACTER_MAX_HIT_POINTS)

lbl_7310:

    next PM
    print
    print
    vtab 23
    print "PRESS RETURN"
    get A$
    goto display_town_main_menu

' -----------------------------------------------------
ask_character_to_inspect:

    home
    vtab 23
    print "INSPECT WHOM " ;
    get A$
    PM = val(A$) - 1
    if PM < 0 or PM > cur_characters then
        goto display_town_main_menu
    end if
    if get_character_data(CHARACTER_AVAILABLE) = 0 then
        goto display_town_main_menu
    end if
    gosub display_character_stats
    goto display_main_town_screen

' -----------------------------------------------------
display_character_stats:

    text
    home
    for X = 131 to 149
        print chr$(get_character_data(X)) ;
    next
    htab 21
    print "ORIGIN: " origin_land(get_character_data(CHARACTER_ORIGIN))
    print "LEVEL " get_character_data(CHARACTER_LEVEL) " " races(get_character_data(CHARACTER_RACE)) " " classes(get_character_data(CHARACTER_CLASS)) " " status_list(get_character_data(CHARACTER_STATUS))
    print
    print "STRENGTH" tab(14) get_character_data(CHARACTER_STRENGTH) tab(27) "AGE  " int((get_character_data(CHARACTER_AGE_HI) * 256 + get_character_data(CHARACTER_AGE_LO)) / 26)
    print "INTELLIGENCE " get_character_data(CHARACTER_INTELLIGENCE) tab(20) "HIT POINTS  " get_character_data(CHARACTER_CURRENT_HIT_POINTS) "/" get_character_data(CHARACTER_MAX_HIT_POINTS)
    print "DEXTERITY    " get_character_data(CHARACTER_DEXTERITY) tab(25) "MAGIC  " get_character_data(CHARACTER_CURRENT_MAGIC) "/" get_character_data(CHARACTER_MAX_MAGIC)
    print "CONSTITUTION " get_character_data(CHARACTER_CONSTITUTION) tab(18) "GOLD IN BANK  " get_character_data(CHARACTER_GOLD_HI) * 256 + get_character_data(CHARACTER_GOLD_LO)
    print "CHARISMA" tab(14) get_character_data(CHARACTER_CHARISMA) tab(17) "EXPER. POINTS  " get_character_data(CHARACTER_EXP_HI) * 2 ^ 16 + get_character_data(CHARACTER_EXP_MID) * 256 + get_character_data(CHARACTER_EXP_LO)
    print "LUCK" tab(14) get_character_data(CHARACTER_LUCK) tab(24) " SCORE  " get_character_data(CHARACTER_SCORE)
    print
    print tab(10) "SKILLS:" tab(29) "EQUIPMENT:"
    print "ATTACK " get_character_data(CHARACTER_SKILL_ATTACK) tab(12) "SPOT TRAP   " get_character_data(CHARACTER_SKILL_SPOT_TRAP) tab(27) items_list(get_character_data(CHARACTER_ITEM_WEAPON_TYPE)) " (" get_character_data(CHARACTER_WEAPON_POINT_VALUE) ")"
    vtab 13
    print "PARRY  " get_character_data(CHARACTER_SKILL_PARRY) tab(12) "DISARM TRAP " get_character_data(CHARACTER_SKILL_DISARM_TRAP) tab(27) items_list(get_character_data(CHARACTER_ITEM_ARMOR_TYPE)) " (" get_character_data(CHARACTER_ARMOR_POINT_VALUE) ")"
    vtab 14
    print "SWIM   " get_character_data(CHARACTER_SKILL_SWIM) tab(12) "FIND ITEM   " get_character_data(CHARACTER_SKILL_FIND_ITEM) tab(27) items_list(get_character_data(CHARACTER_ITEM_SHIELD_TYPE)) " (" get_character_data(CHARACTER_SHIELD_POINT_VALUE) ")"
    vtab 15
    print "LISTEN " get_character_data(CHARACTER_SKILL_LISTEN) tab(12) "PICK LOCK   " get_character_data(CHARACTER_SKILL_PICK_LOCK)
    print tab(12) "TOSS ROCK   " get_character_data(CHARACTER_SKILL_TOSS_ROCK)
    poke 33, 25
    print "SPELLS:" ;
    for X = CHARACTER_SPELL_HEALING_1 to CHARACTER_SPELL_60
        if get_character_data(X) = 1 then
            print "  " X - 60 ;
        end if
    next
    WS = 0
    ST = 0
    W = 0
    SS = 0
    DM = 0
    RB = 0
    FL = 0
    for X = CHARACTER_ITEM_1 to CHARACTER_ITEM_9
        J = get_character_data(X)
        if J < 101 then
            goto lbl_7480
        end if
        WS = WS + (J > 150)
        ST = ST + (J < 141)
        W = W + (J < 111)
        SS = SS + (J < 121)
        DM = DM + (J > 140)
        RB = (J = 150) + RB
        FL = (J = 149) + FL

lbl_7480:

    next
    poke 33, 14
    poke 32, 26
    poke 34, 14
    home
    if W > 0 then
        print W " HL POTIONS"
    end if
    if SS > W then
        print SS - W " MG POTIONS"
    end if
    if ST > SS then
        print ST - SS " SCROLLS"
    end if
    SS = 1
    if WS > 0 then
        print WS " TREASURES"
    end if
    if DM - WS - FL - RB > 0 then
        print DM - WS - FL - RB " AMULETS"
    end if
    if get_character_data(CHARACTER_BEAST_RUNES) = 0 then
        goto lbl_7515
    end if
    MF = get_character_data(CHARACTER_BEAST_RUNES)
    print "BEAST RUNES" ;
    for X = 1 to 8
        if fn D(X) = 1 then
            print " " X ;
        end if
    next
    print

lbl_7515:

    if RB > 0 then
        print "--THE ORB--"
    end if
    if FL > 0 then
        print "-FILMON WAND-"
    end if
    if get_character_data(CHARACTER_WAND_OF_NIKADEMUS) > 99 then
        print "-WAND OF NIK-"
    end if
    ' Not used in this game
    if get_character_data(CHARACTER_AIR_RUNES) = 0 and
       get_character_data(CHARACTER_FIRE_RUNES) = 0 and
       get_character_data(CHARACTER_WATER_RUNES) = 0 then
        goto character_posses_no_runes
    end if
    print "RUNES:  " ;
    for X = 0 to 4
        if get_character_data(CHARACTER_AIR_RUNES + X) > 0 then
            print runes_list(X) " " ;
        end if
    next

character_posses_no_runes:

    text
    vtab 23
    get A$
    return

' -----------------------------------------------------
' Acts on whole party
spell_resurrection:

    for WG = 0 to cur_characters
        if (fn W(CHARACTER_STATUS) <> CHARACTER_STATUS_DEAD) or (fn W(CHARACTER_CONSTITUTION) < 1) then
            goto lbl_7630
        end if
        if rnd(1) * 100 > 50 + 5 * (get_character_data(CHARACTER_LEVEL) - fn W(CHARACTER_LEVEL)) then
            goto lbl_7630
        end if
        print character_names(WG) " IS ALIVE"
        poke get_roster_character_address(CHARACTER_STATUS), CHARACTER_STATUS_OKAY
        poke get_roster_character_address(CHARACTER_CURRENT_HIT_POINTS), fn W(CHARACTER_MAX_HIT_POINTS)
        WG = cur_characters

lbl_7630:

    next WG
    goto wait_3_seconds

    A$ = items_list(get_character_data(CHARACTER_ITEM_WEAPON_TYPE))
    B$ = items_list(get_character_data(CHARACTER_ITEM_ARMOR_TYPE))
    C$ = items_list(get_character_data(CHARACTER_ITEM_SHIELD_TYPE))
    return

' -----------------------------------------------------
' What the hell does that do ?
' (starts at 121 - 120 so SCROLL0 is not loaded here
' (it's loaded inside a dungeon)
read_scroll:

    text
    home
    vtab 23

lbl_8010:

    ' Load scroll at 0x400/1024
    &recall "SCROLLL" + str$(IM - 120)
    get A$
    if peek(1872) = 161 then
        IM = IM + 1
        goto lbl_8010
    end if
    poke 1275, 255
    goto display_main_town_screen

' -----------------------------------------------------
copy_roster_character:

    poke 980, AL / 256
    poke 979, AL - 256 * peek(980)
    poke 983, LB / 256
    poke 982, LB - 256 * peek(983)
    call 976
    return

' -----------------------------------------------------
distribute_and_sell_items:

    if cur_characters = -1 then
        goto unknown_main_menu_cmd
    end if
    text
    home
    gosub display_party_brief_summary

' -----------------------------------------------------
lbl_8130:

    vtab 22
    print "SORTING ITEMS"
    J = 0
    for PM = 0 to cur_characters
        if NW = 1 then
            goto lbl_8160
        end if
        for X = CHARACTER_ITEM_1 to CHARACTER_ITEM_9
            Y = get_character_data(X)
            if Y = 0 then
                goto lbl_8155
            end if
            J = J + 1
            I%(J) = Y
            poke character_data_address(X), 0

lbl_8155:

        next

lbl_8160:

        poke character_data_address(CHARACTER_ITEM_SHIELD_TYPE), 0
        poke character_data_address(CHARACTER_ITEM_ARMOR_TYPE), 0
        poke character_data_address(CHARACTER_ITEM_WEAPON_TYPE), 0
        gosub calc_character_shield_armor_weapon_points
    next
    NW = 0

    ' Add acquired items
    for X = SA + 1210 to SA + 1269
        if peek(X) > 0 then
            J = J + 1
            I%(J) = peek(X)
            poke X, 0
        end if
    next
    ' No items gained
    if J = 0 then
        goto display_main_town_screen
    end if
    for Y = 1 to J - 1
        A = 0
        for X = Y to J
            if I%(X) > (A) then
                A = I%(X)
                Q = X
            end if
        next
        I%(Q) = I%(Y)
        I%(Y) = A
    next Y
    gosub display_party_brief_summary
    K = 1
    vtab 9
    print tab(11) "ITEM" tab(27) "POSSIBLE USERS"
    poke 34, 9

lbl_8200:

    print
    home
    Y = K + 10
    if Y > J then
        Y = J
    end if
    for Q = K to Y
        A$ = items_list(I%(Q))
        print " " A$ tab(13) ;
        if I%(Q) < 21 then
            print I%(Q) " PT SHIELD" ;
            goto lbl_8280
        end if
        if I%(Q) < 31 then
            print I%(Q) - 20 " PT ARMOR" ;
            goto lbl_8280
        end if
        if I%(Q) < 41 then
            print I%(Q) - 29 " PT ARMOR" ;
            goto lbl_8280
        end if
        if I%(Q) < 71 then
            print int(I%(Q) / 2 - 20) " PT WEAPON" ;
            goto lbl_8280
        end if
        if I%(Q) < 101 then
            print int(I%(Q) * .75 - 50) " PT WEAPON" ;
            goto lbl_8280
        end if
        print tab(28) "ALL"
        goto lbl_8300

lbl_8280:

        htab 27
        for WG = 0 to cur_characters
            gosub sub_7100
            if US = 1 then
                print tab(28 + 2 * WG) WG + 1 ;
            end if
        next
        print

lbl_8300:

    next
    vtab 22
    print S$
    vtab 23
    gosub sub_8390
    for Q = K to Y

lbl_8330:

        vtab 10 + Q - K
        htab 1
        get A$
        WG = val(A$) - 1
        if WG = 6 then
            goto sell_item_to_shop
        end if
        if WG < 0 or WG > cur_characters then
            goto lbl_8330
        end if
        R = 0
        for PS = CHARACTER_ITEM_1 to CHARACTER_ITEM_9
            if fn W(PS) > 0 then
                goto lbl_8370
            end if
            R = PS

lbl_8370:

        next
        if R > 0 then
            goto lbl_8400
        end if
        vtab 23
        print character_names(WG) " HAS NINE ITEMS" tab(39) ;
        for PS = 1 to 3000
        next
        gosub sub_8390
        goto lbl_8330

sub_8390:

        htab 1
        print "GIVE ITEM TO WHICH PERSON? (7 TO SELL)" ;
        return

lbl_8400:

        poke get_roster_character_address(R), I%(Q)
        if I%(Q) > 100 then
            goto lbl_8420
        end if
        gosub sub_7100
        if US = 0 then
            goto lbl_8460
        end if

lbl_8420:

        PM = WG
        gosub sub_5512
        gosub display_party_brief_summary
        goto lbl_8460

' -----------------------------------------------------
sell_item_to_shop:

        ' Calc the gold the character will gain
        DM = peek(XA + 399 + I%(Q)) ^ 2 / 8
        LC = GoldPot
        gosub modify_16bit_value
        ' Increase the stock of this item
        WG = peek(town_data_addr + I%(Q)) + 1
        if WG > 99 then
            WG = 99
        end if
        poke(town_data_addr + I%(Q)), WG

lbl_8460:

        vtab 10 + Q - K
        print spc(38)
    next Q
    K = K + 11
    if K <= J then
        goto lbl_8200
    end if
    goto display_main_town_screen

' -----------------------------------------------------
command_prompt:

    vtab 23
    print
    vtab 23
    print "COMMAND:" ;
    get A$
    CM = asc(A$) - 64
    if CM < 1 or CM > 26 then
        goto command_prompt
    end if
    return
