/* --------------------------------
   Phantasie II CD32
   Characters handling
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   
       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* --------------------------------
   Includes
   -------------------------------- */
#include "main.h"
#include "characters.h"
#include "characters_sheets.h"
#include "towns.h"

/* --------------------------------
   Variables
   -------------------------------- */
char *Classes_List[] =
{
    "THIEF",
    "MONK",
    "RANGER",
    "FIGHTER",
    "PRIEST",
    "WIZARD"
};

char *Races_Create_List[] =
{
    "HUMAN",
    "DWARF",
    "ELF",
    "HALFLING",
    "GNOME",

    "RANDOM"
};

char *Races_List[] =
{
    "HUMAN",
    "DWARF",
    "ELF",
    "HALFLING",
    "GNOME",
    "PIXIE",
    "SPRITE",
    "OGRE",
    "GNOLL",
    "TROLL",
    "KOBOLD",
    "ORC",
    "GOBLIN",
    "LIZARD MAN",
    "MINOTAUR",
    "UNDEAD"
};

char *Status_List[] = 
{
    "DEAD",
    "OKAY",
    "SLEEPING"
};

char *Beast_Runes_List[] =
{
    "GIANT CONSTRICTOR RUNE",
    "WINGED DEVIL RUNE",
    "UNDEAD SNAKE RUNE",
    "DEMON-FLY RUNE",
    "WYVERN RUNE",
    "SNAKE-DRAGON RUNE",
    "GIANT WYVERN RUNE",
    "DRAGON-DEMON"
};

PARTY Party;

/* Index starts at 1 */
ITEM Items_List[MAX_ITEMS] =
{
    /* Shields */
    "GLOVE",
    "WOOD SHIELD",
    "WOOD SHIELD +1",
    "SMALL SHIELD",
    "SMALL SHIELD +1",
    "SMALL SHIELD +2",
    "SMALL SHIELD +3",
    "MEDIUM SHIELD",
    "MEDIUM SHIELD +1",
    "MEDIUM SHIELD +2",
    "MEDIUM SHIELD +3",     /* 10 */
    "LARGE SHIELD",
    "LARGE SHIELD +1",
    "LARGE SHIELD +2",
    "LARGE SHIELD +3",
    "GIANT SHIELD",
    "GIANT SHIELD +1",
    "GIANT SHIELD +2",
    "GIANT SHIELD +3",
    "GOD SHIELD",

    /* Armors */
    "CLOTHING",             /* 20 */
    "ROBES",
    "LEATHER",
    "HARD LEATHER",
    "RING MAIL",
    "SCALE MAIL",
    "CHAIN MAIL",
    "SPLINT MAIL",
    "BANDED MAIL",
    "PLATE MAIL",
    "CLOTH +1",             /* 30 */
    "ROBES +1",
    "LEATHER +1",
    "LEATHER +2",
    "RING MAIL +1",
    "RING MAIL +2",
    "CHAIN MAIL +1",
    "CHAIN MAIL +2",
    "GOD ROBES",
    "GOD ARMOR",

    /* Weapons */
    "STICK",                /* 40 */
    "KNIFE",
    "SMALL CLUB",
    "SMALL STAFF",
    "SMALL MACE",
    "DAGGER",
    "SMALL FLAIL",
    "CLUB",
    "MACE",
    "SMALL HAMMER",
    "SMALL AXE",            /* 50 */
    "STAFF",
    "SHORT SWORD",
    "FLAIL",
    "HAMMER",
    "PTCH FORK",
    "SPEAR",
    "AXE",
    "SWORD",
    "HEAVY MACE",
    "MAUL",                 /* 60 */
    "TRIDENT",
    "LARGE SPEAR",
    "LARGE AXE",
    "MORNING STAR",
    "PIKE",
    "LONG SWORD",
    "SPETUM",
    "BARDICHE",
    "HALBERD",
    "SMALL MACE +1",        /* 70 */
    "DAGGER +1",
    "SMALL MACE +2",
    "DAGGER +2",
    "DAGGER +3",
    "STAFF +1",
    "DAGGER +4",
    "FLAIL +1",
    "SPEAR +1",
    "AXE +1",
    "SWORD +1",             /* 80 */
    "SWORD +2",
    "SWORD +3",
    "LARGE AXE +1",
    "SWORD +4",
    "SWORD +5",
    "SWORD +6",
    "HALBERD +1",
    "SWORD +7",
    "HALBERD +2",
    "HALBERD +3",           /* 90 */
    "SWORD +10",
    "HALBERD +4",
    "HALBERD +5",
    "HALBERD +6",
    "HALBERD +7",
    "GOD KNIFE",
    "GOD MACE",
    "GOD AXE",
    "GOD SWORD",

    /* Potions */
    "HEAL POTION 1",        /* 100 */
    "HEAL POTION 2",
    "HEAL POTION 3",
    "HEAL POTION 4",
    "HEAL POTION 5",
    "HEAL POTION 6",
    "HEAL POTION 7",
    "HEAL POTION 8",
    "HEAL POTION 9",
    "HEAL POTION 10",       /* 109 */

    "MAGIC POTION 1",       /* 110 */
    "MAGIC POTION 2",
    "MAGIC POTION 3",
    "MAGIC POTION 4",
    "MAGIC POTION 5",
    "MAGIC POTION 6",
    "MAGIC POTION 7",
    "MAGIC POTION 8",
    "MAGIC POTION 9",
    "MAGIC POTION 10",

    /* Scrolls */
    "SCROLL 0",             /* 120 */
    "SCROLL 1",
    "SCROLL 2",
    "SCROLL 3",
    "SCROLL 4",
    "SCROLL 5",
    "SCROLL 6",
    "SCROLL 7",
    "SCROLL 8",
    "SCROLL 9",
    "SCROLL 10",            /* 130 */
    "SCROLL 11",
    "SCROLL 12",
    "SCROLL 13",
    "SCROLL 14",
    "SCROLL 15",
    "SCROLL 16",
    "SCROLL 17",
    "SCROLL 18",
    "SCROLL 19",

    "AMULET 1",             /* 140 */
    "AMULET 2",
    "AMULET 3",
    "AMULET 4",
    "AMULET 5",
    "AMULET 6",
    "AMULET 7",
    "AMULET 8",

    "FLMN WAND",
    "ORB",
    "QUARTZ",               /* 150 */
    "CRYSTAL",
    "KEY",
    "BRONZE",
    "COPPER",
    "SAPPHIRE",
    "JADE",
    "SILVER",
    "EMERALD",
    "RING",
    "RUBY",                 /* 160 */
    "GOLD",
    "NECKLACE",
    "BRASS",
    "PAINTING",
    "GOLD CUP",
    "BRACELET",
    "STATUETTE",
    "CROWN",
    "EAR RINGS",
    "GOLD BALL",            /* 170 */
    "GOLD KEY",
    "PLATINUM",
    "STATUE",
    "GOLD RING",
    "DIAMOND",
    "GOLD BOOK",
    "SILVER OX",
    "GOLD FORK",
    "GOD RING"
};

char *Origin_Land[] =
{
    "GELNOR",
    "FERRONRAH",
    "NETHERWORLD"
};

char *Adventurer[] =
{
    "A NOVICE ADVENTURER",
    "AN EXPERIENCED ADVENTURER",
    "NOT YET READY TO DESTROY THE ORB",
    "READY TO DESTROY THE ORB",
    "IN NEED OF THE ORB",
    "THE HERO OF FERRONRAH"
};

char *Adventurers[] =
{
    "NOVICE ADVENTURERS",
    "EXPERIENCED ADVENTURERS",
    "NOT YET READY TO DESTROY THE ORB",
    "READY TO DESTROY THE ORB",
    "IN NEED OF THE ORB",
    "HEROES OF FERRONRAH"
};

/* Looks like these Runes were used in Phantasie I only */
char *Runes_List[] =
{
    "AIR RUNE",
    "EARTH RUNE",
    "FIRE RUNE",
    "WATER RUNE",
    "GOD RUNE"
};

SPELL Spells_List[] =
{
    { "HEALING 1",        EVERYWHERE },
    { "HEALING 2",        EVERYWHERE },
    { "HEALING 3",        EVERYWHERE },
    { "HEALING 4",        EVERYWHERE },

    { "FIREFLASH 1",      COMBAT_ACTIVE },
    { "FIREFLASH 2",      COMBAT_ACTIVE },
    { "FIREFLASH 3",      COMBAT_ACTIVE },
    { "FIREFLASH 4",      COMBAT_ACTIVE },

    { "QUICKNESS 1",      COMBAT_PASSIVE },
    { "QUICKNESS 2",      COMBAT_PASSIVE },
    { "QUICKNESS 3",      COMBAT_PASSIVE },
    { "QUICKNESS 4",      COMBAT_PASSIVE },

    { "STRENGTH 1",       COMBAT_PASSIVE },
    { "STRENGTH 2",       COMBAT_PASSIVE },
    { "STRENGTH 3",       COMBAT_PASSIVE },
    { "STRENGTH 4",       COMBAT_PASSIVE },

    { "PROTECTION 1",     COMBAT_PASSIVE },
    { "PROTECTION 2",     COMBAT_PASSIVE },
    { "PROTECTION 3",     COMBAT_PASSIVE },
    { "PROTECTION 4",     COMBAT_PASSIVE },

    { "CONFUSION 1",      COMBAT_ACTIVE },
    { "CONFUSION 2",      COMBAT_ACTIVE },
    { "CONFUSION 3",      COMBAT_ACTIVE },
    { "CONFUSION 4",      COMBAT_ACTIVE },

    { "WEAKNESS 1",       COMBAT_ACTIVE },
    { "WEAKNESS 2",       COMBAT_ACTIVE },
    { "WEAKNESS 3",       COMBAT_ACTIVE },
    { "WEAKNESS 4",       COMBAT_ACTIVE },

    { "BINDING 1",        COMBAT_ACTIVE },
    { "BINDING 2",        COMBAT_ACTIVE },
    { "BINDING 3",        COMBAT_ACTIVE },
    { "BINDING 4",        COMBAT_ACTIVE },

    { "MINDBLAST 1",      COMBAT_ACTIVE },
    { "MINDBLAST 2",      COMBAT_ACTIVE },
    { "MINDBLAST 3",      COMBAT_ACTIVE },
    { "MINDBLAST 4",      COMBAT_ACTIVE },

    { "FLAMEBOLT 1",      COMBAT_ACTIVE },
    { "FLAMEBOLT 2",      COMBAT_ACTIVE },
    { "FLAMEBOLT 3",      COMBAT_ACTIVE },
    { "FLAMEBOLT 4",      COMBAT_ACTIVE },

    { "CHARM",            COMBAT_ACTIVE },
    { "SLEEP",            COMBAT_ACTIVE },
    { "TELEPORTATION",    COMBAT_PASSIVE },
    { "RESURRECTION",     EVERYWHERE },
    { "BERSERK 2",        COMBAT_PASSIVE },
    { "FEAR",             COMBAT_ACTIVE },
    { "DISSOLVE",         COMBAT_ACTIVE },
    { "SUMMON ELEMENTAL", COMBAT_PASSIVE },
    { "DISPELL UNDEAD",   COMBAT_ACTIVE },
    { "BERSERK 1",        COMBAT_PASSIVE },
    { "AWAKEN",           COMBAT_PASSIVE },
    { "EVAL. MONSTER",    COMBAT_PASSIVE },
    { "VISION",           WILDERNESS_DUNGEON },
    { "TRANSPORTATION",   TOWN_ONLY },
    { "UNUSED 1",         0 },
    { "UNUSED 2",         0 },
    { "DIVINE II",        ASTRAL_ONLY }, 
    { "DIVINE I",         DUNGEON_ONLY }
};

short AttrClass_Table[] =
{
    0x08,0x0a,0x0c,0x0a,0x09,
    0x0a,0x0a,0x0b,0x0a,0x0a,
    0x0a,0x0b,0x0a,0x0a,0x0b,
    0x0c,0x08,0x0b,0x0a,0x09,
    0x0a,0x0c,0x0a,0x0a,0x0a,
    0x06,0x0d,0x08,0x0a,0x0b
};

short AttrRace_Table[] =
{
    0x0a,0x0a,0x0a,0x0a,0x0a,
    0x0c,0x09,0x09,0x0b,0x09,
    0x09,0x0b,0x0b,0x09,0x0a,
    0x08,0x0a,0x0c,0x0a,0x09,
    0x0b,0x09,0x0a,0x0b,0x09,
    0x08,0x0a,0x0d,0x08,0x09
};

short AttrMagic_Table[] =
{
    0x06,0x06,0x07,0x07,0x08,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0d,0x0e,0x0e,0x0f,
    0x0a,0x0a,0x0a,0x0b,0x0b,0x0b,0x0c,0x0c,0x0c,0x0d,0x0d,0x0e,0x10,0x12,0x14,
    0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,0x10,0x11,0x12,0x12,0x12,0x12,0x13,
    0x07,0x07,0x08,0x08,0x09,0x09,0x0a,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,0x10,0x12,
    0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,0x10,0x11,0x12,0x13,0x14,0x14,0x14,0x14,0x14,
    0x0a,0x0b,0x0b,0x0c,0x0d,0x0e,0x0e,0x0f,0x10,0x10,0x11,0x11,0x12,0x13,0x14
};

short AttrSkills_Table[] =
{
    0x46,0x32,0x25,0x3c,0x3c,0x1f,0x00,0x33,0x00,0x2a,
    0x4b,0x32,0x25,0x3c,0x36,0x26,0x00,0x35,0x00,0x30,
    0x50,0x37,0x29,0x3c,0x36,0x2d,0x00,0x37,0x00,0x37,
    0x55,0x37,0x29,0x3c,0x36,0x33,0x00,0x38,0x00,0x3e,
    0x5a,0x3c,0x2d,0x41,0x36,0x3a,0x00,0x3a,0x00,0x44,
    0x5f,0x3c,0x2d,0x41,0x36,0x41,0x00,0x3c,0x00,0x4b,
    0x64,0x41,0x31,0x41,0x36,0x47,0x00,0x3e,0x00,0x52,
    0x69,0x41,0x31,0x41,0x36,0x4e,0x00,0x40,0x00,0x59,
    0x6e,0x41,0x35,0x46,0x36,0x55,0x00,0x41,0x00,0x59,
    0x73,0x41,0x35,0x46,0x39,0x5b,0x00,0x43,0x00,0x59,
    0x78,0x41,0x39,0x46,0x39,0x62,0x00,0x45,0x00,0x59,
    0x7d,0x41,0x3d,0x46,0x39,0x69,0x00,0x47,0x00,0x59,
    0x82,0x41,0x41,0x4b,0x39,0x6f,0x00,0x48,0x00,0x59,
    0x87,0x41,0x45,0x4b,0x39,0x76,0x00,0x4a,0x00,0x59,
    0x8c,0x41,0x4d,0x4b,0x3c,0x8a,0x00,0x4c,0x00,0x59,
    
    0x50,0x3c,0x0d,0x23,0x2f,0x1f,0x00,0x2c,0x00,0x2a,
    0x55,0x3c,0x0d,0x23,0x28,0x1f,0x00,0x2c,0x00,0x2f,
    0x5f,0x41,0x11,0x23,0x25,0x23,0x00,0x2c,0x00,0x35,
    0x64,0x41,0x11,0x23,0x22,0x23,0x00,0x2c,0x00,0x3b,
    0x69,0x46,0x15,0x23,0x22,0x26,0x00,0x2c,0x00,0x41,
    0x6e,0x46,0x15,0x23,0x1f,0x26,0x00,0x2c,0x00,0x47,
    0x78,0x4b,0x19,0x23,0x1b,0x29,0x00,0x2c,0x00,0x4d,
    0x7d,0x4b,0x19,0x23,0x1b,0x2d,0x00,0x2c,0x00,0x54,
    0x82,0x50,0x1d,0x46,0x18,0x2d,0x00,0x2c,0x00,0x54,
    0x8c,0x55,0x1d,0x46,0x18,0x30,0x00,0x2c,0x00,0x54,
    0x91,0x5a,0x21,0x46,0x15,0x30,0x00,0x2c,0x00,0x54,
    0x96,0x5f,0x21,0x46,0x0e,0x33,0x00,0x2c,0x00,0x54,
    0x9b,0x64,0x25,0x5f,0x08,0x33,0x00,0x2c,0x00,0x54,
    0xa0,0x69,0x25,0x5f,0x01,0x37,0x00,0x2c,0x00,0x54,
    0xaa,0x6e,0x29,0x5f,0x04,0x37,0x00,0x2c,0x00,0x54,
    
    0x5a,0x3c,0x08,0x05,0x08,0x00,0x00,0x06,0x00,0x2a,
    0x5f,0x3c,0x08,0x05,0x08,0x04,0x00,0x06,0x00,0x32,
    0x64,0x3c,0x08,0x05,0x08,0x07,0x00,0x06,0x00,0x3a,
    0x73,0x3c,0x08,0x05,0x08,0x0a,0x00,0x06,0x00,0x43,
    0x7d,0x41,0x08,0x05,0x08,0x0e,0x00,0x06,0x00,0x4b,
    0x87,0x46,0x0c,0x05,0x0b,0x11,0x00,0x06,0x00,0x54,
    0x91,0x4b,0x0c,0x05,0x0b,0x14,0x00,0x06,0x00,0x54,
    0xa0,0x50,0x0c,0x05,0x0b,0x18,0x00,0x06,0x00,0x54,
    0xaa,0x55,0x0c,0x05,0x0b,0x1b,0x00,0x06,0x00,0x54,
    0xb4,0x55,0x0c,0x05,0x0b,0x1e,0x00,0x06,0x00,0x54,
    0xbe,0x55,0x10,0x05,0x0e,0x22,0x00,0x06,0x00,0x54,
    0xc8,0x55,0x18,0x05,0x0e,0x25,0x00,0x06,0x00,0x54,
    0xd7,0x55,0x20,0x05,0x0e,0x28,0x00,0x06,0x00,0x54,
    0xeb,0x55,0x28,0x05,0x0e,0x2c,0x00,0x06,0x00,0x54,
    0xfa,0x55,0x30,0x41,0x2f,0x90,0x00,0x0d,0x00,0x54,
    
    0x64,0x32,0x00,0x05,0x01,0x00,0x00,0x02,0x00,0x2a,
    0x6e,0x37,0x00,0x05,0x01,0x00,0x00,0x02,0x00,0x32,
    0x78,0x37,0x00,0x05,0x01,0x00,0x00,0x02,0x00,0x3a,
    0x87,0x37,0x00,0x05,0x01,0x04,0x00,0x02,0x00,0x43,
    0x91,0x3c,0x04,0x0a,0x01,0x04,0x00,0x02,0x00,0x4b,
    0x9b,0x3c,0x04,0x0a,0x01,0x04,0x00,0x04,0x00,0x54,
    0xa5,0x3c,0x04,0x0a,0x01,0x07,0x00,0x04,0x00,0x54,
    0xb4,0x41,0x04,0x0a,0x01,0x07,0x00,0x04,0x00,0x54,
    0xbe,0x41,0x08,0x0f,0x01,0x07,0x00,0x04,0x00,0x54,
    0xc8,0x41,0x08,0x0f,0x01,0x0a,0x00,0x04,0x00,0x54,
    0xd2,0x46,0x08,0x0f,0x01,0x0a,0x00,0x06,0x00,0x54,
    0xdc,0x46,0x08,0x0f,0x01,0x0a,0x00,0x06,0x00,0x54,
    0xeb,0x46,0x0c,0x14,0x01,0x0e,0x00,0x06,0x00,0x54,
    0xf5,0x46,0x0c,0x14,0x01,0x0e,0x00,0x06,0x00,0x54,
    0xff,0x46,0x18,0x14,0x01,0x0e,0x00,0x06,0x00,0x54,
    
    0x50,0x32,0x00,0x05,0x08,0x00,0x00,0x04,0x00,0x2a,
    0x5a,0x32,0x00,0x05,0x08,0x00,0x00,0x04,0x00,0x2e,
    0x64,0x32,0x00,0x05,0x08,0x04,0x00,0x04,0x00,0x32,
    0x6e,0x32,0x04,0x05,0x08,0x04,0x00,0x04,0x00,0x36,
    0x78,0x37,0x04,0x0a,0x08,0x07,0x00,0x04,0x00,0x3a,
    0x7d,0x37,0x04,0x0a,0x08,0x07,0x00,0x05,0x00,0x3e,
    0x87,0x37,0x08,0x0a,0x08,0x0a,0x00,0x05,0x00,0x43,
    0x91,0x37,0x08,0x0a,0x08,0x0a,0x00,0x05,0x00,0x47,
    0x9b,0x37,0x08,0x0a,0x08,0x0e,0x00,0x05,0x00,0x4b,
    0xa5,0x3c,0x0c,0x0f,0x08,0x0e,0x00,0x05,0x00,0x4f,
    0xaf,0x3c,0x0c,0x0f,0x08,0x11,0x00,0x06,0x00,0x4f,
    0xb4,0x3c,0x0c,0x0f,0x08,0x11,0x00,0x06,0x00,0x4f,
    0xbe,0x3c,0x10,0x0f,0x08,0x14,0x00,0x06,0x00,0x4f,
    0xc8,0x3c,0x10,0x14,0x08,0x14,0x00,0x06,0x00,0x4f,
    0xd2,0x3c,0x18,0x14,0x08,0x18,0x00,0x06,0x00,0x4f,
    
    0x46,0x28,0x00,0x05,0x01,0x00,0x00,0x00,0x00,0x2a,
    0x46,0x2d,0x00,0x05,0x01,0x00,0x00,0x00,0x00,0x2d,
    0x4b,0x32,0x00,0x05,0x01,0x00,0x00,0x00,0x00,0x30,
    0x4b,0x37,0x00,0x05,0x01,0x00,0x00,0x00,0x00,0x34,
    0x50,0x3c,0x00,0x05,0x01,0x04,0x00,0x00,0x00,0x37,
    0x50,0x3c,0x04,0x0a,0x01,0x04,0x00,0x03,0x00,0x3a,
    0x55,0x3c,0x04,0x0a,0x01,0x04,0x00,0x03,0x00,0x3e,
    0x55,0x3c,0x04,0x0a,0x01,0x04,0x00,0x03,0x00,0x41,
    0x5a,0x3c,0x08,0x0a,0x01,0x07,0x00,0x03,0x00,0x44,
    0x5a,0x3c,0x08,0x0a,0x01,0x07,0x00,0x03,0x00,0x48,
    0x5f,0x3c,0x0c,0x0f,0x01,0x07,0x00,0x06,0x00,0x4b,
    0x5f,0x3c,0x0c,0x0f,0x01,0x07,0x00,0x06,0x00,0x4e,
    0x64,0x3c,0x08,0x0f,0x01,0x07,0x00,0x06,0x00,0x4e,
    0x64,0x3c,0x14,0x0f,0x01,0x0a,0x00,0x06,0x00,0x4e,
    0x7d,0x3c,0x18,0x0f,0x01,0x0e,0x00,0x06,0x00,0x4e
};

short AttrHitPoints_Table[] =
{
    0x03,0x05,0x09,0x0b,0x10,0x14,0x18,0x1b,0x1e,0x23,0x27,0x2b,0x2f,0x33,0x37,
    0x04,0x05,0x09,0x0b,0x10,0x16,0x1b,0x1f,0x23,0x28,0x2c,0x30,0x34,0x38,0x3c,
    0x04,0x09,0x0e,0x14,0x19,0x1e,0x24,0x2a,0x30,0x37,0x3c,0x41,0x46,0x4b,0x50,
    0x05,0x0a,0x10,0x16,0x1c,0x23,0x29,0x2f,0x35,0x3c,0x41,0x46,0x4b,0x50,0x55,
    0x04,0x06,0x0a,0x0f,0x14,0x19,0x1e,0x23,0x28,0x2d,0x31,0x35,0x39,0x3d,0x41,
    0x03,0x04,0x06,0x08,0x0b,0x0e,0x11,0x13,0x16,0x19,0x1c,0x1f,0x22,0x25,0x28
};

short AttrLearnableSpells_Table[] =
{
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0d,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0a,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x04,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

    0x00,0x00,0x00,0x00,0x01,0x03,0x06,0x08,0x0a,0x0e,
    0x00,0x00,0x07,0x0c,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x09,0x0b,0x0d,0x0f,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x04,0x00,0x00,0x07,0x00,0x02,
    0x00,0x00,0x05,0x03,0x00,0x00,0x00,0x00,0x00,0x00,

    0x01,0x04,0x06,0x08,0x01,0x09,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x07,0x00,0x00,
    0x09,0x0c,0x0f,0x00,0x03,0x0b,0x00,0x00,0x05,0x0a,
    0x00,0x00,0x0d,0x00,0x00,0x00,0x0e,0x00,0x00,0x00,
    0x00,0x0b,0x00,0x0d,0x00,0x00,0x00,0x00,0x03,0x00,
    0x07,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,
    0x0c,0x00,0x09,0x00,0x0f,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

    0x01,0x03,0x05,0x08,0x00,0x00,0x09,0x0a,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x06,0x09,0x0e,
    0x02,0x05,0x07,0x0c,0x04,0x07,0x0c,0x0e,0x02,0x06,
    0x0b,0x0d,0x00,0x0b,0x00,0x00,0x0a,0x00,0x0d,0x00,
    0x00,0x04,0x08,0x09,0x00,0x00,0x00,0x0f,0x03,0x00,
    0x01,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

    0x00,0x00,0x00,0x00,0x01,0x03,0x04,0x07,0x01,0x03,
    0x09,0x0d,0x02,0x04,0x0b,0x0e,0x00,0x00,0x00,0x00,
    0x05,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x06,0x08,0x0a,0x0c,0x07,0x09,0x0b,0x0e,
    0x02,0x00,0x00,0x00,0x08,0x07,0x0f,0x0c,0x00,0x06,
    0x08,0x00,0x00,0x05,0x00,0x00,0x00,0x00,0x00,0x00
};

LPMENU Menu_Sheet_Character;
LPMENU Menu_Age_Notification;
BOOL Character_Sheet_Page;
LPMENU Sel_Town_Menu;

/* --------------------------------
   Pick a random race
   -------------------------------- */
int Create_Random_Race()
{
    return PIXIE + int(rnd() * 10);
}

/* --------------------------------
   Calculate a base characteristic value of a character
   -------------------------------- */
int Calc_Character_Base_Attribute(LPCHARACTER Character, int Idx)
{
    int Value;

    Value = int(pow(rnd(), 0.6f) * 6.0f) + int(pow(rnd(), 0.6f) * 6.0f) + int(pow(rnd(), 0.6f) * 6.0f) + 3;
    Value += AttrClass_Table[(Character->Class * 5) + Idx] - 10;
    if(Value > 18) Value = 18;
    Value += AttrRace_Table[(Character->Race * 5) + Idx] - 10;
    if(Value < 3) Value = 3;
    return Value;
}

/* --------------------------------
   Calculate the skill of a character
   -------------------------------- */
int Calc_Character_Skill(LPCHARACTER Character, int Skill)
{
    float Value;

    Value = (Character->Skills[Skill] +
             AttrSkills_Table[(Character->Class * 150) + (Character->Level * 10) + Skill]
            ) / 2 + Character->Skills_Default[Skill] - 49.5f;
    if(Value < 1.0f) Value = 1.0f;
    if(Value > 255.0f) Value = 255.0f;

    return (int) Value;
}

/* --------------------------------
   Calculate modifiable character attributes
   -------------------------------- */
void Calc_Character_Modifiable_Attributes(LPCHARACTER Character, int Level)
{
    int i;
    int Max_Points;
    int Value;

    if(Character->Level != MAX_LEVELS)
    {
        Character->Skills_Default[ATTACK] = (2 * Character->Strength) + 30;
        Character->Skills_Default[PARRY] = (2 * Character->Dexterity) + 30;
        Character->Skills_Default[FIND_ITEM]= (2 * Character->Intelligence) + 30;
        Character->Skills_Default[SPOT_TRAP] = (2 * Character->Intelligence) + 30;
        Character->Skills_Default[DISARM_TRAP] = (2 * Character->Dexterity) + 30;
        Character->Skills_Default[LISTEN] = (2 * Character->Intelligence) + 30;
        Character->Skills_Default[PICK_LOCK] = (2 * Character->Dexterity) + 30;
        Character->Skills_Default[SWIM] = (2 * Character->Dexterity) + 30;
        Character->Skills_Default[TOSS_ROCK] = (Character->Level * 2) +
                                               15 * (Character->Race == DWARF) +
                                               30 * (Character->Race == GNOME) +
                                               Character->Dexterity * 4 +
                                               Character->Strength + 10;

        /* Maximum magic points */
        Character->Max_Class_Magic = AttrMagic_Table[Character->Class * 15 + Character->Level];

        /* Calculate skills */
        Character->Skills[ATTACK] = Calc_Character_Skill(Character, ATTACK);
        Character->Skills[PARRY] = Calc_Character_Skill(Character, PARRY);
        Character->Skills[FIND_ITEM] = Calc_Character_Skill(Character, FIND_ITEM);
        Character->Skills[SPOT_TRAP] = Calc_Character_Skill(Character, SPOT_TRAP);
        Character->Skills[DISARM_TRAP] = Calc_Character_Skill(Character, DISARM_TRAP);
        Character->Skills[LISTEN] = Calc_Character_Skill(Character, LISTEN);
        Character->Skills[TOSS_ROCK] = Calc_Character_Skill(Character, TOSS_ROCK);
        Character->Skills[PICK_LOCK] = Calc_Character_Skill(Character, PICK_LOCK);
        Character->Skills[SWIM] = Calc_Character_Skill(Character, SWIM);

        /* Calculate maximum hit points */
        Max_Points = AttrHitPoints_Table[(Character->Class * MAX_LEVELS) + Character->Level] * (Character->Constitution + 5) / 16;
        Value  = 0;
        for(i = 1; i < Max_Points; i++)
        {
            Value += int(rnd() * 3.0f + 1.0f);
        }
        if(Value <= Character->Max_Hit_Points) Value = Character->Max_Hit_Points + 1;
        if(Value > 255) Value = 255;
        Character->Max_Hit_Points = Value;

        /* Set the learnable spells (Note: last 2 spells aren't learnable nor used) */
        for(i = 0; i < (MAX_SPELLS - 1); i++)
        {
            if(AttrLearnableSpells_Table[(Character->Class * MAX_SPELLS) + i] == (Character->Level + 1))
            {
                /* Make it learnable */
                Character->Spells[i] = LEARNABLE;
            }
        }

        Character->Level = Level + 1;
    }
}

/* --------------------------------
   Reset and calculate a character points values
   -------------------------------- */
void Reset_Character_Shield_Armor_Weapon_Points(LPCHARACTER Character)
{
    Character->Shield_Point_Value = 0;
    Character->Armor_Point_Value = 0;
    Character->Weapon_Point_Value = 0;
    Calc_Character_Shield_Armor_Weapon_Points(Character, MAX_CHARACTER_ITEMS);
}

/* --------------------------------
   Calculate a character points values
   -------------------------------- */
void Calc_Character_Shield_Armor_Weapon_Points(LPCHARACTER Character, int Max_Items)
{
    int i;
    int Str;
    int Str_Dex;
    int Item;

    Str = Character->Strength;
    Str_Dex = 2 * Str + Character->Dexterity;

    for(i = 0; i < Max_Items; i++)
    {
        Item = Character->Items[i];
        if(Item)
        {
            if(Item <= 100)
            {
                if(Item < 21 &&
                   Item > Character->Shield_Point_Value &&
                   Str >= Item)
                {
                    Character->Shield_Point_Value = Item;
                    Character->Equipped_Shield = Item;
                    goto Done_Item;
                }

                if(Item < 31 &&
                   Str >= (2 * Item - 40) &&
                   Item - 20 > Character->Armor_Point_Value)
                {
                    Character->Armor_Point_Value = Item - 20;
                    Character->Equipped_Armor = Item;
                    goto Done_Item;
                }
                if(Item < 41 &&
                   Str >= (2 * Item - 64) &&
                   Item - 29 > Character->Armor_Point_Value)
                {
                    Character->Armor_Point_Value = Item - 29;
                    Character->Equipped_Armor = Item;
                    goto Done_Item;
                }

                if(Item < 71 &&
                   Str_Dex >= (2 * Item - 80) &&
                   (Item / 2 - 20) > Character->Weapon_Point_Value)
                {
                    Character->Weapon_Point_Value = (Item / 2 - 20);
                    Character->Equipped_Weapon = Item;
                    goto Done_Item;
                }
                if(Str_Dex >= (2 * Item - 140) &&
                   (0.75f * Item - 50) > Character->Weapon_Point_Value)
                {
                    Character->Weapon_Point_Value = (int) (0.75f * Item - 50);
                    Character->Equipped_Weapon = Item;
                }
            }
        }

Done_Item:;

    }
}

/* --------------------------------
   Determine the life span of a given character
   -------------------------------- */
void Calc_Character_Life_Span(LPCHARACTER Character)
{
    float Age;

    Age = (float) Character->Luck;

    if(Character->Race > SPRITE)
    {
        Age /= 2.5f;
    }
    if(Character->Race == DWARF ||
       Character->Race == HALFLING ||
       Character->Race == GNOME)
    {
        Age *= 2.5f;
    }
    if(Character->Race == ELF)
    {
        Age *= 19.0f;
    }
    Character->LifeSpan = (int) (Age + 0.5f);
}

/* --------------------------------
   Reset the data of the party
   -------------------------------- */
void Clear_Party_Data()
{
    memset(&Party, 0, sizeof(PARTY));
}

/* --------------------------------
   Aging notification loop
   -------------------------------- */
void Age_Menu_Events(void *Self, int Item, ULONG Pad_Dats)
{
    switch(Pad_Dats)
    {
        case PAD_OK:
        case PAD_CANCEL:

            Hide_Menu(Menu_Age_Notification, Pointer);
            Destroy_Menu(Menu_Age_Notification);
            Construct_Main_Town_Menu();
            Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
            break;
    }
}

/* --------------------------------
   Calculate the new age of the characters
   -------------------------------- */
BOOL Calc_Age()
{
    int i;
    LPCHARACTER Character;
    char Message[300];
    float Diminish;
    BOOL Show_Message = FALSE;

    memset(Message, 0, sizeof(Message));

    strcpy(Message, "AS TIME PASSES...\n ");

    for(i = 0; i < Party.Number_Characters; i++)
    {
        Diminish = 0.0f;
        Character = &Party.Characters[i];

        Character->Age += (Party.Days_Out / 7);

        if((Character->Age >> 8) < (0.7f * Character->LifeSpan))
        {
            goto Character_Not_Aged;
        }
        if((Character->Age >> 8) >= Character->LifeSpan)
        {
            goto Character_Is_Dead;
        }
        if(Character->Already_Aged)
        {
            goto Character_Not_Aged;
        }
        Diminish = 1.0f;
        Character->Already_Aged = TRUE;

Character_Is_Dead:

        /* Either 0 or 0.8 */
        Diminish *= 0.8f;

        strcat(Message, "\n");
        strcat(Message, Character->Name);
        if(Diminish == 0.0f)
        {
            strcat(Message, " HAS DIED");
            Character->Status = DEAD;
        }
        else
        {
            strcat(Message, " HAS AGED");
        }
        Show_Message = TRUE;

        /* Reduce character stats to 80% of their former values or 0 */
        /* Note: the original game reduced the luck too, i don't see why */
        Character->Strength = (int) (Diminish * Character->Strength);
        Character->Intelligence = (int) (Diminish * Character->Intelligence);
        Character->Dexterity = (int) (Diminish * Character->Dexterity);
        Character->Constitution = (int) (Diminish * Character->Constitution);
        Character->Charisma = (int) (Diminish * Character->Charisma);
        Character->Adroitness = (int) (Diminish * Character->Adroitness);

        Reset_Character_Shield_Armor_Weapon_Points(Character);

Character_Not_Aged:;

    }

    Party.Days_Out = 0;

    if(Show_Message)
    {
        Menu_Age_Notification = Construct_Menu(Font,
                                               Pic_Menus,
                                               &Age_Menu_Events,
                                               MENU_TEXT_CENTER | MENU_NO_POINTER,
                                               8, 0,
                                               1, 0,
                                               MOVE_NONE,
                                               Message
                                              );

        Show_Menu(Menu_Age_Notification, Pointer, -1, -1, 0);
    }
    return Show_Message;
}

/* --------------------------------
   Return the amount of experience needed before levelling up
   (< 0 = experience needed) (>= 0 extra experience points)
   -------------------------------- */
int Calc_Needed_Experience(LPCHARACTER Character)
{
    float Experience;
    float Needed_Experience;
    float Value;

    /* Calc the experience that'll be needed for levelling up */
    Experience = Character->Experience;

    Value = (Character->Class == THIEF) *
            (2 * Character->Dexterity +
             Character->Strength
            ) / 19.8f +
            (Character->Class == MONK) *
            (Character->Strength +
             Character->Intelligence +
             Character->Dexterity
            ) / 49.5f +
            (Character->Class == RANGER) *
            (Character->Strength +
             Character->Intelligence +
             Character->Charisma
            ) / 52.8f;

    Value = Value + (Character->Class == FIGHTER) *
                    (Character->Dexterity + 2 *
                     Character->Strength
                    ) / 33.0f +
                    (Character->Class == PRIEST) *
                    (Character->Strength + 2 *
                     Character->Intelligence
                    ) / 36.3f +
                    (Character->Class == WIZARD) *
                    (2 * Character->Intelligence +
                     Character->Luck
                    ) / 42.9f;

    if(Value == 0.0f)
    {
        Value = 0.00001f;
    }

    Needed_Experience = 1000.0f * (-0.8f + pow(1.8f, Character->Level)) / Value;

    Experience = Experience - Needed_Experience;
    return (int) Experience;
}

/* --------------------------------
   Level up a given character
   -------------------------------- */
void Level_Up_Character(LPCHARACTER Character)
{
    Calc_Character_Modifiable_Attributes(Character, Character->Level);
}

/* --------------------------------
   Select a town to teleport to
   -------------------------------- */
void Sel_Town_Events(void *Self, int Item, ULONG Pad_Dats)
{
    int Sel_Town;

    switch(Pad_Dats)
    {
        case PAD_OK:

            Sel_Town = (Item - 2) + 1;
            if(Sel_Town > 0)
            {
                /* Action is irreversible from here */
                Hide_Menu((LPMENU) Self, Pointer);
                Destroy_Menu(Sel_Town_Menu);

                /* Can't transport to certain locations */
                if((Cur_Town_Nbr < BLACK_MOOR && Sel_Town > FERRON) ||
                   ((Cur_Town_Nbr == BLACK_MOOR || Cur_Town_Nbr == DEATHPORT) && (Sel_Town < BLACK_MOOR || Sel_Town > DEATHPORT)) ||
                   (Cur_Town_Nbr > DEATHPORT && Sel_Town < DEMONIAC)
                  )
                {
                    Display_Message(Font, Pic_Menus, -1, "SPELL FAILED");
                    break;
                }
                Display_Message(Font, Pic_Menus, -1, "THE PARTY IS SUDDENLY ENGULFED IN BLACKNESS...");
                Set_Current_Town(Sel_Town);
                
                /* Return to the town picture */
                Cancel_Sheet = TRUE;
                Set_Fade(Black_Palette, 40, MOVE_NONE);
                break;
            }
            break;

        case PAD_CANCEL:

            /* No character selected */
            Hide_Menu((LPMENU) Self, Pointer);
            Destroy_Menu(Sel_Town_Menu);
            break;
    }
}

/* --------------------------------
   Cast a spell
   Note: Character is NULL in case of a party spell
         No need to verify spell cost here as they are
         not available if the character doesn't have enough magic points anyway
   -------------------------------- */
void Cast_Spell(LPCHARACTER Caster, LPCHARACTER Character, int Spell)
{
    int Spell_Cost;
    int i;
    char Names[256];
    int Hit_Points;

    Spell_Cost = Calc_Spell_Cost(Spell);
    Caster->Current_Magic -= Spell_Cost;

    /* Can't have negative magic points */
    if(Caster->Current_Magic < 0)
    {
        Caster->Current_Magic = 0;
    }

    Display_Magic_Points(Caster);
    Swap_Physical_Buffers();
    Display_Magic_Points(Caster);

    /* There's was problem here in the original game:
       healing_4 had the same level effect as healing_1 while costing more magic points */
    if(Spell <= HEALING_4)
    {
        /* Heal character */
        Hit_Points = (int) (pow(Spell - HEALING_1 - 1, 2.0f) +
                     (float) Character->Current_Hit_Points);

        /* Original method */
        /*Hit_Points = int(rnd() *
                     (4 + 6 * Character->Strength * Character->Strength)) +
                     Character->Current_Hit_Points;*/

        /* Can't exceed max hit points */
        if(Hit_Points > Character->Max_Hit_Points)
        {
            Hit_Points = Character->Max_Hit_Points;
        }
        Character->Current_Hit_Points = Hit_Points;

        /* Refresh the screen if needed */
        if(Character == Caster)
        {
            Display_Hit_Points(Character);
            Swap_Physical_Buffers();
            Display_Hit_Points(Character);
            Display_Message(Font, Pic_Menus, -1, "YOU ARE HEALED");
        }
        else
        {
            Display_Message(Font, Pic_Menus, -1, "%s IS HEALED", Character->Name);
        }
    }

    switch(Spell)
    {
        case RESURRECTION:

            /* Resurrection concerns the whole party. */
            memset(Names, 0, 256);
            for(i = 0; i < Party.Number_Characters; i++)
            {
                if(Party.Characters[i].Status == DEAD ||
                   Party.Characters[i].Constitution >= 1)
                {
                    if(rnd() * 100 <=
                       50 + (5 * Party.Characters[i].Level - Caster->Level))
                    {
                        Party.Characters[i].Status = OKAY;
                        Party.Characters[i].Current_Hit_Points = Party.Characters[i].Max_Hit_Points;
                        strcat(Names, Party.Characters[i].Name);
                        strcat(Names, " IS ALIVE\n");
                    }
                }
            }
            Display_Message(Font, Pic_Menus, -1, Names);
            break;

        case TRANSPORTATION:

            strcpy(Names, "TRANSPORT TO WHICH TOWN?\n ");

            // TODO: verify "Party.Number_Characters"
            for(i = 0; i < Party.Number_Characters; i++)
            {
                strcat(Names, "\n");
                strcat(Names, Towns[i].Name);
            }

            Sel_Town_Menu = Construct_Menu(Font,
                                           Pic_Menus,
                                           Sel_Town_Events,
                                           MENU_TEXT_CENTER,
                                           8, 2,
                                           1, 0,
                                           MOVE_VERTICALLY,
                                           Names
                                          );
            Show_Menu(Sel_Town_Menu, Pointer, -1, -1, 0);
            break;

        case DIVINE_I:

            if(Cur_Town != NULL)
            {
                Display_Message(Font, Pic_Menus, -1, "THAT SPELL WILL NOT WORK IN TOWN");
                break;
            }
            /* TODO: the spell (in dungeons) */
            break;

        case DIVINE_II:

            /* TODO: check plane */
            if(Cur_Town != NULL)
            {
                Display_Message(Font, Pic_Menus, -1, "THAT SPELL WILL NOT WORK IN THIS PLANE");
                break;
            }
            /* TODO: the spell (in astral plane) */
            break;
    }
}

/* --------------------------------
   Return the amount of magic point required for a given spell
   -------------------------------- */
int Calc_Spell_Cost(int Spell)
{
    return ((Spell + 1) - 4 * int(((Spell + 1) - 1) / 4));
}
