/* --------------------------------
   Phantasie II CD32
   Characters handling
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

#ifndef _CHARACTERS_H_
#define _CHARACTERS_H_

/* --------------------------------
   Includes
   -------------------------------- */
#include "support/startup.h"

/* --------------------------------
   Constants
   -------------------------------- */
#define MAX_CHARACTERS 7
#define MAX_CLASSES 6
#define MAX_RACES 16
#define MAX_CREATE_RACES 6
#define MAX_LEVELS 15
#define MAX_BEAST_RUNES 8
#define MAX_ITEMS 180

#define RACE_RANDOM 5

#define MAX_CHARACTER_NAME 13
#define MAX_CHARACTER_ITEMS 9

/* Note: AWAKEN is a passive spell not an active one as stated in the manual
         MONSTER EVALUATION is a passive combat spell
         PASSIVE spells aren't cumulative
         (they can be casted several times
          but the target will only beneficiate from them the 1st time)
*/

#define COMBAT_PASSIVE 1      /* Casted on characters */
#define COMBAT_ACTIVE 2       /* Casted on monsters */
#define WILDERNESS_DUNGEON 4

#define TOWN_ONLY 8
#define DUNGEON_ONLY 16
#define ASTRAL_ONLY 32

#define EVERYWHERE 64

/* Select from a list of name or from pictures
 */
enum SEL_TYPE
{
    SHEET,
    COMBAT
};

enum LOCATION
{
    IN_TOWN = 0,
    IN_WILDERNESS,
    IN_DUNGEON
};

enum RUNE
{
    NOT_OWNED = 0,
    DECRYPTED
};

enum ENCOUNTER
{
    DEFEAT = 0,
    TELEPORT,
    VICTORY
};

enum STATUS
{
    DEAD = 0,
    OKAY,
    SLEEPING
};

enum SPELL_LEARN
{
    UNKNOWN = 0,
    LEARNED,
    LEARNABLE
};

enum CLASSES
{
    THIEF = 0,
    MONK,
    RANGER,
    FIGHTER,
    PRIEST,
    WIZARD
};

enum RACES
{
    HUMAN = 0,
    DWARF,
    ELF,
    HALFLING,
    GNOME,

    /* Random creatures */
    PIXIE,
    SPRITE,
    OGRE,
    GNOLL,
    TROLL,
    KOBOLD,
    ORC,
    GOBLIN,
    LIZARDMAN,
    MINOTAUR,

    /* Specific */
    UNDEAD,
    ELEMENTAL
};

enum SPELLS_LABELS
{
    HEALING_1 = 0,
    HEALING_2,
    HEALING_3,
    HEALING_4,
    FIREFLASH_1,
    FIREFLASH_2,
    FIREFLASH_3,
    FIREFLASH_4,
    QUICKNESS_1,
    QUICKNESS_2,
    QUICKNESS_3,
    QUICKNESS_4,
    STRENGTH_1,
    STRENGTH_2,
    STRENGTH_3,
    STRENGTH_4,
    PROTECTION_1,
    PROTECTION_2,
    PROTECTION_3,
    PROTECTION_4,
    CONFUSION_1,
    CONFUSION_2,
    CONFUSION_3,
    CONFUSION_4,
    WEAKNESS_1,
    WEAKNESS_2,
    WEAKNESS_3,
    WEAKNESS_4,
    BINDING_1,
    BINDING_2,
    BINDING_3,
    BINDING_4,
    MINDBLAST_1,
    MINDBLAST_2,
    MINDBLAST_3,
    MINDBLAST_4,
    FLAMEBOLT_1,
    FLAMEBOLT_2,
    FLAMEBOLT_3,
    FLAMEBOLT_4,
    CHARM,
    SLEEP,
    TELEPORTATION,
    RESURRECTION,
    BERSERK_2,
    FEAR,
    DISSOLVE,
    SUMMON_ELEMENTAL,
    DISPELL_UNDEAD,
    BERSERK_1,
    AWAKEN,
    EVAL_MONSTER,
    VISION,
    TRANSPORTATION,
    UNUSED_1,
    UNUSED_2,
    DIVINE_II,
    DIVINE_I,        /* Only works in dungeons */
    MAX_SPELLS,      /* 58 spells */
};

enum SKILLS
{
    ATTACK = 0,
    PARRY,
    FIND_ITEM,
    SPOT_TRAP,
    DISARM_TRAP,
    LISTEN,
    TOSS_ROCK,
    PICK_LOCK,
    SWIM,
    MAX_SKILLS
};

enum ITEMS
{
    NONE = 0,
    GLOVE,
    WOOD_SHIELD,
    WOOD_SHIELD_1,
    SMALL_SHIELD,
    SMALL_SHIELD_1,
    SMALL_SHIELD_2,
    SMALL_SHIELD_3,
    MEDIUM_SHIELD,
    MEDIUM_SHIELD_1,
    MEDIUM_SHIELD_2,    /* 10 */
    MEDIUM_SHIELD_3,
    LARGE_SHIELD,
    LARGE_SHIELD_1,
    LARGE_SHIELD_2,
    LARGE_SHIELD_3,
    GIANT_SHIELD,
    GIANT_SHIELD_1,
    GIANT_SHIELD_2,
    GIANT_SHIELD_3,
    GOD_SHIELD,         /* 20 */

    CLOTHING,
    ROBES,
    LEATHER,
    HARD_LEATHER,
    RING_MAIL,
    SCALE_MAIL,
    CHAIN_MAIL,
    SPLINT_MAIL,
    BANDED_MAIL,
    PLATE_MAIL,         /* 30 */
    CLOTH_1,
    ROBES_1,
    LEATHER_1,
    LEATHER_2,
    RING_MAIL_1,
    RING_MAIL_2,
    CHAIN_MAIL_1,
    CHAIN_MAIL_2,
    GOD_ROBES,
    GOD_ARMOR,          /* 40 */

    STICK,
    KNIFE,
    SMALL_CLUB,
    SMALL_STAFF,
    SMALL_MACE,
    DAGGER,
    SMALL_FLAIL,
    CLUB,
    MACE,
    SMALL_HAMMER,       /* 50 */
    SMALL_AXE,
    STAFF,
    SHORT_SWORD,
    FLAIL,
    HAMMER,
    PTCH_FORK,
    SPEAR,
    AXE,
    SWORD,
    HEAVY_MACE,
    MAUL,               /* 60 */
    TRIDENT,
    LARGE_SPEAR,
    LARGE_AXE,
    MORNING_STAR,
    PIKE,
    LONG_SWORD,
    SPETUM,
    BARDICHE,
    HALBERD,            /* 70 */
    SMALL_MACE_1,
    DAGGER_1,
    SMALL_MACE_2,
    DAGGER_2,
    DAGGER_3,
    STAFF_1,
    DAGGER_4,
    FLAIL_1,
    SPEAR_1,
    AXE_1,              /* 80 */
    SWORD_1,
    SWORD_2,
    SWORD_3,
    LARGE_AXE_1,
    SWORD_4,
    SWORD_5,
    SWORD_6,
    HALBERD_1,
    SWORD_7,
    HALBERD_2,          /* 90 */
    HALBERD_3,
    SWORD_10,
    HALBERD_4,
    HALBERD_5,
    HALBERD_6,
    HALBERD_7,
    GOD_KNIFE,
    GOD_MACE,
    GOD_AXE,
    GOD_SWORD,          /* 100 */

    HEAL_POTION_1,
    HEAL_POTION_2,
    HEAL_POTION_3,
    HEAL_POTION_4,
    HEAL_POTION_5,
    HEAL_POTION_6,
    HEAL_POTION_7,
    HEAL_POTION_8,
    HEAL_POTION_9,
    HEAL_POTION_10,     /* 110 */
    MAGIC_POTION_1,
    MAGIC_POTION_2,
    MAGIC_POTION_3,
    MAGIC_POTION_4,
    MAGIC_POTION_5,
    MAGIC_POTION_6,
    MAGIC_POTION_7,
    MAGIC_POTION_8,
    MAGIC_POTION_9,
    MAGIC_POTION_10,    /* 120 */

    SCROLL_0,           /* 121 */
    SCROLL_1,
    SCROLL_2,
    SCROLL_3,
    SCROLL_4,
    SCROLL_5,
    SCROLL_6,
    SCROLL_7,
    SCROLL_8,
    SCROLL_9,           /* 130 */
    SCROLL_I0,
    SCROLL_11,
    SCROLL_12,
    SCROLL_13,
    SCROLL_14,
    SCROLL_15,
    SCROLL_16,
    SCROLL_17,
    SCROLL_18,
    SCROLL_19,          /* 140 */

    AMULET_1,
    AMULET_2,
    AMULET_3,
    AMULET_4,
    AMULET_5,
    AMULET_6,
    AMULET_7,
    AMULET_8,

    FLMN_WAND,          /* 149 */
    ORB,                /* 150 */

    /* Considered as misc. treasures */
    QUARTZ,
    CRYSTAL,
    KEY,
    BRONZE,
    COPPER,
    SAPPHIRE,
    JADE,
    SILVER,
    EMERALD,
    RING,               /* 160 */
    RUBY,
    GOLD,
    NECKLACE,
    BRASS,
    PAINTING,
    GOLD_CUP,
    BRACELET,
    STATUETTE,
    CROWN,
    EAR_RINGS,          /* 170 */
    GOLD_BALL,
    GOLD_KEY,
    PLATINUM,
    STATUE,
    GOLD_RING,
    DIAMOND,
    GOLD_BOOK,
    SILVER_OX,
    GOLD_FORK,
    GOD_RING
};

/* --------------------------------
   Structures
   -------------------------------- */
typedef struct
{
    BYTE Strength;
    BYTE Intelligence;
    BYTE Dexterity;
    BYTE Constitution;
    BYTE Charisma;

    BYTE Luck;
    BYTE Adroitness;
    BYTE Score;
    BYTE Max_Class_Magic;
    BYTE Max_Magic;
    BYTE Current_Magic;
    BYTE Max_Hit_Points;
    BYTE Current_Hit_Points;
    int Gold;
    int Age;
    BYTE Status;

    BYTE Skill_Weapon;
    BYTE Skills[MAX_SKILLS];
    BYTE Skills_Default[MAX_SKILLS];

    BYTE Origin;
    char Wand_Of_Nikademus;

    BYTE Equipped_Shield;
    BYTE Equipped_Armor;
    BYTE Equipped_Weapon;

    int Experience;
    BYTE Level;

    BYTE Temp_Armor_Point_Value;
    BYTE Temp_Weapon_Point_Value;
    BYTE Temp_Skill_Attack;

    BYTE Available;
    BYTE Used_Magic;

    BYTE Spells[MAX_SPELLS];

    char Name[MAX_CHARACTER_NAME];

    BYTE Class;

    char Items[MAX_CHARACTER_ITEMS];

    BYTE Race;

    BYTE Weapon_Point_Value;
    BYTE Shield_Point_Value;
    BYTE Armor_Point_Value;
    BYTE Current_Town;

    int LifeSpan;
    BYTE Already_Aged;

    BYTE Picture;
} CHARACTER, *LPCHARACTER;

/* (There's a slot reserved for summoning an elemental during combats) */
typedef struct
{
    int Number_Characters;
    int Gold;
    int Earned_XP;
    int Days_Out;
    short Level;
    short Score;
    CHARACTER Characters[MAX_CHARACTERS];
    /* Amulets converted into RUNES */
    BYTE Beast_Runes[MAX_BEAST_RUNES];
} PARTY, *LPPARTY;

typedef struct
{
    char *Name;
} ITEM, *LPITEM;

typedef struct
{
    char *Name;
    int Type;
} SPELL, *LPSPELL;

/* --------------------------------
   Variables
   -------------------------------- */
extern char *Classes_List[];
extern char *Races_List[];
extern char *Races_Create_List[];
extern char *Origin_Land[];
extern char *Adventurer[];
extern char *Adventurers[];
extern char *Status_List[];
extern char *Runes_List[];
extern SPELL Spells_List[];
extern char *Beast_Runes_List[];

extern ITEM Items_List[MAX_ITEMS];

extern PARTY Party;

extern LPMENU Menu_Sheet_Character;
extern LPMENU Menu_Age_Notification;
extern LPCHARACTER Cur_Character;

extern BOOL Character_Sheet_Page;

/* --------------------------------
   Functions
   -------------------------------- */
int Create_Random_Race();
int Calc_Character_Base_Attribute(LPCHARACTER Character, int Idx);
void Calc_Character_Modifiable_Attributes(LPCHARACTER Character, int Level);
void Reset_Character_Shield_Armor_Weapon_Points(LPCHARACTER Character);
void Calc_Character_Shield_Armor_Weapon_Points(LPCHARACTER Character, int Max_Items);
void Calc_Character_Life_Span(LPCHARACTER Character);
void Clear_Party_Data();
BOOL Calc_Age();
void Construct_Character_Sheet_Menu(LPCHARACTER Character, BOOL In_Town);
void Display_Character_Stats(LPCHARACTER Character, BOOL Page2, BOOL Copy);
void Train_Character(LPCHARACTER Character);
int Calc_Needed_Experience(LPCHARACTER Character);
void Level_Up_Character(LPCHARACTER Character);
void Learn_Spell(LPCHARACTER Character);
void Use_Item(LPCHARACTER Owner, LPCHARACTER Character, int Item);
void Cast_Spell(LPCHARACTER Caster, LPCHARACTER Character, int Spell);
int Calc_Spell_Cost(int Spell);

#endif
