/* --------------------------------
   Phantasie II CD32
   Characters sheets
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   
       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* TODO: - Buy  <<< Armory
         - Sell <<< Armory
         - Give
         - Bury
*/

/* --------------------------------
   Includes
   -------------------------------- */
#include "main.h"
#include "characters.h"
#include "characters_sheets.h"
#include "towns.h"
#include "scrolls.h"

/* --------------------------------
   Constants
   -------------------------------- */
#define MENU_CHARACTER_DEAD 0
#define MENU_CHARACTER_TOWN 1
#define MENU_CHARACTER_WILDERNESS 2

#define MENU_CHARACTER_CHANGE_PAGE 0
#define MENU_CHARACTER_DEAD_TAKE_ITEMS 2
#define MENU_CHARACTER_DEAD_BURY 3

#define MENU_CHARACTER_TOWN_LEARN_SPELL 2
#define MENU_CHARACTER_TOWN_CAST_SPELL 3
#define MENU_CHARACTER_TOWN_BUY_ITEMS 4
#define MENU_CHARACTER_TOWN_SELL_ITEMS 5
#define MENU_CHARACTER_TOWN_GIVE_ITEMS 6
#define MENU_CHARACTER_TOWN_USE_ITEM 7

#define MENU_CHARACTER_WILDERNESS_CAST_SPELL 2
#define MENU_CHARACTER_WILDERNESS_GIVE_ITEMS 3
#define MENU_CHARACTER_WILDERNESS_USE_ITEM 4

/* --------------------------------
   Variables
   -------------------------------- */
BYTE Menu_Type;
int Current_Party_Location;
LPCHARACTER Cur_Character;
int Cur_Character_Idx;
BOOL Cancel_Sheet;
LPMENU Learn_Spells_Menu;
int Amount_Spells_Learn;
int Total_Cost_Learning;
int Cost_Per_Spell;
BYTE Learn_Table[MAX_SPELLS];
LPMENU Use_Item_Menu;
LPMENU Sel_Character_Menu;
LPMENU Sel_Spell_Menu;
volatile int Sel_Character = -1;
int Sel_Item;
int Sel_Item_Idx;
LPCHARACTER Object_Owner;
int Cur_Sel_Type;
int Selected_Spell;

char Items_To_Use[MAX_CHARACTER_ITEMS];
char Items_To_Use_Label[MAX_CHARACTER_ITEMS];

char Spells_To_Use[MAX_SPELLS];
char Spells_To_Use_Label[MAX_SPELLS];

/* --------------------------------
   Functions
   -------------------------------- */
BOOL Display_Spell_Group(LPCHARACTER Character,
                        int Spell,
                         int *Column,
                         int *Line,
                         char *Name);
BOOL Display_Spell(LPCHARACTER Character,
                   int Spell,
                   int *Column,
                   int *Line,
                   char *Name);
void Create_Character_Sheet_Context(int Idx, BOOL Renew);
void Set_Sheet_Palette();
void Dispatch_Use_Item(LPCHARACTER Owner,
                       int Item,
                       int Place);
void Select_Spell_Events(void *Self,
                         int Item,
                         ULONG Pad_Dats);

/* --------------------------------
   Free the memory allocated by the sheet
   -------------------------------- */
void Destroy_Sheet()
{
    Destroy_Menu(Sel_Spell_Menu);
    Destroy_Menu(Sel_Character_Menu);
    Destroy_Menu(Use_Item_Menu);
    Destroy_Menu(Learn_Spells_Menu);
    Destroy_Menu(Menu_Sheet_Character);
}

/* --------------------------------
   Events of the menu of a character sheet
   -------------------------------- */
void Character_Sheet_Menu_Events(void *Self, int Item, ULONG Pad_Dats)
{
    switch(Pad_Dats)
    {
        case PAD_PREVIOUS:

            /* Previous character */
            Cur_Character_Idx--;
            if(Cur_Character_Idx < 0)
            {
                Cur_Character_Idx = Party.Number_Characters - 1;
            }
            Set_Fade(Black_Palette, 40, MOVE_NONE);
            break;

        case PAD_NEXT:

            /* Next character */
            Cur_Character_Idx++;
            if(Cur_Character_Idx > Party.Number_Characters - 1)
            {
                Cur_Character_Idx = 0;
            }
            Set_Fade(Black_Palette, 40, MOVE_NONE);
            break;

        case PAD_OK:

            if(Item == MENU_CHARACTER_CHANGE_PAGE)
            {
                /* Next / Previous Page */
                Character_Sheet_Page ^= TRUE;

                Display_Character_Stats(Cur_Character, Character_Sheet_Page, FALSE);
                Swap_Physical_Buffers();
                Display_Character_Stats(Cur_Character, Character_Sheet_Page, TRUE);

                if(Character_Sheet_Page)
                {
                    Set_Menu_Line_Text((LPMENU) Self, "PREVIOUS PAGE", 0, ' ', TRUE);
                }
                else
                {
                    Set_Menu_Line_Text((LPMENU) Self, "  NEXT PAGE  ", 0, ' ', TRUE);
                }
            }
            else
            {
                switch(Menu_Type)
                {
                    case MENU_CHARACTER_DEAD:

                        switch(Item)
                        {
            
                            case MENU_CHARACTER_DEAD_TAKE_ITEMS:

                                /* TODO: Take Items (same as give) */
                                if(Party.Number_Characters > 1)
                                {

                                }
                                break;
            
                            case MENU_CHARACTER_DEAD_BURY:

                                /* TODO: Bury Character */
            
                                break;
            
                        }

                        break;
    
                    case MENU_CHARACTER_TOWN:

                        switch(Item)
                        {

                            case MENU_CHARACTER_TOWN_LEARN_SPELL:

                                /* Learn Spells */
                                Learn_Spell(Cur_Character);
                                break;

                            case MENU_CHARACTER_TOWN_CAST_SPELL:

                                /* Cast Spell */
                                Display_Spells_List(Cur_Character, &Select_Spell_Events,
                                                    TOWN_ONLY);
                                break;

                            case MENU_CHARACTER_TOWN_BUY_ITEMS:

                                /* TODO: Buy Items */

                                break;

                            case MENU_CHARACTER_TOWN_SELL_ITEMS:

                                /* TODO: Sell Items */
            
                                break;

                            case MENU_CHARACTER_TOWN_GIVE_ITEMS:

                                /* TODO: Give Items */
                                if(Party.Number_Characters > 1)
                                {

                                }
                                break;

                            case MENU_CHARACTER_TOWN_USE_ITEM:

                                /* Use Item */
                                Character_Select_Item();
                                break;

                        }

                        break;

                    case MENU_CHARACTER_WILDERNESS:

                        switch(Item)
                        {
            
                            case MENU_CHARACTER_WILDERNESS_CAST_SPELL:

                                /* Cast Spell */
                                Display_Spells_List(Cur_Character, &Select_Spell_Events,
                                                    WILDERNESS_DUNGEON | ASTRAL_ONLY);
                                break;
            
                            case MENU_CHARACTER_WILDERNESS_GIVE_ITEMS:

                                /* TODO: Give Items */
                                if(Party.Number_Characters > 1)
                                {

                                }
                                break;
            
                            case MENU_CHARACTER_WILDERNESS_USE_ITEM:

                                /* Use Item */
                                Character_Select_Item();
                                break;
            
                        }
                        break;
    
                }
            }
            break;

        case PAD_CANCEL:

            Cancel_Sheet = TRUE;
            Set_Fade(Black_Palette, 40, MOVE_NONE);

            break;
    }

    if(!Doing_Fade && Ptr_Palette == Black_Palette)
    {
        if(Cancel_Sheet)
        {
            Cancel_Sheet = FALSE;
            Hide_Menu((LPMENU) Self, Pointer);
            Fade_Town_Pic();
            Fix_Current_Town();
            Set_Town_Pic(TRUE);
            Set_Town_Pic(FALSE);
            Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
            Set_Fade(Current_Palette, 40, MOVE_VERTICALLY);
        }
        else
        {
            Create_Character_Sheet_Context(Cur_Character_Idx, TRUE);
            Set_Fade(Current_Palette, 40, MOVE_VERTICALLY);
        }
    }
}

/* --------------------------------
   Construct the menu of a character according to it's state
   -------------------------------- */
void Construct_Character_Sheet_Menu(LPCHARACTER Character, int Location, BOOL Renew)
{
    char Page[32];

    Cur_Sel_Type = SHEET;

    if(Character_Sheet_Page)
    {
        strcpy(Page, "PREVIOUS PAGE");
    }
    else
    {
        strcpy(Page, "  NEXT PAGE  ");
    }

    if(Menu_Sheet_Character)
    {
        Hide_Menu(Menu_Sheet_Character, Pointer);
        Destroy_Menu(Menu_Sheet_Character);
    }

    if(Character->Status == DEAD)
    {
        Menu_Sheet_Character = Construct_Menu(Font,
                                              Pic_Menus,
                                              &Character_Sheet_Menu_Events,
                                              MENU_TEXT_CENTER,
                                              5, 0,
                                              1, 0,
                                              MOVE_VERTICALLY,
                                              "%s\n \n%s\n%s",
                                              Page,
                                              "TAKE ITEMS",
                                              "BURY CHARACTER"
                                              );
        Menu_Type = MENU_CHARACTER_DEAD;
    }
    else
    {
        if(Location == IN_TOWN)
        {
            Menu_Sheet_Character = Construct_Menu(Font,
                                                  Pic_Menus,
                                                  &Character_Sheet_Menu_Events,
                                                  MENU_TEXT_CENTER,
                                                  5, 0,
                                                  1, 0,
                                                  MOVE_VERTICALLY,
                                                  "%s\n \n%s\n%s\n%s\n%s\n%s\n%s\n",
                                                  Page,
                                                  "LEARN SPELLS",
                                                  "CAST SPELL",
                                                  "BUY ITEMS",
                                                  "SELL ITEMS",
                                                  "GIVE ITEMS",
                                                  "USE ITEM"
                                                 );
            Menu_Type = MENU_CHARACTER_TOWN;
        }
        else
        {
            Menu_Sheet_Character = Construct_Menu(Font,
                                                  Pic_Menus,
                                                  &Character_Sheet_Menu_Events,
                                                  MENU_TEXT_CENTER,
                                                  5, 0,
                                                  1, 0,
                                                  MOVE_VERTICALLY,
                                                  "%s\n \n%s\n%s\n%s",
                                                  Page,
                                                  "CAST SPELL",
                                                  "GIVE ITEMS",
                                                  "USE ITEM"
                                                 );
            Menu_Type = MENU_CHARACTER_WILDERNESS;
        }
    }
    Show_Menu(Menu_Sheet_Character, Pointer, 5, 180, 0);
}

/* --------------------------------
   Display the infos of a character
   -------------------------------- */
void Display_Character_Stats(LPCHARACTER Character, BOOL Page2, BOOL Copy)
{
    int i;
    int Needed_Exp = 0;

    if(Page2 == FALSE)
    {
        Clear_Array(8, 4 + (9 * 5) - 8, SCREEN_WIDTH - 8, 15 * 9);
        Clear_Array(158, 4 + (9 * 17) - 8, SCREEN_WIDTH - 158, 10 * 9);

        if(Copy)
        {
            Copy_From_Back(0, 0, SCREEN_WIDTH, (15 * 9) + 41);
            Copy_From_Back(158, 4 + (9 * 17) - 8, SCREEN_WIDTH - 158, 10 * 9);
        }
        else
        {
            /* Character pic */
            Copy_To_Screen(Pic_Party,
                           0, Character->Race * 32,
                           4, 2,
                           32, 32);

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  40, 4, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "%-12s       STATUS: %s",
                  Character->Name,
                  Status_List[Character->Status]
                 );

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  40, 4 + (9 * 2) - 4, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "  RACE: %-11s CLASS: %-7s",
                  Races_List[Character->Race],
                  Classes_List[Character->Class]
                 );

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  40, 4 + (9 * 3) - 4, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "ORIGIN: %-11s LEVEL: %-2d",
                  Origin_Land[Character->Origin],
                  Character->Level,
                  Races_List[Character->Race]
                 );

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 5) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "    STRENGTH: %2d          AGE: %d",
                  Character->Strength,
                  Character->Age / 26
                 );

            Display_Hit_Points(Character);

            Display_Magic_Points(Character);

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 8) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "CONSTITUTION: %2d GOLD IN BANK: %d",
                  Character->Constitution,
                  Character->Gold
                 );

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 9) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "    CHARISMA: %2d   EXPERIENCE: %d",
                  Character->Charisma,
                  Character->Experience
                 );

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 10) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "        LUCK: %2d        SCORE: %d",
                  Character->Luck,
                  Character->Score
                 );

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 12) - 12, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "      ATTACK: %2d    SPOT TRAP: %2d",
                  Character->Skills[ATTACK],
                  Character->Skills[SPOT_TRAP]
                 );
            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 13) - 12, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "       PARRY: %2d  DISARM TRAP: %2d",
                  Character->Skills[PARRY],
                  Character->Skills[DISARM_TRAP]
                 );
            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 14) - 12, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "        SWIM: %2d    FIND ITEM: %2d",
                  Character->Skills[SWIM],
                  Character->Skills[FIND_ITEM]
                 );
            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 15) - 12, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "      LISTEN: %2d    PICK LOCK: %2d",
                  Character->Skills[LISTEN],
                  Character->Skills[PICK_LOCK]
                 );

            Needed_Exp = Calc_Needed_Experience(Character);

            if(Character->Level >= MAX_LEVELS && Needed_Exp >= 0)
            {
                /* No more level up */
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      8, 4 + (9 * 16) - 12, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "   TOSS ROCK: %2d",
                      Character->Skills[TOSS_ROCK]
                     );
            }
            else
            {
            
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      8, 4 + (9 * 16) - 12, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "   TOSS ROCK: %2d  (NEXT LEVEL: %d)",
                      Character->Skills[TOSS_ROCK],
                      -Needed_Exp
                     );
            }
        
            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 17) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "%16s\n%16s\n%16s",
                  Items_List[Character->Equipped_Weapon],
                  Items_List[Character->Equipped_Armor],
                  Items_List[Character->Equipped_Shield]
                 );
        
            int Healing_Potions = 0;
            int Magic_Potions = 0;
            int Scrolls = 0;
            int Treasures = 0;
            int Amulets = 0;
            int Beast_Runes = 0;
            int Have_Orb = 0;
            int Have_Filmon_Wand = 0;
            int Have_Nikademus_Wand = 0;
            int Data;
            int Line = 17;
        
            for(i = 0; i < MAX_CHARACTER_ITEMS; i++)
            {
                Data = Character->Items[i];
        
                Have_Orb |= (Data == ORB);
                Have_Filmon_Wand |= (Data == FLMN_WAND);
        
                if(Data >= HEAL_POTION_1 &&
                   Data <= HEAL_POTION_10)
                {
                    Healing_Potions++;
                }
        
                if(Data >= MAGIC_POTION_1 &&
                   Data <= MAGIC_POTION_10)
                {
                    Magic_Potions++;
                }
        
                if(Data >= SCROLL_1 &&
                   Data <= SCROLL_19)
                {
                    Scrolls++;
                }
        
                if(Data >= AMULET_1 &&
                   Data <= AMULET_8)
                {
                    Amulets++;
                }
        
                if(Data >= QUARTZ)
                {
                    Treasures++;
                }
            }
        
            Have_Nikademus_Wand |= (Character->Wand_Of_Nikademus > 99);
        
            for(i = 0; i < MAX_BEAST_RUNES; i++)
            {
                Data = Party.Beast_Runes[i];
                Beast_Runes += (Data == DECRYPTED);
            }
        
            if(Healing_Potions)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "HEALING POTIONS: %2d",
                      Healing_Potions
                     );
                Line++;
            }
            
            if(Magic_Potions)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "  MAGIC POTIONS: %2d",
                      Magic_Potions
                     );
                Line++;
            }
            
            if(Scrolls)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "        SCROLLS: %2d",
                      Scrolls
                     );
                Line++;
            }
            
            if(Treasures)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "      TREASURES: %2d",
                      Treasures
                     );
                Line++;
            }
            
            if(Amulets)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "        AMULETS: %2d",
                      Amulets
                     );
                Line++;
            }
        
            if(Beast_Runes)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "    BEAST RUNES: %2d",
                      Beast_Runes
                     );
                Line++;
            }
        
            Line++;
            
            if(Have_Orb)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158 + 8, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "HAVE THE ORB"
                     );
                Line++;
            }
            
            if(Have_Filmon_Wand)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158 + 8, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "HAVE FILMON WAND"
                     );
                Line++;
            }
            
        
            if(Have_Nikademus_Wand)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      158 + 8, 4 + (9 * Line) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "HAVE NIKADEMUS WAND"
                     );
                Line++;
            }
        }
    }
    else
    {
        if(Copy)
        {
            Copy_From_Back(0, 0, SCREEN_WIDTH, (15 * 9) + 41);
            Copy_From_Back(158, 4 + (9 * 17) - 8, SCREEN_WIDTH - 158, 10 * 9);
        }
        else
        {
            int Line = 4 + (9 * 9) - 8;
            int Column = 8 + (7 * 8);
            BOOL Element = FALSE;
    
            /* Character pic */
            Copy_To_Screen(Pic_Party,
                           0, Character->Race * 32,
                           4, 2,
                           32, 32);

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  40, 4, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "%-12s       STATUS: %s",
                  Character->Name,
                  Status_List[Character->Status]
                 );

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  40, 4 + (9 * 2) - 4, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "  RACE: %-11s CLASS: %-7s",
                  Races_List[Character->Race],
                  Classes_List[Character->Class]
                 );

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  40, 4 + (9 * 3) - 4, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "ORIGIN: %-11s LEVEL: %-2d",
                  Origin_Land[Character->Origin],
                  Character->Level,
                  Races_List[Character->Race]
                 );

            Clear_Array(8, 4 + (9 * 5) - 8, SCREEN_WIDTH - 8, 15 * 9);
            Clear_Array(158, 4 + (9 * 17) - 8, SCREEN_WIDTH - 158, 10 * 9);

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 5) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "RUNES:"
                 );

            if(Element == FALSE)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      Column, 4 + (9 * 5) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "(NONE)"
                     );
            }

            Print(Font, MENU_TEXT_SECONDARY_FONT,
                  8, 4 + (9 * 7) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "SPELLS:"
                 );

            Column = 0;
            Line = 0;
            Element = FALSE;

            Element |= Display_Spell_Group(Character, HEALING_1, &Column, &Line, "HEALING");
            Element |= Display_Spell_Group(Character, FIREFLASH_1, &Column, &Line, "FIREFLASH");
            Element |= Display_Spell_Group(Character, QUICKNESS_1, &Column, &Line, "QUICKNESS");
            Element |= Display_Spell_Group(Character, STRENGTH_1, &Column, &Line, "STRENGTH");
            Element |= Display_Spell_Group(Character, PROTECTION_1, &Column, &Line, "PROTECTION");
            Element |= Display_Spell_Group(Character, CONFUSION_1, &Column, &Line, "CONFUSION");
            Element |= Display_Spell_Group(Character, WEAKNESS_1, &Column, &Line, "WEAKNESS");
            Element |= Display_Spell_Group(Character, BINDING_1, &Column, &Line, "BINDING");
            Element |= Display_Spell_Group(Character, MINDBLAST_1, &Column, &Line, "MINDBLAST");
            Element |= Display_Spell_Group(Character, FLAMEBOLT_1, &Column, &Line, "FLAMEBOLT");

            Element |= Display_Spell(Character, CHARM, &Column, &Line, Spells_List[CHARM].Name);
            Element |= Display_Spell(Character, SLEEP, &Column, &Line, Spells_List[SLEEP].Name);
            Element |= Display_Spell(Character, TELEPORTATION, &Column, &Line, Spells_List[TELEPORTATION].Name);
            Element |= Display_Spell(Character, RESURRECTION, &Column, &Line, Spells_List[RESURRECTION].Name);
            Element |= Display_Spell(Character, BERSERK_1, &Column, &Line, Spells_List[BERSERK_1].Name);
            Element |= Display_Spell(Character, BERSERK_2, &Column, &Line, Spells_List[BERSERK_2].Name);
            Element |= Display_Spell(Character, FEAR, &Column, &Line, Spells_List[FEAR].Name);
            Element |= Display_Spell(Character, DISSOLVE, &Column, &Line, Spells_List[DISSOLVE].Name);
            Element |= Display_Spell(Character, SUMMON_ELEMENTAL, &Column, &Line, Spells_List[SUMMON_ELEMENTAL].Name);
            Element |= Display_Spell(Character, DISPELL_UNDEAD, &Column, &Line, Spells_List[DISPELL_UNDEAD].Name);
            Element |= Display_Spell(Character, AWAKEN, &Column, &Line, Spells_List[AWAKEN].Name);
            Element |= Display_Spell(Character, EVAL_MONSTER, &Column, &Line, Spells_List[EVAL_MONSTER].Name);
            Element |= Display_Spell(Character, VISION, &Column, &Line, Spells_List[VISION].Name);
            Element |= Display_Spell(Character, TRANSPORTATION, &Column, &Line, Spells_List[TRANSPORTATION].Name);
            Element |= Display_Spell(Character, DIVINE_I, &Column, &Line, Spells_List[DIVINE_I].Name);
            Element |= Display_Spell(Character, DIVINE_II, &Column, &Line, Spells_List[DIVINE_II].Name);

            if(Element == FALSE)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      8 + (8 * 8), 4 + (9 * 7) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "(NONE)"
                     );
            }
        }
    }
}

/* --------------------------------
   Display the dexterity and magic of a given character
   -------------------------------- */
void Display_Hit_Points(LPCHARACTER Character)
{
    if(Character_Sheet_Page == FALSE)
    {
        Clear_Array(8, 4 + (9 * 6) - 8, SCREEN_WIDTH - 8, 8);
        Print(Font, MENU_TEXT_SECONDARY_FONT,
              8, 4 + (9 * 6) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "INTELLIGENCE: %2d   HIT POINTS: %d/%d",
              Character->Intelligence,
              Character->Current_Hit_Points, Character->Max_Hit_Points
             );
    }
}

/* --------------------------------
   Display the dexterity and magic of a given character
   -------------------------------- */
void Display_Magic_Points(LPCHARACTER Character)
{
    if(Character_Sheet_Page == FALSE)
    {
        Clear_Array(8, 4 + (9 * 7) - 8, SCREEN_WIDTH - 8, 8);
        Print(Font, MENU_TEXT_SECONDARY_FONT,
              8, 4 + (9 * 7) - 8, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "   DEXTERITY: %2d        MAGIC: %d/%d",
              Character->Dexterity,
              Character->Current_Magic, Character->Max_Magic
             );
    }
}

/* --------------------------------
   Display a group of spell on the character sheet
   -------------------------------- */
BOOL Display_Spell_Group(LPCHARACTER Character,
                         int Spell,
                         int *Column,
                         int *Line,
                         char *Name)
{
    int i;
    BOOL Result = FALSE;

    if(Character->Spells[Spell] == LEARNED ||
       Character->Spells[Spell + 1] == LEARNED ||
       Character->Spells[Spell + 2] == LEARNED ||
       Character->Spells[Spell + 3] == LEARNED)
    {
        Print(Font, MENU_TEXT_SECONDARY_FONT,
              (*Column * 8), *Line + (4 + (9 * 9) - 8), SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "%15s",
              Name  
             );
        for(i = 0; i < 4; i++)
        {
            if(Character->Spells[Spell + i] == LEARNED)
            {
                Print(Font, MENU_TEXT_SECONDARY_FONT,
                      ((*Column + i + 16) * 8), *Line + (4 + (9 * 9) - 8),
                      SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "%d",
                      i + 1
                     );
            }
        }
        if(*Line >= (11 * 9))
        {
            *Column = 17;
            *Line += 9;
        }
        else
        {
            *Column ^= 17;
            if(*Column == 0) *Line += 9;
            if(*Line >= (11 * 9)) *Column = 17;
        }
        Result = TRUE;
    }
    return Result;
}

/* --------------------------------
   Display a single spell on the character sheet
   -------------------------------- */
BOOL Display_Spell(LPCHARACTER Character,
                   int Spell,
                   int *Column,
                   int *Line,
                   char *Name)
{
    BOOL Result = FALSE;

    if(Character->Spells[Spell] == LEARNED)
    {
        Print(Font, MENU_TEXT_SECONDARY_FONT,
              ((*Column + 4) * 8), *Line + (4 + (9 * 9) - 8),
              SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "%16s",
              Name  
             );
        if(*Line >= (11 * 9))
        {
            *Column = 17;
            *Line += 9;
        }
        else
        {
            *Column ^= 17;
            if(*Column == 0) *Line += 9;
            if(*Line >= (11 * 9)) *Column = 17;
        }
        Result = TRUE;
    }
    return Result;
}

/* --------------------------------
   Prepare the context of a character sheet
   -------------------------------- */
void Prepare_Character_Sheet_Context(int Location, int Idx)
{
    Set_Sheet_Palette();

    Current_Party_Location = Location;

    Set_Fade(Current_Palette, 40, MOVE_VERTICALLY);

    Character_Sheet_Page = FALSE;
    Create_Character_Sheet_Context(Idx, FALSE);
}

/* --------------------------------
   Create the data of a character sheet
   -------------------------------- */
void Create_Character_Sheet_Context(int Idx, BOOL Renew)
{
    /* The character to display */
    Cur_Character_Idx = Idx;
    Cur_Character = &Party.Characters[Idx];

    Clear_Array(0, 0, SCREEN_WIDTH, 4 + (9 * 5) - 8);
    Display_Character_Stats(Cur_Character, Character_Sheet_Page, FALSE);
    Swap_Physical_Buffers();
    Clear_Array(0, 0, SCREEN_WIDTH, 4 + (9 * 5) - 8);
    Display_Character_Stats(Cur_Character, Character_Sheet_Page, TRUE);

    /* FIXME: just a hack for testing purpose */
    /* Cur_Character->Items[0] = HEAL_POTION_1; */

    /* Create the menu for this character */
    Construct_Character_Sheet_Menu(Cur_Character, Current_Party_Location, Renew);
}

/* --------------------------------
   Prepare the palette for a character sheet
   -------------------------------- */
void Set_Sheet_Palette()
{
    int i;

    for(i = 0; i < PARTY_COLORS; i++)
    {
        Current_Palette[i] = Party_Palette[i];
    }
    for(i = 239; i < 239 + MENUS_COLORS; i++)
    {
        Current_Palette[i] = Menus_Palette[i];
    }
    Current_Palette[254] = 0xa5a09a;
    Current_Palette[255] = 0;
    Current_Palette[237] = 0x706a60;
    Current_Palette[238] = 0xc5c0ba;
}

/* --------------------------------
   Events of the learning spells menu
   -------------------------------- */
void Learn_Spell_Menu_Events(void *Self, int Item, ULONG Pad_Dats)
{
    char Text_Line[48];
    char Learn_Report[512];
    int i;
    BOOL No_Spell_Sel;

    switch(Pad_Dats)
    {
        case PAD_LEFT:

            if(Item >= 5 && (Item < (5 + Amount_Spells_Learn)))
            {
                if(Total_Cost_Learning >= Cost_Per_Spell)
                {
                    Total_Cost_Learning -= Cost_Per_Spell;
                    Learn_Table[Item - 5] &= 0x7f;
    
                    sprintf(Text_Line, "%16s:    <  SKIP >\n", Spells_List[Learn_Table[Item - 5] & 0x7f].Name);
                    Set_Menu_Line_Text((LPMENU) Self, Text_Line, Item, ' ', TRUE);

                    sprintf(Text_Line, "GOLD PIECES CARRIED: %5d\n", Party.Gold - Total_Cost_Learning);
                    Set_Menu_Line_Text((LPMENU) Self, Text_Line, 3, ' ', TRUE);
                }
            }
            break;

        case PAD_RIGHT:

            if(Item >= 5 && (Item < (5 + Amount_Spells_Learn)))
            {
                if((Total_Cost_Learning + Cost_Per_Spell) <= Party.Gold)
                {
                    Total_Cost_Learning += Cost_Per_Spell;
                    /* Within range */
                    Learn_Table[Item - 5] |= 0x80;
    
                    sprintf(Text_Line, "%16s:    < LEARN >\n", Spells_List[Learn_Table[Item - 5] & 0x7f].Name);
                    Set_Menu_Line_Text((LPMENU) Self, Text_Line, Item, ' ', TRUE);

                    sprintf(Text_Line, "GOLD PIECES CARRIED: %5d\n", Party.Gold - Total_Cost_Learning);
                    Set_Menu_Line_Text((LPMENU) Self, Text_Line, 3, ' ', TRUE);
                }
            }
            break;

        case PAD_OK:

            /* Done */
            if(Item >= ((5 + Amount_Spells_Learn) + 1))
            {
                Hide_Menu((LPMENU) Self, Pointer);
                Destroy_Menu(Learn_Spells_Menu);

                /* Take all the gold away */
                Party.Gold -= Total_Cost_Learning;

                memset(Learn_Report, 0, sizeof(Learn_Report));

                No_Spell_Sel = FALSE;
                for(i = 0; i < Amount_Spells_Learn; i++)
                {
                    /* Want to learn this spell */
                    if(Learn_Table[i] & 0x80)
                    {
                        No_Spell_Sel = TRUE;
                        /* Learning Spells takes time */
                        Cur_Character->Age += 2;

                        /* And it may not even be learned successfully */
                        if ((rnd() * 100.0f) < (Cur_Character->Intelligence * 5))
                        {
                            /* Spell learned */
                            Cur_Character->Spells[Learn_Table[i] & 0x7f] = LEARNED;
                            strcat(Learn_Report, Spells_List[Learn_Table[i] & 0x7f].Name);
                            strcat(Learn_Report, ": LEARNED\n");
                        }
                        else
                        {
                            /* Failed to learn it */
                            strcat(Learn_Report, Spells_List[Learn_Table[i] & 0x7f].Name);
                            strcat(Learn_Report, ": FAILED\n");
                        }
                    }
                }

                /* Print the report if necessary */
                if(No_Spell_Sel)
                {
                    Display_Message(Font, Pic_Menus, -1, Learn_Report);
                }

                /* Refresh the 2nd sheet page if it's currently displayed */
                if(Character_Sheet_Page)
                {
                    Display_Character_Stats(Cur_Character, Character_Sheet_Page, FALSE);
                    Swap_Physical_Buffers();
                    Display_Character_Stats(Cur_Character, Character_Sheet_Page, TRUE);
                }
            }
            break;

        case PAD_CANCEL:

            Hide_Menu((LPMENU) Self, Pointer);
            Destroy_Menu(Learn_Spells_Menu);
            break;
    }
}

/* --------------------------------
   Make a given character to learn a spell
   -------------------------------- */
void Learn_Spell(LPCHARACTER Character)
{
    int i;
    int Possible_Spells;
    int Learned_Spells;
    char Message[512];
    char Line[64];

    strcpy(Message, "THE COST IS %d GOLD PIECES\n");
    strcat(Message, "FOR EACH SPELL YOU TRY TO LEARN\n");
    strcat(Message, " \n");
    strcat(Message, "GOLD PIECES CARRIED: %5d\n");
    strcat(Message, " \n");

    Possible_Spells = (int) (Character->Level *
                             pow(Character->Intelligence, 1.177f) / 90.0f *
                             (Character->Class + 1 - 3 * (Character->Class == FIGHTER))
                            );

    Learned_Spells = 0;
    Amount_Spells_Learn = 0;
    for(i = 0; i < MAX_SPELLS; i++)
    {
        if(Character->Spells[i] == LEARNED)
        {
            Learned_Spells++;
        }
        if(Character->Spells[i] == LEARNABLE)
        {
            sprintf(Line, "%16s:    <  SKIP >\n", Spells_List[i].Name);
            strcat(Message, Line);
            /* Store the index of the spell */
            Learn_Table[Amount_Spells_Learn] = i;
            Amount_Spells_Learn++;
        }
    }
    strcat(Message, " \nDONE");

    if(!Amount_Spells_Learn)
    {
        Display_Message(Font, Pic_Menus, -1, "YOU NEED MORE EXPERIENCE");
        return;
    }

    if(Learned_Spells > Possible_Spells)
    {
        Display_Message(Font, Pic_Menus, -1, "YOU CAN NOT LEARN ANY MORE");
        return;
    }

    Cost_Per_Spell = int(Prices_Multiplier * 9000.0f /
                         Character->Charisma /
                         Character->Intelligence * Character->Level /
                         (9 - Character->Class)
                        );
    
    if(Cost_Per_Spell > Party.Gold)
    {
        Display_Message(Font, Pic_Menus, -1, "YOU NEED TO CARRY AT LEAST\n%d GOLD PIECES", Cost_Per_Spell);
        return;
    }

    Total_Cost_Learning = 0;

    Learn_Spells_Menu = Construct_Menu(Font,
                                       Pic_Menus,
                                       &Learn_Spell_Menu_Events,
                                       MENU_TEXT_CENTER,
                                       12, 5,
                                       1, 0,
                                       MOVE_VERTICALLY | MOVE_HORIZONTALLY,
                                       Message,
                                       Cost_Per_Spell,
                                       Party.Gold
                                      );

    Show_Menu(Learn_Spells_Menu, Pointer, -1, -1, 0);
}

/* --------------------------------
   Events of the use item menu
   -------------------------------- */
void Select_Item_Events(void *Self, int Item, ULONG Pad_Dats)
{
    switch(Pad_Dats)
    {
        case PAD_OK:

            if(Item >= 2)
            {
                Dispatch_Use_Item(Cur_Character,
                                  Item - 2,
                                  Current_Party_Location);
            }
            break;

        case PAD_CANCEL:

            Hide_Menu((LPMENU) Self, Pointer);
            Destroy_Menu(Use_Item_Menu);
            break;

    }
}

/* --------------------------------
   Current character uses an item
   -------------------------------- */
void Character_Select_Item()
{
    int i;
    int Items = 0;
    int Item_Idx;
    char Message[256];

    strcpy(Message, "SELECT AN ITEM TO USE\n ");

    for(i = 0; i < MAX_CHARACTER_ITEMS; i++)
    {
        Item_Idx = Cur_Character->Items[i];
        /* Only display the usable items */
        if(Item_Idx > 0 &&
           ((Item_Idx >= HEAL_POTION_1 && Item_Idx <= HEAL_POTION_10) ||
           (Item_Idx >= MAGIC_POTION_1 && Item_Idx <= MAGIC_POTION_10) ||
           (Item_Idx >= SCROLL_1 && Item_Idx <= SCROLL_19))
          )
        {
            strcat(Message, "\n");
            strcat(Message, Items_List[Item_Idx - 1].Name);
            /* Store the index */
            Items_To_Use[Items] = i;
            /* Store the label */
            Items_To_Use_Label[Items] = Item_Idx;
            Items++;
        }
    }
    
    if(!Items)
    {
        Display_Message(Font, Pic_Menus, -1, "YOU CAN NOT USE ANY ITEM");
        return;
    }

    Use_Item_Menu = Construct_Menu(Font,
                                   Pic_Menus,
                                   &Select_Item_Events,
                                   MENU_TEXT_CENTER,
                                   11, 2,
                                   1, 0,
                                   MOVE_VERTICALLY,
                                   Message
                                  );

    Show_Menu(Use_Item_Menu, Pointer, -1, -1, 0);
}

/* --------------------------------
   Use a potion on a given character
   -------------------------------- */
void Use_Healing_Item(LPCHARACTER Owner,
                      int Idx_In_Owner_List,
                      LPCHARACTER Target_Character,
                      int Item)
{
    int Value;

    /* Used */
    if(Item >= HEAL_POTION_1 &&
       Item <= HEAL_POTION_10)
    {
        Value = (int) (pow(Item - (HEAL_POTION_1 - 1), 2.0f) + (float) Target_Character->Current_Hit_Points);
        if(Value > Target_Character->Max_Hit_Points)
        {
            Value = Target_Character->Max_Hit_Points;
        }
        Target_Character->Current_Hit_Points = Value;
        Owner->Items[Idx_In_Owner_List] = NONE;
        if(Owner == Target_Character)
        {
            Display_Message(Font, Pic_Menus, -1, "YOU ARE HEALED");
        }
        else
        {
            Display_Message(Font, Pic_Menus, -1, "%s IS HEALED", Target_Character->Name);
        }
    }
    else if(Item >= MAGIC_POTION_1 &&
            Item <= MAGIC_POTION_10)
    {
        Value = ((Item * 3) - ((MAGIC_POTION_1 - 1) * 3)) + Target_Character->Current_Magic;
        if(Value > Target_Character->Max_Magic)
        {
            Value = Target_Character->Max_Magic;
        }
        Target_Character->Current_Magic = Value;

        /* Used so remove it */
        Owner->Items[Idx_In_Owner_List] = NONE;
        if(Owner == Target_Character)
        {
            Display_Message(Font, Pic_Menus, -1, "YOU RECEIVE MAGIC");
        }
        else
        {
            Display_Message(Font, Pic_Menus, -1, "%s RECEIVES MAGIC", Target_Character->Name);
        }
    }

    /* Refresh the page if necessary */
    if(!Character_Sheet_Page || (Owner == Target_Character))
    {
        Display_Character_Stats(Cur_Character, Character_Sheet_Page, FALSE);
        Swap_Physical_Buffers();
        Display_Character_Stats(Cur_Character, Character_Sheet_Page, TRUE);
    }
}

/* --------------------------------
   Events of the use item menu
   -------------------------------- */
void Use_Item_Events(void *Self, int Item, ULONG Pad_Dats)
{
    switch(Pad_Dats)
    {
        case PAD_OK:

            Sel_Character = (Item - 2) + 1;
            if(Sel_Character > 0)
            {
                /* Action is irreversible from here */
                Hide_Menu((LPMENU) Self, Pointer);
                Destroy_Menu(Sel_Character_Menu);

                Hide_Menu(Use_Item_Menu, Pointer);
                Destroy_Menu(Use_Item_Menu);

                Use_Healing_Item(Object_Owner, Sel_Item_Idx,
                                 &Party.Characters[Sel_Character - 1], Sel_Item);
                Sel_Character = -1;
            }
            break;

        case PAD_CANCEL:

            /* No character selected */
            Hide_Menu((LPMENU) Self, Pointer);
            Destroy_Menu(Sel_Character_Menu);
            break;
    }
}

/* --------------------------------
   Use the selected item
   -------------------------------- */
void Dispatch_Use_Item(LPCHARACTER Owner,
                       int Item,
                       int Place)
{
    int Score;
    int Value;

    Sel_Item_Idx = Items_To_Use[Item];
    Sel_Item = Items_To_Use_Label[Item];
    Object_Owner = Owner;

    if((Sel_Item >= HEAL_POTION_1 && Sel_Item <= HEAL_POTION_10) ||
       (Sel_Item >= MAGIC_POTION_1 && Sel_Item <= MAGIC_POTION_10))
    {
        Select_Character(&Use_Item_Events, Cur_Sel_Type);
    }
    else if(Sel_Item >= SCROLL_1 && Sel_Item <= SCROLL_19)
    {
        Hide_Menu(Use_Item_Menu, Pointer);
        Destroy_Menu(Use_Item_Menu);

        Display_Scroll(Sel_Item - SCROLL_1);
    }
    else if(Sel_Item == FLMN_WAND)
    {
        Hide_Menu(Use_Item_Menu, Pointer);
        Destroy_Menu(Use_Item_Menu);

        Score = Owner->Score;
        if(Score != 200)
        {
            Display_Message(Font, Pic_Menus, -1, "FILMON WAND CAN NOT BE USED RIGHT NOW");
            return;
        }
        else
        {
            /* Remove it from inventory */
            Owner->Items[Sel_Item_Idx] = NONE;
            Owner->Score = 250;

            Value = Owner->Strength;
            Owner->Strength = Value + (Value < 20) + (Value < 9) + (Value < 18) * (Owner->Luck > 11);
            
            Value = Owner->Intelligence;
            Owner->Intelligence = Value + (Value < 20) + (Value < 9) + (Value < 18) * (Owner->Luck > 11);
            
            Value = Owner->Dexterity;
            Owner->Dexterity = Value + (Value < 20) + (Value < 9) + (Value < 18) * (Owner->Luck > 11);
            
            Value = Owner->Constitution;
            Owner->Constitution = Value + (Value < 20) + (Value < 9) + (Value < 18) * (Owner->Luck > 11);
            
            Value = Owner->Charisma;
            Owner->Charisma = Value + (Value < 20) + (Value < 9) + (Value < 18) * (Owner->Luck > 11);
        }

        Display_Message(Font, Pic_Menus, -1, "YOU FEEL IMPROVED");

        if(!Character_Sheet_Page)
        {
            Display_Character_Stats(Cur_Character, Character_Sheet_Page, FALSE);
            Swap_Physical_Buffers();
            Display_Character_Stats(Cur_Character, Character_Sheet_Page, TRUE);
        }
    }
}

/* --------------------------------
   Create a menu with the characters in the party
   TODO: we'll use the same function for combat so we need
         to have a graphical selector (not a list of names)
   -------------------------------- */
void Select_Character(void (*Events)(void *, int, ULONG), int Selection_Type)
{
    int i;
    char Names[256];

    Sel_Character = -1;

    if(Sel_Character_Menu)
    {
        Destroy_Menu(Sel_Character_Menu);
    }

    if(Party.Number_Characters == 1)
    {
        /* No need to select a character */
        Hide_Menu(Use_Item_Menu, Pointer);
        Destroy_Menu(Use_Item_Menu);
        Use_Healing_Item(Cur_Character, Sel_Item_Idx, Cur_Character, Sel_Item);
        return;
    }

    strcpy(Names, "ON WHOM\n ");

    for(i = 0; i < Party.Number_Characters; i++)
    {
        strcat(Names, "\n");
        strcat(Names, Party.Characters[i].Name);
    }

    Sel_Character_Menu = Construct_Menu(Font,
                                        Pic_Menus,
                                        Events,
                                        MENU_TEXT_CENTER,
                                        8, 2,
                                        1, 0,
                                        MOVE_VERTICALLY,
                                        Names
                                       );
    Show_Menu(Sel_Character_Menu, Pointer,
              ((SCREEN_WIDTH - Sel_Character_Menu->Width) / 2) + 30, -1, 0);
}

/* --------------------------------
   Events of the cast spell menu
   -------------------------------- */
void Cast_Spell_Events(void *Self, int Item, ULONG Pad_Dats)
{
    switch(Pad_Dats)
    {
        case PAD_OK:

            Sel_Character = (Item - 2) + 1;
            if(Sel_Character > 0)
            {
                /* Action is irreversible from here */
                Hide_Menu((LPMENU) Self, Pointer);
                Destroy_Menu(Sel_Character_Menu);

                Hide_Menu(Sel_Spell_Menu, Pointer);
                Destroy_Menu(Sel_Spell_Menu);

                Cast_Spell(Cur_Character, &Party.Characters[Sel_Character - 1], Selected_Spell);
                Sel_Character = -1;
            }
            break;

        case PAD_CANCEL:

            /* No character selected */
            Hide_Menu((LPMENU) Self, Pointer);
            Destroy_Menu(Sel_Character_Menu);
            break;
    }
}

/* --------------------------------
   Wait for the user to select a spell
   -------------------------------- */
void Select_Spell_Events(void *Self, int Item, ULONG Pad_Dats)
{
    switch(Pad_Dats)
    {
        case PAD_OK:

            Selected_Spell = Spells_To_Use[Item - 2];
            switch(Selected_Spell)
            {
                /* Toward characters */
                case HEALING_1:
                case HEALING_2:
                case HEALING_3:
                case HEALING_4:
                case QUICKNESS_1:
                case QUICKNESS_2:
                case QUICKNESS_3:
                case QUICKNESS_4:
                case STRENGTH_1:
                case STRENGTH_2:
                case STRENGTH_3:
                case STRENGTH_4:
                case PROTECTION_1:
                case PROTECTION_2:
                case PROTECTION_3:
                case PROTECTION_4:
                case BERSERK_2:
                case BERSERK_1:

                    Select_Character(&Cast_Spell_Events, Cur_Sel_Type);
                    break;

                /* Toward monsters */
                case FIREFLASH_1:
                case FIREFLASH_2:
                case FIREFLASH_3:
                case FIREFLASH_4:
                case CONFUSION_1:
                case CONFUSION_2:
                case CONFUSION_3:
                case CONFUSION_4:
                case WEAKNESS_1:
                case WEAKNESS_2:
                case WEAKNESS_3:
                case WEAKNESS_4:
                case BINDING_1:
                case BINDING_2:
                case BINDING_3:
                case BINDING_4:
                case MINDBLAST_1:
                case MINDBLAST_2:
                case MINDBLAST_3:
                case MINDBLAST_4:
                case FLAMEBOLT_1:
                case FLAMEBOLT_2:
                case FLAMEBOLT_3:
                case FLAMEBOLT_4:
                case CHARM:
                case SLEEP:
                case FEAR:
                case DISSOLVE:

                    /* TODO: select monster */
                    break;

                /* Global */
                case TELEPORTATION:
                case RESURRECTION:
                case SUMMON_ELEMENTAL:
                case DISPELL_UNDEAD:
                case AWAKEN:
                case EVAL_MONSTER:
                case VISION:
                case TRANSPORTATION:
                case DIVINE_II:
                case DIVINE_I:

                    Cast_Spell(Cur_Character, NULL, Selected_Spell);
                    break;
            }
            break;

        case PAD_CANCEL:

            /* No spell selected */
            Hide_Menu((LPMENU) Self, Pointer);
            Destroy_Menu(Sel_Spell_Menu);
            break;
    }
}

/* --------------------------------
   Create a menu with the spells of a character
   -------------------------------- */
void Display_Spells_List(LPCHARACTER Character,
                         void (*Events)(void *, int, ULONG),
                         int Type)
{
    int i;
    int Idx_Spell;
    char Names[256];
    char Spell_Name[64];
    LPSPELL Spell;
    int Needed_Magic;

    strcpy(Names, "WHICH SPELL ?\n ");

    if(Sel_Spell_Menu)
    {
        Destroy_Menu(Sel_Spell_Menu);
    }

    Idx_Spell = 0;

    for(i = 0; i < MAX_SPELLS; i++)
    {
        if(Character->Spells[i] == LEARNED)
        {
            Spell = &Spells_List[i];
            if(Spell->Type & EVERYWHERE ||
               Spell->Type & Type
              )
            {
                /* Only add the spell in the list if the character
                   have enough magic points
                 */
                Needed_Magic = Calc_Spell_Cost(i);
                if(Character->Current_Magic >= Needed_Magic)
                {
                    strcat(Names, "\n");
                    sprintf(Spell_Name, "%s (%d POINT%s)", Spell->Name, Needed_Magic, (Needed_Magic > 1) ? "S" : "");
                    strcat(Names, Spell_Name);
                    Spells_To_Use[Idx_Spell] = i;
                    Idx_Spell++;
                }
            }
        }
    }

    if(!Idx_Spell)
    {
        Display_Message(Font, Pic_Menus, -1, "YOU CAN NOT CAST ANY SPELL");
        return;
    }

    Sel_Spell_Menu = Construct_Menu(Font,
                                    Pic_Menus,
                                    Events,
                                    MENU_TEXT_CENTER,
                                    10, 2,
                                    1, 0,
                                    MOVE_VERTICALLY,
                                    Names
                                    );
    Show_Menu(Sel_Spell_Menu, Pointer, -1, -1, 0);
}
