/* --------------------------------
   Phantasie II CD32
   Main program
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   
       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* TODO: Options to set CD and SFX volumes and maybe a jukebox
         (i'll probably use the START/PLAY BUTTON or maybe not). */

/* --------------------------------
   Includes
   -------------------------------- */
#include "main.h"
#include "towns.h"
#include "scrolls.h"
#include "characters.h"
#include "characters_sheets.h"
#include "save.h"

/* --------------------------------
   Variables
   -------------------------------- */
const char *PROGNAME = "Phantasie II";
const char *INPUTNAME = "Phantasie II input handler";

/* Note: the CD32 usually have pad 1 in port 0 (i think) */
int JOYPAD_PORT = 1;
const WORD SCREEN_WIDTH = 320;
const WORD SCREEN_HEIGHT = 256;
const WORD SCREEN_BPP = 8;
const WORD SCREEN_COLORS = 256;

ULONG Current_Palette[256];
ULONG Title_Palette[256];
ULONG Menus_Palette[256];
ULONG Towns_Palette[256];
ULONG Party_Palette[256];
ULONG Tiles_Palette[256];
ULONG *Ptr_Palette;

int Current_Sequence = SEQUENCE_START;
int Doing_Fade = TRUE;

BOOL Logo_Clouds = FALSE;
int Pos_Clouds_F = 1;
int Pos_Clouds_B = 2;
int Temp_Clouds = 1;

int Allow_Move;
int Fade_Speed;

LPBITMAP Pic_Logo;
LPBITMAP Pic_Font;
LPBITMAP Pic_Menus;
LPBITMAP Pic_Monsters_32;
LPBITMAP Pic_Monsters_48;
LPBITMAP Pic_Monsters_64;
LPBITMAP Pic_Monsters_80;
LPBITMAP Pic_Party;
LPBITMAP Pic_DemonHead;
LPBITMAP Pic_Towns;
LPBITMAP Pic_Tiles;

BYTE *RawPic_Clouds;

LPPOINTER Pointer;

LPFONT Font;

LPMENU Menu_Main;

extern LPBITMAP BitMap_Ptr_Front;
extern LPBITMAP BitMap_Ptr_Back;

/* --------------------------------
   VBlank interrupt
   -------------------------------- */
void VBlank_Interrupt()
{
    struct BitMap *BitM;
    BYTE *Src;
    UBYTE *Dst;

    if(Logo_Clouds)
    {
        Temp_Clouds--;
        if(!Temp_Clouds)
        {
            /* Display on both buffers with a small shift to simulate interlace */
            BitM = &BitMap_Ptr_Front->Bmp;

            Dst = BitM->Planes[7] + (BitM->BytesPerRow * 39) + (BitM->BytesPerRow * (Pic_Logo->Height + 1));
            Src = RawPic_Clouds + (Pos_Clouds_F * 40);
    
            asm("\tmovem.l d0-a5,-(a7)\n"
                "\tmove.l %[src],a0\n"
                "\tmove.l %[dst],a1\n"
                "\tmove.w #18-1,d7\n"
            "copy_clouds:\n"
                "\tmovem.l (a0)+,d0-d5/a2-a5\n"
                "\tmovem.l d0-d5/a2-a5,-(a1)\n"
                "\tmovem.l (a0)+,d0-d5/a2-a5\n"
                "\tmovem.l d0-d5/a2-a5,-(a1)\n"
                "\tmovem.l (a0)+,d0-d5/a2-a5\n"
                "\tmovem.l d0-d5/a2-a5,-(a1)\n"
                "\tmovem.l (a0)+,d0-d5/a2-a5\n"
                "\tmovem.l d0-d5/a2-a5,-(a1)\n"
                "\tdbf     d7,copy_clouds\n"   
                "\tmovem.l (a7)+,d0-a5\n"
                :
                : [dst] "m" (Dst), [src] "m" (Src)
               );

            BitM = &BitMap_Ptr_Back->Bmp;
            Dst = BitM->Planes[7] + (BitM->BytesPerRow * 39) + (BitM->BytesPerRow * (Pic_Logo->Height + 1));
            Src = RawPic_Clouds + (Pos_Clouds_B * 40);
    
            asm("\tmovem.l d0-a5,-(a7)\n"
                "\tmove.l %[src],a0\n"
                "\tmove.l %[dst],a1\n"
                "\tmove.w #18-1,d7\n"
            "copy_clouds2:\n"
                "\tmovem.l (a0)+,d0-d5/a2-a5\n"
                "\tmovem.l d0-d5/a2-a5,-(a1)\n"
                "\tmovem.l (a0)+,d0-d5/a2-a5\n"
                "\tmovem.l d0-d5/a2-a5,-(a1)\n"
                "\tmovem.l (a0)+,d0-d5/a2-a5\n"
                "\tmovem.l d0-d5/a2-a5,-(a1)\n"
                "\tmovem.l (a0)+,d0-d5/a2-a5\n"
                "\tmovem.l d0-d5/a2-a5,-(a1)\n"
                "\tdbf     d7,copy_clouds2\n"   
                "\tmovem.l (a7)+,d0-a5\n"
                :
                : [dst] "m" (Dst), [src] "m" (Src)
               );

            Pos_Clouds_F--;
            if(Pos_Clouds_F == 0) Pos_Clouds_F = 320;

            Pos_Clouds_B--;
            if(Pos_Clouds_B == 0) Pos_Clouds_B = 320;

            Temp_Clouds = 3;
        }
    }
}

/* --------------------------------------------
   Audio interrupt
   -------------------------------------------- */
/*void Audio_Interrupt()
{

}*/

/* --------------------------------
   Handle pad specifically for our game
   -------------------------------- */
ULONG Get_Pad_Infos(BYTE Flip_Flop, int Allow_Move)
{
    ULONG Pad_Dats;
    int Speed = 1;

    if(!Flip_Flop)
    {
        Pad_Dats = Get_Pad(8, 1);
    
        if(Pad_Dats & PAD_BUTTON_GREEN)
        {
            Speed = 10;
        }
        if(Pad_Dats & PAD_BUTTON_DOWN)
        {
            if(Get_Focus_Menu())
            {
                Set_Selected_Menu_Line(Get_Focus_Menu(),
                                       Get_Selected_Menu_Line(Get_Focus_Menu()) + Speed, Speed > 1 ? FALSE: TRUE);
            }
        }
        if(Pad_Dats & PAD_BUTTON_UP)
        {
            if(Get_Focus_Menu())
            {
                Set_Selected_Menu_Line(Get_Focus_Menu(),
                                       Get_Selected_Menu_Line(Get_Focus_Menu()) - Speed, Speed > 1 ? FALSE: TRUE);
            }
        }
        if(Pad_Dats & PAD_BUTTON_RIGHT)
        {
            if(Get_Focus_Menu())
            {
                if(Speed == 10)
                {
                    Set_Selected_Menu_Column(Get_Focus_Menu(),
                                             Get_Menu_Columns(Get_Focus_Menu()) - 1);
                }
                else
                {
                    Set_Selected_Menu_Column(Get_Focus_Menu(),
                                             Get_Selected_Menu_Column(Get_Focus_Menu()) + 1);
                }
            }
            return PAD_RIGHT | (Pad_Dats & PAD_BUTTON_GREEN ? PAD_SPEED : 0);
        }
        if(Pad_Dats & PAD_BUTTON_LEFT)
        {
            if(Get_Focus_Menu())
            {
                if(Speed == 10)
                {
                    Set_Selected_Menu_Column(Get_Focus_Menu(), 0);
                }
                else
                {
                    Set_Selected_Menu_Column(Get_Focus_Menu(),
                                             Get_Selected_Menu_Column(Get_Focus_Menu()) - 1);
                }
            }
            return PAD_LEFT | (Pad_Dats & PAD_BUTTON_GREEN ? PAD_SPEED : 0);
        }
    
        if(Pad_Dats & PAD_BUTTON_RED)
        {
            return PAD_OK;
        }
        if(Pad_Dats & PAD_BUTTON_BLUE)
        {
            return PAD_CANCEL;
        }
        if(Pad_Dats & PAD_BUTTON_YELLOW)
        {
            return PAD_PANIC;
        }
        if(Pad_Dats & PAD_BUTTON_FORWARD)
        {
            return PAD_NEXT;
        }
        if(Pad_Dats & PAD_BUTTON_REVERSE)
        {
            return PAD_PREVIOUS;
        }
    }
    return PAD_NOP;
}

/* --------------------------------
   Program initializations
   -------------------------------- */
#ifdef LUA
BYTE Init(lua_State *Lua)
#else
BYTE Init()
#endif
{
    int i;

    /* Load the various pictures we'll need */
    Pic_Logo = Load_Picture("gfx/logo.iff", TRUE, Title_Palette, SCREEN_COLORS);
    if(Pic_Logo == NULL) return FALSE;
    Pic_Font = Load_Picture("gfx/font.iff", TRUE, NULL, 0);
    if(Pic_Font == NULL) return FALSE;
    Pic_Menus = Load_Picture("gfx/menus.iff", TRUE, Menus_Palette, SCREEN_COLORS);
    if(Pic_Menus == NULL) return FALSE;
    Pic_Towns = Load_Picture("gfx/town.iff", TRUE, Towns_Palette, SCREEN_COLORS);
    if(Pic_Towns == NULL) return FALSE;
    Pic_Monsters_32 = Load_Picture("gfx/mstr_32_39.iff", FALSE, NULL, 0);
    if(Pic_Monsters_32 == NULL) return FALSE;

    Pic_Tiles = Load_Picture("gfx/tiles.iff", TRUE, Tiles_Palette, SCREEN_COLORS);
    if(Pic_Tiles == NULL) return FALSE;
    
    if(Init_Menu_Context(Pic_Menus) == FALSE) return FALSE;

    //Pic_Monsters_48 = Load_Picture("gfx/mstr_48_31.iff", TRUE, NULL, 0);
    //if(Pic_Monsters_48 == NULL) return FALSE;
    //Pic_Monsters_64 = Load_Picture("gfx/mstr_64_39.iff", TRUE, NULL, 0);
    //if(Pic_Monsters_64 == NULL) return FALSE;
    //Pic_Monsters_80 = Load_Picture("gfx/mstr_80_79.iff", TRUE, NULL, 0);
    //if(Pic_Monsters_80 == NULL) return FALSE;
    
    Pic_Party = Load_Picture("gfx/party_32_32.iff", TRUE, Party_Palette, SCREEN_COLORS);
    if(Pic_Party == NULL) return FALSE;
    Pic_DemonHead = Load_Picture("gfx/demonhead.iff", TRUE, NULL, 0);
    if(Pic_DemonHead == NULL) return FALSE;

    RawPic_Clouds = (BYTE *) Load_File("gfx/clouds_320x640.raw");
    if(RawPic_Clouds == NULL) return FALSE;

    if(Load_Towns() == FALSE) return FALSE;

    /* Create the font and the hand pointer */
    Font = Create_Font(Pic_Font, NULL, 0, 0, 8, 8);

    Pointer = Create_Pointer(16, 16, Pic_Menus);

    /* Init the colors of the clouds behind the logo */
    for(i = 0; i < LOGO_COLORS; i++)
    {
        Title_Palette[i + 128] = Title_Palette[i];
    }
    Title_Palette[128] = 0xe0f0fa;

    /* Init the first palette
      (black with menus colors available in case we need to display any NVRAM warning) */
    /* Copy the palette of the menus and pointer at the end of the title palette */
    for(i = 239; i < 239 + MENUS_COLORS; i++)
    {
        Title_Palette[i] = Menus_Palette[i];
        Current_Palette[i] = Menus_Palette[i];
    }

    Current_Palette[237] = 0x706a60;
    Current_Palette[238] = 0xc5c0ba;
    Current_Palette[254] = 0xa5a09a;
    Current_Palette[255] = 0;

#if SEQUENCE_START == SEQUENCE_TOWN
    Set_Palette(Black_Palette, 0, SCREEN_COLORS);
    Set_Fade(Current_Palette, 16, MOVE_HORIZONTALLY | MOVE_VERTICALLY);
    if(!Init_Town(1)) return FALSE;
    Destroy_Bitmap(Pic_Logo);
    Destroy_File(RawPic_Clouds);
#endif

    Title_Palette[239] = 0xd1c5be;
    Title_Palette[240] = 0x908070;
    Title_Palette[254] = 0xa5a09a;
    Title_Palette[255] = 0;

    return TRUE;
}

/* --------------------------------
   Set the current palette fading behaviour
   -------------------------------- */
void Set_Fade(ULONG *Palette, int Speed, int PadMove)
{
    Ptr_Palette = Palette;
    Calc_Fade_Palette(Palette, 0, SCREEN_COLORS);
    Doing_Fade = TRUE;
    Fade_Speed = Speed;
    Set_Pad_Move(PadMove);
}

/* --------------------------------
   Release allocated resources
   -------------------------------- */
void Release()
{
    Destroy_Menu(Menu_Ask_Saving);

    Destroy_Sheet();
    Destroy_Town();

    Destroy_File(RawPic_Clouds);

    Destroy_Pointer(Pointer);
    Destroy_Menu(Menu_Main);

    Destroy_Font(Font);

    Destroy_Bitmap(Pic_DemonHead);
    Destroy_Bitmap(Pic_Party);
    Destroy_Bitmap(Pic_Monsters_80);
    Destroy_Bitmap(Pic_Monsters_64);
    Destroy_Bitmap(Pic_Monsters_48);
    Destroy_Bitmap(Pic_Monsters_32);

    Destroy_Menu_Context();

    Destroy_Bitmap(Pic_Tiles);
    Destroy_Bitmap(Pic_Towns);
    Destroy_Bitmap(Pic_Menus);
    Destroy_Bitmap(Pic_Font);
    Destroy_Bitmap(Pic_Logo);
}

/* --------------------------------
   Main program loop
   -------------------------------- */
ULONG Loop(BYTE Flip_Flop)
{
    ULONG Pad_Dats;
    int Default_Line;

    Pad_Dats = Get_Pad_Infos(Flip_Flop, Allow_Move);

    switch(Current_Sequence)
    {
        case SEQUENCE_SYS_CHECK:

            Check_Save_Game();

            Set_Palette(Black_Palette, 0, SCREEN_COLORS);
            Print(Font, MENU_TEXT_SECONDARY_FONT, -1, 120, SCREEN_WIDTH, TRUE, PRINT_BACKGROUND, "%s", "BASED UPON THE WORK OF");
            Print(Font, MENU_TEXT_SECONDARY_FONT, -1, 128, SCREEN_WIDTH, TRUE, PRINT_BACKGROUND, "%s", "WINSTON DOUGLAS WOOD");
            Set_Fade(Current_Palette, 2, MOVE_NONE);
            Current_Sequence++;
            break;

        case SEQUENCE_INTRO:

            if(Doing_Fade)
            {
                Doing_Fade = Fade_Main_Palette(Ptr_Palette, 0, SCREEN_COLORS, Fade_Speed);
            }

            /* Fade out */
            if(Pad_Dats & PAD_OK)
            {
                Set_Fade(Black_Palette, 16, MOVE_NONE);
            }

            /* End of intro */
            if(Doing_Fade == FALSE)
            {
                if(Ptr_Palette == Current_Palette)
                {
                    Set_Fade(Black_Palette, 1, MOVE_NONE);
                    break;
                }
                else if(Ptr_Palette == Black_Palette)
                {
                    /* Create and start the main menu */
                    if(Can_Save && Saved_Game)
                    {
                        Menu_Main = Construct_Menu(Font,
                                                   Pic_Menus,
                                                   NULL,
                                                   MENU_TEXT_CENTER,
                                                   5, 0,
                                                   1, 0,
                                                   MOVE_VERTICALLY,
                                                   "%s\n%s",
                                                   "START NEW GAME",
                                                   "CONTINUE SAVED GAME"
                                                  );
                        Default_Line = 1;
                    }
                    else
                    {
                        Menu_Main = Construct_Menu(Font,
                                                   Pic_Menus,
                                                   NULL,
                                                   MENU_TEXT_CENTER,
                                                   5, 0,
                                                   1, 0,
                                                   MOVE_VERTICALLY,
                                                   "%s",
                                                   "START NEW GAME"
                                                  );
                        Default_Line = 0;
                    }
                    Clear_Screen(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
                    Static_To_Screen(Pic_Logo, 0, 0, 0, 40, Pic_Logo->Width, Pic_Logo->Height);
                    Print(Font, 0, -1, 118, SCREEN_WIDTH, TRUE, PRINT_BACKGROUND, "%s", "STORY OF FERRONRAH");
                    Print(Font, 0, -1, 245, SCREEN_WIDTH, TRUE, PRINT_BACKGROUND, "%s", "WRITTEN BY FRANCK CHARLET");
                    Show_Menu(Menu_Main, Pointer, -1, 170, 0);
                    Set_Selected_Menu_Line(Menu_Main, Default_Line, FALSE);
                    Logo_Clouds = TRUE;

                    CD_Play_Track(TRACK_TITLE);

                    Set_Fade(Title_Palette, 3, MOVE_HORIZONTALLY | MOVE_VERTICALLY);
                    Current_Sequence++;
                }
            }
            break;

        case SEQUENCE_MENU_MAIN:

            if(Doing_Fade)
            {
                Doing_Fade = Fade_Main_Palette(Ptr_Palette, 0, SCREEN_COLORS, Fade_Speed);
            }

            Erase_Pointer(Pointer);
            Refresh_Menu(Get_Focus_Menu());
            Display_Pointer(Pointer, Get_Focus_Menu(), Get_Selected_Menu_Line(Get_Focus_Menu()), 36, 9);
            
            /* Fade out */
            if(Pad_Dats & PAD_OK)
            {
                //Ptr_Palette = Black_Palette;
                Set_Fade(Black_Palette, 16, MOVE_NONE);
            }

            if(Doing_Fade == FALSE &&
               Ptr_Palette == Black_Palette)
            {
                switch(Get_Selected_Menu_Line(Get_Focus_Menu()))
                {
                    case MENU_MAIN_NEW_GAME:

                        Logo_Clouds = FALSE;
                        Hide_Menu(Get_Focus_Menu(), Pointer);
                        Clear_Screen(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
                        Destroy_Menu(Menu_Main);
                        Destroy_Bitmap(Pic_Logo);
                        Destroy_File(RawPic_Clouds);
                        Current_Sequence = SEQUENCE_TOWN;
                        Clear_Party_Data();
                        CD_Stop_Track();
                        if(!Init_Town(1)) return TRUE;
                        break;

                    case MENU_MAIN_RESTORE_GAME:

                        Logo_Clouds = FALSE;
                        Hide_Menu(Get_Focus_Menu(), Pointer);
                        Clear_Screen(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
                        Destroy_Menu(Menu_Main);
                        Destroy_Bitmap(Pic_Logo);
                        Destroy_File(RawPic_Clouds);
                        CD_Stop_Track();
                        Current_Sequence = SEQUENCE_RESTORE;
                        break;
                }
            }
            break;

        case SEQUENCE_TOWN:

            if(Doing_Fade)
            {
                Doing_Fade = Fade_Main_Palette(Ptr_Palette, 0, SCREEN_COLORS, Fade_Speed);
            }
            Loop_Town(Pad_Dats);
            break;

        case SEQUENCE_SCROLL:

            if(Doing_Fade)
            {
                Doing_Fade = Fade_Main_Palette(Ptr_Palette, 0, SCREEN_COLORS, Fade_Speed);
            }
            Loop_Scroll(Pad_Dats);
            break;

        case SEQUENCE_RESTORE:

            if(!Restore_Game())
            {
                /* Failed: return to title sequence */
                Pic_Logo = Load_Picture("gfx/logo.iff", TRUE, NULL, 0);
                if(!Pic_Logo) exit(EXIT_FAILURE);
                RawPic_Clouds = (BYTE *) Load_File("gfx/clouds_320x640.raw");
                if(!RawPic_Clouds) exit(EXIT_FAILURE);
                Current_Sequence = SEQUENCE_INTRO;
            }
            break;

        case SEQUENCE_WILDERNESS:

            // TODO: display wilderness part

            break;

        case SEQUENCE_DUNGEON:

            // TODO: display dungeon part

            break;

        case SEQUENCE_GRAVEYARD:

            // TODO: display graveyard sequence

            break;

        case SEQUENCE_END:

            // TODO: display end of game sequence

            break;

    }

    return(Pad_Dats & PAD_PANIC);
}
