/* --------------------------------
   Phantasie II CD32
   CD drive support code
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

#ifdef CD32

/* --------------------------------
   Includes
   -------------------------------- */
#include "startup.h"

/* --------------------------------
   Variables
   -------------------------------- */
STATIC struct MsgPort *CDPort;
STATIC struct IOStdReq *CDDeviceMReq;
STATIC struct Device *CDDevice;
int Current_Track;
struct CDInfo CDInfo;
struct TagItem XLConfigList[] =
{
    { TAGCD_READXLSPEED, 150 },
    { TAG_END, 0 }
};
struct TagItem ConfigList[] =
{
    { TAGCD_READSPEED, 0},
    { TAG_END, 0 }
};

/* --------------------------------
   Double the speed of the cd data reading
   -------------------------------- */
void CD_Set_Double_Speed()
{
    if(CDDeviceMReq)
    {
        // Get the max. speed
        CDDeviceMReq->io_Command = CD_INFO;
        CDDeviceMReq->io_Data = (APTR) &CDInfo;
        CDDeviceMReq->io_Offset = 0;
        CDDeviceMReq->io_Length = sizeof(struct CDInfo);
        DoIO((struct IORequest *) CDDeviceMReq);
        if(!CDDeviceMReq->io_Error)
        {
            // Set it
            ConfigList[0].ti_Data = CDInfo.MaxSpeed;
            CDDeviceMReq->io_Command = CD_CONFIG;
            CDDeviceMReq->io_Data = (APTR) &ConfigList;
            CDDeviceMReq->io_Length = 0;
            DoIO((struct IORequest *) CDDeviceMReq);
    
            CDDeviceMReq->io_Command = CD_CONFIG;
            CDDeviceMReq->io_Data = (APTR) &XLConfigList;
            CDDeviceMReq->io_Length = 0;
            DoIO((struct IORequest *) CDDeviceMReq);
        }
    }
}

/* --------------------------------
   Open the cd device driver
   -------------------------------- */
int CD_Device_Init()
{
	if((CDPort = CreateMsgPort()))
	{
	    if((CDDeviceMReq = (struct IOStdReq *) CreateIORequest(CDPort, sizeof(struct IOStdReq))))
	    {
		    if(!OpenDevice("cd.device", 0, (struct IORequest *) CDDeviceMReq, 0))
		    {
		        CDDevice = CDDeviceMReq->io_Device;
                CD_Set_Double_Speed();
                CD_Set_Volume(0x7fff);
		    }
	    }
	}

	if(!CDDevice)
	{
	    return(FALSE);
	}
    return(TRUE);
}

/* --------------------------------
   Close the cd device driver
   -------------------------------- */
void CD_Device_Term()
{
    if(CDDeviceMReq)
    {
	    if(CDDevice)
	    {
            CD_Stop_Track();
	        CloseDevice((struct IORequest *) CDDeviceMReq);
	        CDDevice = NULL;
	    }
	    DeleteIORequest(CDDeviceMReq);
	    CDDeviceMReq = NULL;
    }

    if(CDPort)
    {
	    DeleteMsgPort(CDPort);
	    CDPort = NULL;
    }
}

/* --------------------------------
   Play a given audio track
   -------------------------------- */
void CD_Play_Track(int Track_Number)
{
    if(CDDevice)
    {
        CDDeviceMReq->io_Command = CD_PLAYTRACK;
        CDDeviceMReq->io_Offset = Track_Number;
        Current_Track = Track_Number;
        CDDeviceMReq->io_Length = 1;
        CDDeviceMReq->io_Data = NULL;
        SendIO((struct IORequest *) CDDeviceMReq);
    }
}

/* --------------------------------
   Loop a currently playing audio track
   -------------------------------- */
void CD_Loop_Track(void)
{
    if(Current_Track && CDDevice)
    {
        CDDeviceMReq->io_Command = CD_PLAYTRACK;
        CDDeviceMReq->io_Offset = Current_Track;
        CDDeviceMReq->io_Length = 1;
        CDDeviceMReq->io_Data = NULL;
        if(CheckIO((struct IORequest *) CDDeviceMReq))
        {
            /* Restart track */
            CD_Play_Track(Current_Track);
        }
    }
}

/* --------------------------------
   Stop the currently playing audio track
   -------------------------------- */
void CD_Stop_Track(void)
{
    if(Current_Track && CDDevice)
    {
        CDDeviceMReq->io_Command = CD_PLAYTRACK;
        CDDeviceMReq->io_Offset = Current_Track;
        CDDeviceMReq->io_Length = 1;
        CDDeviceMReq->io_Data = NULL;
        AbortIO((struct IORequest *) CDDeviceMReq);
        WaitIO((struct IORequest *) CDDeviceMReq);
        Current_Track = 0;
    }
}

/* --------------------------------
   Set the volume of an audio track [0..0x7fff]
   -------------------------------- */
void CD_Set_Volume(int Volume)
{
    if(CDDevice)
    {
        CDDeviceMReq->io_Command = CD_ATTENUATE;
        CDDeviceMReq->io_Offset = Volume;
        CDDeviceMReq->io_Length = 0;
        CDDeviceMReq->io_Data = NULL;
        SendIO((struct IORequest *) CDDeviceMReq);
        WaitIO((struct IORequest *) CDDeviceMReq);
    }
}

#endif
