/* --------------------------------
   Phantasie II CD32
   Files support code
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* --------------------------------
   Includes
   -------------------------------- */
#include "startup.h"

/* --------------------------------
   Open a file for reading or writing
   -------------------------------- */
BPTR F_Open(char *FileName, LONG Mode)
{
    Permit();
    return Open((STRPTR) FileName, Mode);
}

/* --------------------------------
   Return the size of an opened file
   Note: we can't use any other method otherwise
         the ISO9660 file system will pull a 209
   -------------------------------- */
int F_Size(BPTR File)
{
    Seek(File, 0, OFFSET_END);
    return Seek(File, 0, OFFSET_BEGINNING);
}

/* --------------------------------
   Read a given amount of bytes from a file
   -------------------------------- */
int F_Read(BPTR File, void *Block, int Size)
{
    return Read(File, (APTR) Block, Size);
}

/* --------------------------------
   Write a given amount of bytes to a file
   -------------------------------- */
int F_Write(BPTR File, void *Block, int Size)
{
    return Write(File, (CONST APTR) Block, Size);
}

/* --------------------------------
   Close an opened file
   -------------------------------- */
void F_Close(BPTR File)
{
    if(File) Close(File);
    Forbid();
}

/* --------------------------------
   Load a file into Memory
   -------------------------------- */
void *Load_File(char *FileName)
{
    BPTR Handle;
    void *Memory;
    int Size = -1;
    int FileSize;

    Handle = F_Open(FileName, MODE_OLDFILE);
    if(Handle)
    {
        Size = F_Size(Handle);
        if(Size)
        {
            Memory = Mem_Alloc(Size);
            if(Memory)
            {
                FileSize = F_Read(Handle, Memory, Size);
                if(FileSize == Size)
                {
                    F_Close(Handle);
                    return Memory;
                }
                Printf("Size error: %ld/%ld ", FileSize, Size);
                Mem_Free(Memory);
                Memory = NULL;
            }
            else
            {
                Printf("Not enough memory");
            }
        }
        F_Close(Handle);
    }
    Printf("Couldn't open file: %s (Size: %ld) (DOS error %ld)\n", FileName, Size, IoErr());
    return NULL;        
}

/* --------------------------------
   Free the memory taken by a file
   -------------------------------- */
void _Destroy_File(void *Memory)
{
    Mem_Free(Memory);
}
