/* --------------------------------
   Phantasie II CD32
   Gfx support code
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

#ifndef _GFX_H_
#define _GFX_H_

/* --------------------------------
   Constants
   -------------------------------- */
#define ID_ILBM MAKE_ID('I', 'L', 'B', 'M')
#define ID_BMHD MAKE_ID('B', 'M', 'H', 'D')
#define ID_CMAP MAKE_ID('C', 'M', 'A', 'P')
#define ID_BODY MAKE_ID('B', 'O', 'D', 'Y')

#define cmpNone 0
#define cmpByteRun1 1

#define RowBytes(w) ((((w) + 15) >> 4) << 1)

#define PRINT_BACKGROUND ABC

/* --------------------------------
   Structures
   -------------------------------- */
typedef struct __attribute__((packed))
{
    WORD Width;
    WORD Height;
    WORD Depth;
    struct BitMap Bmp;
    struct BitMap *Mask;
    WORD Colors;
} BITMAP, *LPBITMAP;

typedef struct __attribute__((packed))
{
    WORD Width;
    WORD Height;
    WORD Old_XA;
    WORD Old_YA;
    WORD Old_XB;
    WORD Old_YB;
    LPBITMAP Bank;
    LPBITMAP BackA;
    LPBITMAP BackB;
    WORD Saved;
} OBJECT, *LPOBJECT;

typedef struct __attribute__((packed))
{
    UBYTE *Sizes;
    WORD Width;
    WORD Height;
    LPBITMAP Bank;
    WORD Src_X;
    WORD Src_Y;
} FONT, *LPFONT;

typedef struct __attribute__((packed))
{
    UWORD w;
    UWORD h;
    WORD x;
    WORD y;
    UBYTE nPlanes;
    UBYTE masking;
    UBYTE compression;
    UBYTE reserved1;
    UWORD transparentColor;
    UBYTE xAspect;
    UBYTE yAspect;
    WORD pageWidth;
    WORD pageHeight;
} IFFHEADER, *LPIFFHEADER;

typedef struct __attribute__((packed))
{
    UBYTE R;
    UBYTE G;
    UBYTE B;
} IFFRGB, *LPIFFRGB;

typedef struct __attribute__((packed))
{
    IFFRGB Palette[256];
} IFFPALETTE, *LPIFFPALETTE;

typedef struct __attribute__((packed))
{
    ULONG R;
    ULONG G;
    ULONG B;
} RGB32, *LPRGB32;

typedef struct __attribute__((packed))
{
    ULONG Amount;
    RGB32 Palette[257];
} PALETTE32, *LPPALETTE32;

/* --------------------------------
   Functions
   -------------------------------- */
int Create_Bitmap_Mask(LPBITMAP BitMap);
struct BitMap *Get_Screen_BitMap();
void Set_Palette(ULONG *Palette,
                 WORD Start,
                 WORD Amount);
void Calc_Fade_Palette(ULONG *Palette,
                       WORD Start,
                       WORD Amount
                       /*,int Speed*/);
BOOL Fade_Main_Palette(ULONG *Palette,
                       WORD Start,
                       WORD Amount,
                       int Speed);
void Fade_Palette(ULONG *Palette_Src,
                  ULONG *Palette_Dst,
                  WORD Start,
                  WORD Amount,
                  int Speed);
LPBITMAP Create_Bitmap(short Width,
                       short Height,
                       short Depth,
                       BOOL Clear,
                       struct BitMap *Mask);
#define Destroy_Bitmap(x) _Destroy_Bitmap(x); x = NULL;
void _Destroy_Bitmap(LPBITMAP Bitm);
void Draw_Line(short X1,
               short Y1,
               short X2,
               short Y2,
               UBYTE Color);
void Clear_Array(short X,
                 short Y,
                 short Width,
                 short Height);
void Fill_Array(short X,
                short Y,
                short Width,
                short Height);
void Clear_Screen(WORD X,
                  WORD Y,
                  WORD Width,
                  WORD Height);
void Copy_To_Screen(LPBITMAP Src,
                    short Src_X,
                    short Src_Y,
                    short X,
                    short Y,
                    short Width,
                    short Height);
void Static_To_Screen(LPBITMAP Src,
                      WORD Src_X,
                      WORD Src_Y,
                      WORD Dst_X,
                      WORD Dst_Y,
                      WORD Width,
                      WORD Height);
void Copy_To_Bitmap(LPBITMAP Src,
                    WORD Src_X,
                    WORD Src_Y,
                    LPBITMAP Dst,
                    WORD Dst_X,
                    WORD Dst_Y,
                    WORD Width,
                    WORD Height,
                    UBYTE Minterms);
void Scroll(short X,
            short Y,
            short Width,
            short Height,
            short Dx,
            short Dy);
void Move_Screen(short X, short Y);
void Lock_Blitter();
void Unlock_Blitter();
LPFONT Create_Font(LPBITMAP Source,
                   UBYTE *Sizes,
                   WORD Src_X,
                   WORD Src_Y,
                   WORD Width,
                   WORD Height);
#define Destroy_Font(x) _Destroy_Font(x); x = NULL;
void _Destroy_Font(LPFONT Font);
int Get_Text_Length(LPFONT Font,
                    char *Text,
                    int Len,
                    int *Size,
                    int *Chars);
void Print(LPFONT Font,
           int Flags,
           WORD Pos_X,
           WORD Pos_Y,
           WORD Max_X,
           int Static,
           UBYTE Minterms,
           char *Format,
           ...);
LPBITMAP Load_Picture(char *FileName,
                      BOOL Create_Mask,
                      ULONG *Palette,
                      int Amount_Colors);
LPOBJECT Create_Object(WORD Width,
                       WORD Height,
                       LPBITMAP Bitmap_Bank);
void Set_Object_Bank(LPOBJECT Obj,
                     LPBITMAP Bitmap_Bank);
#define Destroy_Object(x) _Destroy_Object(x); x = NULL;
void _Destroy_Object(LPOBJECT Obj);
void Erase_Object(LPOBJECT Obj);
void Erase_Objects(LPOBJECT Objs,
                   short Amount);
void Display_Object(LPOBJECT Obj,
                    WORD Pos_X,
                    WORD Pos_Y,
                    WORD Src_X,
                    WORD Src_Y);
void Copy_From_Back(WORD X,
                    WORD Y,
                    WORD Width,
                    WORD Height);

#endif
