/* --------------------------------
   Phantasie II CD32
   Menus support code
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* --------------------------------
   Includes
   -------------------------------- */
#include "startup.h"
#include "gfx.h"

/* --------------------------------
   Constants
   -------------------------------- */
#define FOCUS_STACK_SIZE 64

/* --------------------------------
   Variables
   -------------------------------- */
extern LPBITMAP BitMap_Ptr_Front;
extern LPBITMAP BitMap_Ptr_Back;
LPBITMAP BitMap_Center;

LPMENU Focus_Stack[FOCUS_STACK_SIZE];
int Focus_Stack_Pos = -1;

/* --------------------------------
   Functions
   -------------------------------- */
void Display_Arrows(LPMENU Menu, int Top_Line);
void Push_Focus_Menu(LPMENU Menu);
void Pop_Focus_Menu();

/* --------------------------------
   Create the bitmaps used by the menus
   -------------------------------- */
BOOL Init_Menu_Context(LPBITMAP Pic)
{
    int i;
    int j;
    int Tiles_X;
    int Tiles_Y;

    Tiles_X = (SCREEN_WIDTH / 8) - 2;
    Tiles_Y = (SCREEN_HEIGHT / 9) - 2;

    BitMap_Center = Create_Bitmap(Tiles_X * 8,
                                  Tiles_Y * 9,
                                  SCREEN_BPP,
                                  TRUE,
                                  NULL);

    if(BitMap_Center)
    {
        // Create a background for clearing purposes
        for(j = 0; j < Tiles_Y; j++)
        {
            for(i = 0; i < Tiles_X; i++)
            {
                Copy_To_Bitmap(Pic, 4 * 9, 0, BitMap_Center, i * 8, j * 9, 8, 9, ABC | ABNC);
            }
        }
        return TRUE;
    }
    return FALSE;
}

/* --------------------------------
   Create the bitmaps used by the menus
   -------------------------------- */
void Destroy_Menu_Context()
{
    Destroy_Bitmap(BitMap_Center);
}

/* --------------------------------
   Quickly print a (converted) text on the screen
   -------------------------------- */
void FastPrint(LPMENU Menu,
               WORD Pos_X,
               WORD Pos_Y,
               char *String,
               int Len)
{
    int i;
    int Car;
    int Font_Offset;
    LPFONT Font;

    i = 0;

    Font = Menu->Font;
    Font_Offset = Font->Src_Y + (Menu->Flags & MENU_TEXT_SECONDARY_FONT ? Font->Height : 0);

    /* Display all chars */
    while(i < Len)
    {
        Car = String[i];
        Copy_To_Screen(Font->Bank,
                       Font->Src_X + (Car * Font->Width),
                       Font_Offset,
                       Pos_X,
                       Pos_Y,
                       Font->Width,
                       Font->Height);
        /* Next pos */
/*        if(Font->Sizes)
        {
            // Proportional size
            Pos_X += Font->Sizes[Car];
        }
        else
        {*/
            // Fixed size
            Pos_X += Font->Width;
        //}
        i++;
    }
}

/* --------------------------------
   Remove the menu from the screen
   This is here because of a bug in GCC
   -------------------------------- */
void Hide_Menu(LPMENU Menu, LPPOINTER Pointer)
{
    if(Menu)
    {
        if(Pointer)
        {
            Erase_Pointer(Pointer);
        }
        Copy_To_Bitmap(Menu->Bitmap_Save,
                       0,
                       0,
                       BitMap_Ptr_Front,
                       Menu->X,
                       Menu->Y,
                       Menu->Width,
                       Menu->Height,
                       ABC | ABNC
                      );
        Swap_Physical_Buffers();
        if(Pointer)
        {
            Erase_Pointer(Pointer);
        }
        Copy_To_Bitmap(Menu->Bitmap_Save,
                       0,
                       0,
                       BitMap_Ptr_Front,
                       Menu->X,
                       Menu->Y,
                       Menu->Width,
                       Menu->Height,
                       ABC | ABNC
                      );
        Pop_Focus_Menu();
        if(Pointer)
        {
            Display_Pointer(Pointer, Get_Focus_Menu(), Get_Selected_Menu_Line(Get_Focus_Menu()), 36, 9);
            Swap_Physical_Buffers();
            Display_Pointer(Pointer, Get_Focus_Menu(), Get_Selected_Menu_Line(Get_Focus_Menu()), 36, 9);
        }
        Set_Pad_Move(Get_Focus_Menu()->Movements);
    }
}

/* --------------------------------
   Free a menu object
   -------------------------------- */
void _Destroy_Menu(LPMENU Menu)
{
    if(Menu)
    {
        Destroy_Bitmap(Menu->Bitmap_Save);
        Destroy_Bitmap(Menu->Bitmap);
        Mem_Free(Menu->Text);
        Mem_Free(Menu);
    }
}

/* --------------------------------
   Construct a menu object
   -------------------------------- */
LPMENU Construct_Menu(LPFONT Font,
                      LPBITMAP Pic,
                      void (*Events)(void *Self, int Item, ULONG Joypad_Dats),
                      int Flags,
                      int Max_Lines,                /* Maximum number of lines to display */
                      int Min_Sel,                  /* Top line for selection */
                      int Max_Columns,              /* Number of columns */
                      int Columns_Size,             /* Size of the columns in chars */
                      int Allow_Movements,          /* Pad movements allowed when this menu is active */
                      char *Format,
                      ...)
{
    char szBuffer[2048];
    TXTLINE Lines_Offsets[512];
    va_list Arg;
    int Len;
    int i;
    int j;
    /*int More;*/
    LPMENU Menu;
    int Line_Size_X;
    int Size_X;
    int Size_Y;
    int Lines;
    int Size_Chars;

    Menu = NULL;
    Size_X = 0;
    Size_Y = 0;
    Lines = 0;
    /*More = TRUE;*/

    va_start(Arg, Format);
        Len = vsprintf(szBuffer, Format, (char *) Arg);

        /* First we need to determine the size of the menu in X & Y */
        i = 0;
        j = 0;
        while(j < Len)
        {
            i = Get_Text_Length(Font, &szBuffer[j], Len - j, &Line_Size_X, &Size_Chars);
            if(Line_Size_X > Size_X)
            {
                Size_X = Line_Size_X;
            }
            if(Line_Size_X)
            {
                Size_Y += Font->Height + 1;
                /* Store the starting character */
                Lines_Offsets[Lines].Pos = j;
                Lines_Offsets[Lines].Size = Line_Size_X;
                Lines_Offsets[Lines].Len = Size_Chars;
                Lines++;
            }
            if(i == 0) break;
            j += i;
            szBuffer[j - 1] = '\0';
        }

        Menu = (LPMENU) Mem_Alloc(sizeof(MENU));
        if(Menu)
        {
            /* Store the number of number menu pic parts */
            Menu->Font = Font;
            Menu->Tiles_X = (Size_X / 8) + ((Size_X % 8) ? 1 : 0) + 4;
            Menu->Tiles_Y = (Size_Y / 8) + ((Size_Y % 8) ? 1 : 0);
            Menu->Width = Menu->Tiles_X * 8;
            Menu->Lines = Lines;
            Max_Lines += Min_Sel;
            Menu->Max_Lines = Max_Lines;
            Menu->Pic = Pic;
            Menu->Max_Columns = Max_Columns;
            Menu->Columns_Size = (Columns_Size + 1) * Font->Width;
            Menu->Min_Line_Sel = Min_Sel;
            Menu->Events = Events;
            Menu->Flags = Flags;
            Menu->Movements = Allow_Movements;

            /* Make sure we don't display more than we should */
            if((Menu->Tiles_Y * 8) > ((Max_Lines * Font->Height + 1) + 16))
            {
                Menu->Tiles_Y = ((((Max_Lines * Font->Height + 1) + 16) / 8));
            }

            Menu->Tiles_Y += 2;
            Menu->Height = Menu->Tiles_Y * 8;
            memcpy(Menu->Lines_Offsets, Lines_Offsets, sizeof(Lines_Offsets));

            if(Flags & MENU_TEXT_LAST_LINE_CENTER)
            {
                for(i = 0; i < Lines - 1; i++)
                {
                    Menu->Lines_Offsets[i].Size = 16;
                }
                Menu->Lines_Offsets[i].Size = (Menu->Width - Menu->Lines_Offsets[i].Size) / 2;
            }
            else
            {
                for(i = 0; i < Lines; i++)
                {
                    if(Flags & MENU_TEXT_LEFT && (i >= Min_Sel))
                    {
                        Menu->Lines_Offsets[i].Size = 16;
                    }
                    else
                    {
                        Menu->Lines_Offsets[i].Size = (Menu->Width - Menu->Lines_Offsets[i].Size) / 2;
                    }
                }
            }

            Menu->Text = (char *) Mem_Alloc(Len + 1);
            if(!Menu->Text)
            {
                Destroy_Menu(Menu);
            }
            else
            {
                /* Store the text of the menu */
                for(i = 0; i < Len; i++)
                {
                    //for(j = 0; j < Font->ASCII_Len; j++)
                    //{
                      //  if(Font->ASCII[j] == )
                       // {
                            Menu->Text[i] = szBuffer[i] - 0x20;
                         //   break;
                        //}
                    //}
                }

                /* Precalculate the complete mmenu picture */
                Menu->Bitmap = Create_Bitmap(Menu->Width,
                                             Menu->Height,
                                             SCREEN_BPP,
                                             TRUE,
                                             NULL);
                if(!Menu->Bitmap)
                {
                    Destroy_Menu(Menu);
                }
                else
                {
                    Menu->Bitmap_Save = Create_Bitmap(Menu->Width,
                                                      Menu->Height,
                                                      SCREEN_BPP,
                                                      TRUE,
                                                      NULL);
                    if(!Menu->Bitmap_Save)
                    {
                        Destroy_Menu(Menu);
                    }
                    else
                    {
                        // Top border

                        // Top left Corner
                        Copy_To_Bitmap(Pic, 0, 0, Menu->Bitmap, 0, 0, 8, 8, ABC | ABNC);

                        for(i = 1; i < Menu->Tiles_X - 1; i++)
                        {
                            Copy_To_Bitmap(Pic, (rand() % 3) * 9, 1 * 9, Menu->Bitmap, i * 8, 0, 8, 8, ABC | ABNC);
                        }

                        // Right top corner
                        Copy_To_Bitmap(Pic, 1 * 9, 0, Menu->Bitmap, i * 8, 0, 8, 8, ABC | ABNC);
        
                        // Body
                        Copy_To_Bitmap(BitMap_Center, 0, 0, Menu->Bitmap, 1 * 8, 1 * 8, (Menu->Tiles_X - 2) * 8, (Menu->Tiles_Y - 2) * 8, ABC | ABNC);
                        // Left & right borders
                        for(j = 1; j < Menu->Tiles_Y - 1; j++)
                        {
                            Copy_To_Bitmap(Pic, (rand() % 3) * 9, 3 * 9, Menu->Bitmap, 0, j * 8, 8, 8, ABC | ABNC);
                            Copy_To_Bitmap(Pic, (rand() % 3) * 9, 4 * 9, Menu->Bitmap, i * 8, 8 * j, 8, 8, ABC | ABNC);
                        }
    
                        // Left bottom border
                        Copy_To_Bitmap(Pic, 3 * 9, 0, Menu->Bitmap, 0, j * 8, 8, 8, ABC | ABNC);

                        for(i = 1; i < Menu->Tiles_X - 1; i++)
                        {
                            Copy_To_Bitmap(Pic, (rand() % 3) * 9, 2 * 9, Menu->Bitmap, i * 8, j * 8, 8, 8, ABC | ABNC);
                        }

                        // Right bottom border
                        Copy_To_Bitmap(Pic, 2 * 9, 0, Menu->Bitmap, i * 8, j * 8, 8, 8, ABC | ABNC);
    
                        Menu->Text_Start_Y = (Menu->Height - (((Menu->Max_Lines < Lines ? Menu->Max_Lines : Lines) * (Font->Height + 1)) - 1)) / 2;
                        if((Menu->Height - (((Menu->Max_Lines < Lines ? Menu->Max_Lines : Lines) * (Font->Height + 1)) - 1)) % 2)
                        {
                            Menu->Text_Start_Y += 1;
                        }

                        // Create the mask for the whole menu
                        if(Create_Bitmap_Mask(Menu->Bitmap) == FALSE)
                        {
                            Destroy_Menu(Menu);
                        }
                    }
                }

            }
        }

    va_end(Arg);

    return Menu;
}

/* --------------------------------
   Display a menu on both screens
   -------------------------------- */
void Show_Menu(LPMENU Menu,
               LPPOINTER Ptr,
               WORD Pos_X,
               WORD Pos_Y,
               WORD Extra_X)
{
    int X;
    int Y;
    int i;
    int index;
    int Text_Y;

    X = Pos_X;
    Y = Pos_Y;
    if(X == -1)
    {
        X = (SCREEN_WIDTH - Menu->Width) / 2;
    }
    if(X == -2)
    {
        X = (SCREEN_WIDTH - Menu->Width) - 1;
    }
    if(Y == -1)
    {
        Y = (SCREEN_HEIGHT - Menu->Height) / 2;
    }
    if(Y == -2)
    {
        Y = (SCREEN_HEIGHT - Menu->Height) - 1;
    }
    Menu->X = X;
    Menu->Y = Y;
    /* Used as an offset for the pointer */
    Menu->Extra_X = Extra_X;
    Menu->Line_Top = 0;
    Menu->Line_Selected = Menu->Min_Line_Sel;
    Menu->Old_Line_Selected = Menu->Min_Line_Sel;
    Menu->Refresh = 0;

    if(Ptr)
    {
        Erase_Pointer(Ptr);
    }

    /* Save the background */
    Copy_To_Bitmap(BitMap_Ptr_Front,
                   Menu->X,
                   Menu->Y,
                   Menu->Bitmap_Save,
                   0,
                   0,
                   Menu->Width,
                   Menu->Height,
                   ABC | ABNC
                  );

    /* Display it on first screen */
    Copy_To_Screen(Menu->Bitmap,
                   0,
                   0,
                   X,
                   Y,
                   Menu->Width,
                   Menu->Height);

    Text_Y = Menu->Y + Menu->Text_Start_Y;
    for(i = 0; i < Menu->Max_Lines; i++)
    {
        index = (Menu->Line_Top + i);
        if(index >= Menu->Lines) break;
        FastPrint(Menu,
                  Menu->X + Menu->Lines_Offsets[index].Size,
                  Text_Y,
                  &Menu->Text[Menu->Lines_Offsets[index].Pos],
                  Menu->Lines_Offsets[index].Len
             );
        Text_Y += Menu->Font->Height + 1;
    }
    Display_Arrows(Menu, Menu->Line_Top);

    /* Display it on second screen */
    Swap_Physical_Buffers();

    if(Ptr)
    {
        Erase_Pointer(Ptr);
    }
    Copy_To_Bitmap(BitMap_Ptr_Back,
                   Menu->X,
                   Menu->Y,
                   BitMap_Ptr_Front,
                   Menu->X,
                   Menu->Y,
                   Menu->Width,
                   Menu->Height,
                   ABC | ABNC
                  );

    /* Make it the current menu */
    Push_Focus_Menu(Menu);
    if(Ptr)
    {
        if(!(Menu->Flags & MENU_NO_POINTER))
        {
            Display_Pointer(Ptr, Menu, Get_Selected_Menu_Line(Menu), 36, 9);
            Swap_Physical_Buffers();
            Display_Pointer(Ptr, Menu, Get_Selected_Menu_Line(Menu), 36, 9);
        }
    }
    Set_Pad_Move(Menu->Movements);
}

/* --------------------------------
   Move the displayed lines if necessary
   -------------------------------- */
void Refresh_Menu(LPMENU Menu)
{
    int i;
    int Y;
    int Min_Y;
    int index;
    int Delta;
    int Lines_To_Draw;
    int Selected;
    int Max;

    if(Menu->Old_Line_Selected != Menu->Line_Selected)
    {
        Selected = Menu->Line_Selected - Menu->Min_Line_Sel;
        Max = Menu->Max_Lines - Menu->Min_Line_Sel;

        /* Determine if the wanted line is already shown */
        if(Selected >= Menu->Line_Top &&
           Selected < Menu->Line_Top + Max)
        {
            /* We don't need to move the list only the pointer will */
            return;
        }

        if(Selected > Menu->Line_Top)
        {
            Delta = (Selected - Menu->Line_Top) - (Max - 1);
            Lines_To_Draw = Delta;
        }
        else
        {
            Delta = Selected - Menu->Line_Top;
            Lines_To_Draw = -Delta;
        }

        Min_Y = (Menu->Min_Line_Sel * (Menu->Font->Height + 1));

        if(Lines_To_Draw >= Max)
        {
            if(Delta > 0) Delta -= Menu->Min_Line_Sel;
            else Delta +=  Menu->Min_Line_Sel;

            /* A complete page redraw is required */
            
            Y = Menu->Y + Menu->Text_Start_Y + Min_Y;
            index = Selected;
            if(index > (Menu->Lines - Max)) index = Menu->Lines - Max;
            if(index < Menu->Min_Line_Sel) index = Menu->Min_Line_Sel;

            Copy_To_Screen(BitMap_Center,
                           0,
                           0,
                           Menu->X + 16,
                           Y,
                           Menu->Width - 16 - 16,
                           9 * Max);

            for(i = 0; i < Max; i++)
            {
                // Print
                FastPrint(Menu,
                          Menu->X + Menu->Lines_Offsets[index].Size,
                          Y,
                          &Menu->Text[Menu->Lines_Offsets[index].Pos],
                          Menu->Lines_Offsets[index].Len
                         );
                Y += Menu->Font->Height + 1;
                index += 1;
            }
        }
        else
        {
            WaitBlit();
            Scroll(Menu->X + 16,
                   Menu->Y + Menu->Text_Start_Y + Min_Y,
                   Menu->Width - 16 - 16,
                   (Max * (Menu->Font->Height + 1)),
                   0, 
                   Delta * (Menu->Font->Height + 1)
                  );
    
            if(Delta > 0)
            {
                /* Draw at the bottom */
                Y = Menu->Y + Menu->Text_Start_Y + Min_Y + ((Max - Lines_To_Draw) * (Menu->Font->Height + 1));
                index = Menu->Line_Top + Delta + (Max - Lines_To_Draw) + Menu->Min_Line_Sel;

                Copy_To_Screen(BitMap_Center,
                               0,
                               0,
                               Menu->X + 16,
                               Y,
                               Menu->Width - 16 - 16,
                               9 * Lines_To_Draw);

                for(i = 0; i < Lines_To_Draw; i++)
                {
                    FastPrint(Menu,
                              Menu->X + Menu->Lines_Offsets[index].Size,
                              Y,
                              &Menu->Text[Menu->Lines_Offsets[index].Pos],
                              Menu->Lines_Offsets[index].Len
                         );
                    Y += Menu->Font->Height + 1;
                    index += 1;
                }
            }
            else
            {
                /* Draw at the top */
                Y = Menu->Y + Menu->Text_Start_Y + Min_Y;
                index = Menu->Line_Top + Delta + Menu->Min_Line_Sel;

                Copy_To_Screen(BitMap_Center,
                               0,
                               0,
                               Menu->X + 16,
                               Y,
                               Menu->Width - 16 - 16,
                               9 * Lines_To_Draw);

                for(i = 0; i < Lines_To_Draw; i++)
                {
                    FastPrint(Menu,
                              Menu->X + Menu->Lines_Offsets[index].Size,
                              Y,
                              &Menu->Text[Menu->Lines_Offsets[index].Pos],
                              Menu->Lines_Offsets[index].Len
                         );
                    Y += Menu->Font->Height + 1;
                    index += 1;
                }
            }
        }

        /* Refresh the indication arrows */
        Display_Arrows(Menu, Menu->Line_Top + Delta);

        /* Refresh lasts for 2 frames */       
        Menu->Refresh++;
        if(Menu->Refresh == 2)
        {
            Menu->Refresh = 0;
            /* Done */
            Menu->Line_Top += Delta;
            Menu->Old_Line_Selected = Menu->Line_Selected;
        }
    }
}

/* --------------------------------
   Display the arrows to notify more text available
   -------------------------------- */
void Display_Arrows(LPMENU Menu, int Top_Line)
{
    WORD Pos_Arrow_X;
    WORD Pos_Arrow_Y;

    if(Menu->Lines > Menu->Max_Lines)
    {
        Pos_Arrow_X = Menu->X + Menu->Width - 15;
        Pos_Arrow_Y = Menu->Y + Menu->Text_Start_Y + (Menu->Min_Line_Sel * (Menu->Font->Height + 1));
        if(Top_Line)
        {
            /* We're not at the top of the list */
            Copy_To_Screen(Menu->Pic, 27, 9, Pos_Arrow_X, Pos_Arrow_Y, 8, 7);
        }
        else
        {
            /* Clear the arrow */
            Copy_To_Screen(Menu->Pic, 36, 0, Pos_Arrow_X, Pos_Arrow_Y, 8, 7);
        }
        Pos_Arrow_Y = Menu->Y + ((Menu->Max_Lines - 1) * (Menu->Font->Height + 1)) + Menu->Text_Start_Y;
        if(Top_Line < (Menu->Lines - Menu->Max_Lines))
        {
            /* We're not at the bottom of the list */
            Copy_To_Screen(Menu->Pic, 27, 16, Pos_Arrow_X, Pos_Arrow_Y, 8, 7);
        }
        else
        {
            /* Clear the arrow */
            Copy_To_Screen(Menu->Pic, 36, 0, Pos_Arrow_X, Pos_Arrow_Y, 8, 7);
        }
    }
}

/* --------------------------------
   Return the index of the currently selected line of a menu
   -------------------------------- */
int Get_Selected_Menu_Line(LPMENU Menu)
{
    return Menu->Line_Selected;
}

/* --------------------------------
   Set the index of the currently selected line of a menu
   -------------------------------- */
int Set_Selected_Menu_Line(LPMENU Menu, int Value, int WrapAround)
{
    int Back_Value;
    int Way = 0;

    if(Menu->Line_Selected != Value)
    {
Diff_Line:
        Back_Value = Value;
        if(Back_Value < Menu->Line_Selected)
        {
            Way = -1;
        }
        else
        {
            Way = 1;
        }
        if(WrapAround)
        {
            /* Wrap around the boundaries of the list */
            if(Back_Value < Menu->Min_Line_Sel)
            {
                Back_Value = Menu->Lines - abs(Back_Value - Menu->Min_Line_Sel);
                Way = -1;
            }
            if(Back_Value > Menu->Lines - 1)
            {
                Back_Value = Back_Value - Menu->Lines + Menu->Min_Line_Sel;
                Way = 1;
            }
        }
        else
        {
            /* Don't cross boundaries */
            if(Back_Value < Menu->Min_Line_Sel)
            {
                Back_Value = Menu->Min_Line_Sel;
                Way = -1;
            }
            if(Back_Value > Menu->Lines - 1)
            {
                Back_Value = Menu->Lines - 1;
                Way = 1;
            }
        }
       
        if(Back_Value != Menu->Line_Selected)
        {
            if(Way > 0)
            {
                // Make sure we don't stop on an empty line
                if(Menu->Lines_Offsets[Back_Value].Len == 1)
                {
                    Value++;
                    if(Value < Menu->Lines)
                    {
                        goto Diff_Line;
                    }    
                }
            }
            else
            {
                // Make sure we don't stop on an empty line
                if(Menu->Lines_Offsets[Back_Value].Len == 1)
                {
                    Value--;
                    if(Value >= Menu->Min_Line_Sel)
                    {
                        goto Diff_Line;
                    }    
                }
            }
        }
        Menu->Old_Line_Selected = Menu->Line_Selected;
        Menu->Line_Selected = Back_Value;
    }
    return Menu->Line_Selected;
}

/* --------------------------------
   Return the index of the currently selected column of a menu
   -------------------------------- */
int Get_Selected_Menu_Column(LPMENU Menu)
{
    return Menu->Column_Selected;
}

/* --------------------------------
   Return the index of the currently selected column of a menu
   -------------------------------- */
int Get_Menu_Columns(LPMENU Menu)
{
    return Menu->Max_Columns;
}

/* --------------------------------
   Set the index of the currently selected column of a menu
   -------------------------------- */
int Set_Selected_Menu_Column(LPMENU Menu, int Value)
{
    if(Menu->Column_Selected != Value)
    {
        Menu->Column_Selected = Value;
        /* Check about the boundaries of the list */
        if(Menu->Column_Selected < 0)
        {
            Menu->Column_Selected = Menu->Max_Columns - 1;
        }

        if(Menu->Column_Selected > Menu->Max_Columns - 1)
        {
            Menu->Column_Selected = 0;
        }
    }
    return Menu->Column_Selected;
}

/* --------------------------------
   Push the menu into the focus stack
   -------------------------------- */
void Push_Focus_Menu(LPMENU Menu)
{
    if(Focus_Stack_Pos < (FOCUS_STACK_SIZE - 1))
    {
        Focus_Stack_Pos++;
        Focus_Stack[Focus_Stack_Pos] = Menu;
    }
}

/* --------------------------------
   Pop the menu from the focus stack
   -------------------------------- */
void Pop_Focus_Menu()
{
    if(Focus_Stack_Pos >= 0)
    {
        Focus_Stack[Focus_Stack_Pos] = NULL;
        Focus_Stack_Pos--;
    }
}

/* --------------------------------
   Return the current
   -------------------------------- */
LPMENU Get_Focus_Menu()
{
    if(Focus_Stack_Pos == -1) return NULL;
    return Focus_Stack[Focus_Stack_Pos];
}

/* --------------------------------
   Pass the events to the menu handler if there're some
   -------------------------------- */
void Dispatch_Menu_Events(ULONG Pad_Dats)
{
    LPMENU Cur_Menu;

    Cur_Menu = Get_Focus_Menu();
    if(Cur_Menu->Events)
    {
        Cur_Menu->Events(Cur_Menu, Get_Selected_Menu_Line(Cur_Menu), Pad_Dats);
    }
}

/* --------------------------------
   Create the object used for pointing
   -------------------------------- */
LPPOINTER Create_Pointer(WORD Width,
                         WORD Height,
                         LPBITMAP Bitmap_Bank)
{
    LPPOINTER Ptr;

    Ptr = (LPPOINTER) Mem_Alloc(sizeof(POINTER));
    
    if(Ptr)
    {
        Ptr->Object = Create_Object(Width, Height, Bitmap_Bank);
        if(!Ptr->Object)
        {
            return NULL;
        }
        
    }
    return Ptr;
}

/* --------------------------------
   Free a pointing object
   -------------------------------- */
void Destroy_Pointer(LPPOINTER Ptr)
{
    if(Ptr)
    {
        Destroy_Object(Ptr->Object);
        Mem_Free(Ptr);
    }   
}

/* --------------------------------
   Erase a pointing object
   -------------------------------- */
void Erase_Pointer(LPPOINTER Ptr)
{
    if(Ptr->Menu == Get_Focus_Menu())
    {
        Erase_Object(Ptr->Object);
    }
}

/* --------------------------------
   Display the pointing object on a given menu
   -------------------------------- */
void Display_Pointer(LPPOINTER Ptr,
                     LPMENU Menu,
                     int Line,
                     WORD Src_X,
                     WORD Src_Y)
{
    Line -= Menu->Line_Top + Menu->Min_Line_Sel;
    if(Line < 0)
    {
        Line = 0;
    }
    if(Line >= (Menu->Max_Lines - Menu->Min_Line_Sel) - 1)
    {
        Line = (Menu->Max_Lines - Menu->Min_Line_Sel) - 1;
    }
    Ptr->Menu = Menu;
    Display_Object(Ptr->Object,
                   Menu->X - 4 + (Menu->Column_Selected * Menu->Columns_Size) + Menu->Extra_X,
                   Menu->Y + Menu->Text_Start_Y + ((Line + Menu->Min_Line_Sel) * (Menu->Font->Height + 1)),
                   Src_X,
                   Src_Y);
}

/* --------------------------------
   Internal function to change a text line
   -------------------------------- */
void _Refresh_Menu_Line(LPMENU Menu, int Y, int Line)
{
    Copy_To_Screen(BitMap_Center,
                   0,
                   0,
                   Menu->X + 16,
                   Y,
                   Menu->Width - 16 - 16,
                   Menu->Font->Height + 1);

    FastPrint(Menu,
              Menu->X + Menu->Lines_Offsets[Line].Size,
              Y,
              &Menu->Text[Menu->Lines_Offsets[Line].Pos],
              Menu->Lines_Offsets[Line].Len
             );
    Swap_Physical_Buffers();
    Copy_To_Bitmap(BitMap_Ptr_Back,
                   Menu->X + 16,
                   Y,
                   BitMap_Ptr_Front,
                   Menu->X + 16,
                   Y,
                   Menu->Width - 16 - 16,
                   Menu->Font->Height + 1,
                   ABC | ABNC
                  );
}

/* --------------------------------
   Change the line of the text (new text CAN'T have more chars than a menu line)
   -------------------------------- */
void Set_Menu_Line_Text(LPMENU Menu,
                        char *Text,
                        int Line,
                        char Clear_Char,
                        BOOL Refresh)
{
    int i;
//    int j;
    int Y;
    int Text_Len;

    if(Line >= 0 &&
       Line < Menu->Lines)
    {
        Text_Len = strlen(Text);
        if(Text_Len > Menu->Lines_Offsets[Line].Len) Text_Len = Menu->Lines_Offsets[Line].Len;

        /* Clean up the text of the menu */
        for(i = 0; i < Menu->Lines_Offsets[Line].Len; i++)
        {
    //        for(j = 0; j < Menu->Font->ASCII_Len; j++)
      //      {
        //        if(Menu->Font->ASCII[j] == )
          //      {
                    Menu->Text[i + Menu->Lines_Offsets[Line].Pos] = Clear_Char - 0x20;
            //        break;
              //  }
            //}
        }
        /* Complete the entry with spaces */
        for(i = 0; i < Text_Len; i++)
        {
            //for(j = 0; j < Menu->Font->ASCII_Len; j++)
//            {
  //              if(Menu->Font->ASCII[j] == Text[i])
    //            {
                    Menu->Text[i + Menu->Lines_Offsets[Line].Pos] = Text[i] - 0x20;
      //              break;
        //        }
          //  }
        }

        /* We may want to refresh a title line */
        if(Line < Menu->Min_Line_Sel)
        {
            Y = Menu->Y + Menu->Text_Start_Y + (Line * (Menu->Font->Height + 1));
            if(Refresh)
            {
                if(Line < Menu->Line_Top + (Menu->Max_Lines - Menu->Min_Line_Sel))
                {
                    _Refresh_Menu_Line(Menu, Y, Line);
                }
            }
        }
        else
        {
            Y = Menu->Y + Menu->Text_Start_Y + (Line * (Menu->Font->Height + 1));
            Line -= Menu->Min_Line_Sel;
            if(Refresh)
            {
                if(Line >= Menu->Line_Top &&
                   Line < Menu->Line_Top + (Menu->Max_Lines - Menu->Min_Line_Sel))
                {
                    Line += Menu->Min_Line_Sel;
                    _Refresh_Menu_Line(Menu, Y, Line);
                }
            }
        }
    }
}

/* --------------------------------
   Display a simple message in screen
   -------------------------------- */
void Display_Message(LPFONT Font,
                     LPBITMAP Pic,
                     int Y,
                     char *Format,
                     ...)
{
    va_list Arg;
    char szBuffer[2048];
    LPMENU Menu;

    va_start(Arg, Format);
        vsprintf(szBuffer, Format, (char *) Arg);

        Menu = Construct_Menu(Font,
                              Pic,
                              NULL,
                              MENU_TEXT_CENTER | MENU_NO_POINTER,
                              15, 0,
                              1, 0,
                              MOVE_NONE,
                              szBuffer
                          );
        Show_Menu(Menu, NULL, -1, Y, 0);
        Wait_Pad_Ok_Cancel();
        Hide_Menu(Menu, NULL);
        Destroy_Menu(Menu);

    va_end(Arg);
}
