/* --------------------------------
   Phantasie II CD32
   Menus support code
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

#ifndef _MENUS_H_
#define _MENUS_H_

/* --------------------------------
   Constants
   -------------------------------- */
#define MENU_TEXT_CENTER 0
#define MENU_TEXT_LEFT 1
#define MENU_TEXT_SECONDARY_FONT 2
#define MENU_TEXT_LAST_LINE_CENTER 4
#define MENU_NO_POINTER 8

/* --------------------------------
   Structures
   -------------------------------- */
typedef struct __attribute__((packed))
{
    int Size;
    int Pos;
    int Len;
} TXTLINE, *LPTXTLINE;

typedef struct __attribute__((packed))
{
    int X;
    int Y;
    int Width;
    int Height;
    int Tiles_X;
    int Tiles_Y;
    int Max_Lines;
    int Max_Columns;
    int Columns_Size;
    int Min_Line_Sel;
    int Lines;
    int Text_Start_Y;
    int Line_Top;
    int Line_Selected;
    int Old_Line_Selected;
    int Column_Selected;
    int Flags;
    int Movements;
    int Extra_X;
    char *Text;
    void (*Events)(void *Self, int Item, ULONG Pad_Dats);
    LPFONT Font;
    LPBITMAP Bitmap;
    LPBITMAP Bitmap_Save;
    LPBITMAP Pic;
    TXTLINE Lines_Offsets[512];
    WORD Refresh;
} MENU, *LPMENU;

typedef struct __attribute__((packed))
{
    LPMENU Menu;
    LPOBJECT Object;
} POINTER, *LPPOINTER;

/* --------------------------------
   Macros
   -------------------------------- */
void _Destroy_Menu(LPMENU Menu);
#define Destroy_Menu(x) _Destroy_Menu(x); x = NULL;

/* --------------------------------
   Functions
   -------------------------------- */
BOOL Init_Menu_Context(LPBITMAP Pic);
void Destroy_Menu_Context();
LPMENU Construct_Menu(LPFONT Font,
                      LPBITMAP Pic,
                      void (*Events)(void *, int, ULONG),
                      int Flags,
                      int Max_Lines,
                      int Min_Sel,
                      int Max_Columns,
                      int Columns_Size,
                      int Allow_Movements,
                      char *Format,
                      ...);
void Show_Menu(LPMENU Menu,
               LPPOINTER Ptr,
               WORD Pos_X,
               WORD Pos_Y,
               WORD Extra_X);
void Hide_Menu(LPMENU Menu, LPPOINTER Pointer);
void Refresh_Menu(LPMENU Menu);
LPMENU Get_Focus_Menu();
int Get_Selected_Menu_Line(LPMENU Menu);
int Set_Selected_Menu_Line(LPMENU Menu, int Value, int WrapAround);
int Get_Selected_Menu_Column(LPMENU Menu);
int Set_Selected_Menu_Column(LPMENU Menu, int Value);
int Get_Menu_Columns(LPMENU Menu);
void Dispatch_Menu_Events(ULONG Pad_Dats);
LPPOINTER Create_Pointer(WORD Width,
                         WORD Height,
                         LPBITMAP Bitmap_Bank);
void Destroy_Pointer(LPPOINTER Ptr);
void Erase_Pointer(LPPOINTER Ptr);
void Display_Pointer(LPPOINTER Ptr,
                     LPMENU Menu,
                     int Line,
                     WORD Src_X,
                     WORD Src_Y);
void Set_Menu_Line_Text(LPMENU Menu,
                        char *Text,
                        int Line,
                        char Clear_Char,
                        BOOL Refresh);
void Push_Focus_Menu(LPMENU Menu);
void Pop_Focus_Menu();
void Display_Message(LPFONT Font,
                     LPBITMAP Pic_Menus,
                     int Y,
                     char *Format,
                     ...);

#endif
