/* --------------------------------
   Phantasie II CD32
   Packing support code
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* --------------------------------
   Includes
   -------------------------------- */
#include "startup.h"

/* --------------------------------
   Pack a block of memory
   -------------------------------- */
UBYTE *Pack_Data(UBYTE *Memory,
                 int *Size)
{
    z_stream c_stream;
    UBYTE *Final_Mem_Out = (UBYTE *) Mem_Alloc(*Size + 2);
    WORD *wFinal_Mem_Out;

    memset(&c_stream, 0, sizeof(c_stream));
    deflateInit(&c_stream, Z_BEST_COMPRESSION);
    c_stream.next_in = (Bytef *) Memory;
    c_stream.next_out = Final_Mem_Out + 2;
    while((int) c_stream.total_in != (int) *Size)
    {
        c_stream.avail_in = 1;
        c_stream.avail_out = 1;
        deflate(&c_stream, Z_NO_FLUSH);
    }
    for(;;)
    {
        c_stream.avail_out = 1;
        if(deflate(&c_stream, Z_FINISH) == Z_STREAM_END) break;
    }
    deflateEnd(&c_stream);
    wFinal_Mem_Out = (WORD *) Final_Mem_Out;
    wFinal_Mem_Out[0] = c_stream.total_out;
    *Size = c_stream.total_out + 2;
    return(Final_Mem_Out);
}

/* --------------------------------
   Depack a block of memory
   -------------------------------- */
UBYTE *Depack_Data(UBYTE *Memory,
                   int Size_Out)
{
    z_stream d_stream;
    int Sizen;

    UBYTE *Dest_Mem = (UBYTE *) Mem_Alloc(Size_Out);
    if(Dest_Mem)
    {
        memset(&d_stream, 0, sizeof(d_stream));
        /* Get size from data */
        Sizen = ((WORD *) Memory)[0];
        d_stream.next_in = Memory + 2;
        d_stream.next_out = Dest_Mem;
        inflateInit(&d_stream);
        while((int) d_stream.total_out < Size_Out &&
              (int) d_stream.total_in < Sizen)
        {
            d_stream.avail_in = 1;
            d_stream.avail_out = 1;
            if(inflate(&d_stream, Z_NO_FLUSH) == Z_STREAM_END) break;
        }
        inflateEnd(&d_stream);
    }
    return(Dest_Mem);
}

