/* --------------------------------
   Phantasie II CD32
   Pad controller handling
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* --------------------------------
   Includes
   -------------------------------- */
#include "startup.h"

/* --------------------------------
   Variables
   -------------------------------- */
extern struct LowLevelBase *LowLevelBase;
extern int JOYPAD_PORT;

ULONG Old_Pad_Dats[4];
BYTE Temp_Pad[4];
BYTE Slow_Pad[4];
ULONG Pressed_Pad_Buttons;
int Pad_Allowed_Move;

/* --------------------------------
   Functions
   -------------------------------- */
extern "C" ULONG PadRead(int Port);

/* --------------------------------
   Handle pad arrows tempo
   -------------------------------- */
ULONG Check_Pad_Arrow(ULONG Pad_Dats,
                      ULONG Button,
                      int Idx,
                      BYTE Throttle_Slow,
                      BYTE Throttle_Fast)
{
    ULONG Pad_Arrows;

    if(Pad_Dats & Button)
    {
        Pad_Arrows = Pad_Dats & Button;
        if(Pad_Arrows != Old_Pad_Dats[Idx])
        {
            Temp_Pad[Idx] = 0;
            Slow_Pad[Idx] = 0;
            Old_Pad_Dats[Idx] = Pad_Arrows;
        }
        else
        {
            if(Temp_Pad[Idx])
            {
                Temp_Pad[Idx]--;
            }
            else
            {
                if(Slow_Pad[Idx])
                {
                    Temp_Pad[Idx] = Throttle_Fast;
                }
                else
                {
                    Slow_Pad[Idx] = 1;
                    Temp_Pad[Idx] = Throttle_Slow;
                }
            }
            if(Temp_Pad[Idx])
            {
                Pad_Dats &= ~Button;
            }
        }
    }
    else
    {
        Temp_Pad[Idx] = 0;
        Slow_Pad[Idx] = 0;
        Old_Pad_Dats[Idx] = 0;
    }
    return Pad_Dats;
}

/* --------------------------------
   Retrieve the data of a CD32 joypad (or Megadrive or Master System)
   -------------------------------- */
ULONG Get_Pad(BYTE Throttle_Slow, BYTE Throttle_Fast)
{
    ULONG Pad_Dats;

    if(LowLevelBase)
    {
        Pad_Dats = ReadJoyPort(JOYPAD_PORT);
    }
    else
    {
        /* Supply our own routine if lowlevel.library isn't present */
        Pad_Dats = PadRead(JOYPAD_PORT);
    }

    if(!Throttle_Slow && !Throttle_Fast)
    {
        return Pad_Dats;
    }

    if(Pad_Allowed_Move & MOVE_VERTICALLY)
    {
        Pad_Dats = Check_Pad_Arrow(Pad_Dats, PAD_BUTTON_DOWN, 0, Throttle_Slow, Throttle_Fast);
        Pad_Dats = Check_Pad_Arrow(Pad_Dats, PAD_BUTTON_UP, 1, Throttle_Slow, Throttle_Fast);
    }
    else
    {
        Pad_Dats &= ~PAD_BUTTON_DOWN;
        Pad_Dats &= ~PAD_BUTTON_UP;
    }
    if(Pad_Allowed_Move & MOVE_HORIZONTALLY)
    {
        Pad_Dats = Check_Pad_Arrow(Pad_Dats, PAD_BUTTON_LEFT, 2, Throttle_Slow, Throttle_Fast);
        Pad_Dats = Check_Pad_Arrow(Pad_Dats, PAD_BUTTON_RIGHT, 3, Throttle_Slow, Throttle_Fast);
    }
    else
    {
        Pad_Dats &= ~PAD_BUTTON_LEFT;
        Pad_Dats &= ~PAD_BUTTON_RIGHT;
    }

    /* Make sure there 2 buttons have a pressed / released sequence */
    if(Pad_Dats & PAD_BUTTON_RED)
    {
        if((Pressed_Pad_Buttons & PAD_BUTTON_RED) == 0)
        {
            Pressed_Pad_Buttons |= PAD_BUTTON_RED;
        }
        else
        {
            Pad_Dats &= ~PAD_BUTTON_RED;
        }
    }
    else
    {
        Pressed_Pad_Buttons &= ~PAD_BUTTON_RED;
    }

    if(Pad_Dats & PAD_BUTTON_BLUE)
    {
        if((Pressed_Pad_Buttons & PAD_BUTTON_BLUE) == 0)
        {
            Pressed_Pad_Buttons |= PAD_BUTTON_BLUE;
        }
        else
        {
            Pad_Dats &= ~PAD_BUTTON_BLUE;
        }
    }
    else
    {
        Pressed_Pad_Buttons &= ~PAD_BUTTON_BLUE;
    }

    if(Pad_Dats & PAD_BUTTON_FORWARD)
    {
        if((Pressed_Pad_Buttons & PAD_BUTTON_FORWARD) == 0)
        {
            Pressed_Pad_Buttons |= PAD_BUTTON_FORWARD;
        }
        else
        {
            Pad_Dats &= ~PAD_BUTTON_FORWARD;
        }
    }
    else
    {
        Pressed_Pad_Buttons &= ~PAD_BUTTON_FORWARD;
    }

    if(Pad_Dats & PAD_BUTTON_REVERSE)
    {
        if((Pressed_Pad_Buttons & PAD_BUTTON_REVERSE) == 0)
        {
            Pressed_Pad_Buttons |= PAD_BUTTON_REVERSE;
        }
        else
        {
            Pad_Dats &= ~PAD_BUTTON_REVERSE;
        }
    }
    else
    {
        Pressed_Pad_Buttons &= ~PAD_BUTTON_REVERSE;
    }

    return Pad_Dats;
}

/* --------------------------------
   Wait for the user to press OK (RED button)
   -------------------------------- */
void Wait_Pad_Ok()
{
    ULONG Pad_Dats;

    do
    {
        Pad_Dats = Get_Pad(8, 1);
    } while((Pad_Dats & PAD_BUTTON_RED) == 0);
}

/* --------------------------------
   Wait for the user to press OK (RED button) or CANCEL (BLUE button)
   -------------------------------- */
void Wait_Pad_Ok_Cancel()
{
    ULONG Pad_Dats;

    do
    {
        Pad_Dats = Get_Pad(8, 1);
    } while((Pad_Dats & PAD_BUTTON_RED) == 0 &&
            (Pad_Dats & PAD_BUTTON_BLUE) == 0);
}

/* --------------------------------
   Define the allowed movement for the pad
   -------------------------------- */
void Set_Pad_Move(int Pad_Move)
{
    Pad_Allowed_Move = Pad_Move;
}
