/* --------------------------------
   Phantasie II CD32
   Application entry point
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* --------------------------------
   Includes
   -------------------------------- */
#include "startup.h"

/* --------------------------------
   External functions
   -------------------------------- */
void Start_Audio_Interrupt(void (*Interrupt)());
void Stop_Audio_Interrupt();

#ifdef LUA
BYTE Init(lua_State *Lua);
#else
BYTE Init();
#endif

ULONG Loop(BYTE Flip_Flop);
void Release();

#ifndef CD32
struct InputEvent *Input_Handler(struct InputEvent *Input_Events, void *Data);
#endif

extern "C" void Handler_Stub();
void Stop_VBlank_Interrupt();
void VBlank_Interrupt();
void Free_Resources();

#ifdef LUA
static int Lua_Report_Error(lua_State *L, int status);
#endif

/* --------------------------------
   Variables
   -------------------------------- */
#ifdef LUA
lua_State *Lua_Main;
int Lua_Main_Status;
#endif
struct DOSBase *DOSBase;
struct DOSBase *__DOSBase;
struct UtilityBase *__UtilityBase;
struct Message *_WBenchMsg;
struct ExecBase *SysBase;
struct IntuitionBase *IntuitionBase;
struct GfxBase *GfxBase;
struct IFFParseBase *IFFParseBase;
struct LowLevelBase *LowLevelBase;
struct MathBase *MathBase;
struct MathIeeeSingBasBase *MathIeeeSingBasBase;
struct MathIeeeSingBasBase *__MathIeeeSingBasBase;
struct MathIeeeSingTransBase *MathIeeeSingTransBase;
struct MathIeeeSingTransBase *__MathIeeeSingTransBase;
struct MathIeeeDoubBasBase *MathIeeeDoubBasBase;
struct MathIeeeDoubBasBase *__MathIeeeDoubBasBase;
struct MathIeeeDoubTransBase *MathIeeeDoubTransBase;
struct MathIeeeDoubTransBase *__MathIeeeDoubTransBase;

struct MsgPort *InputMP;
struct IOStdReq *InputIO;
struct InputBase *InputBase;
BYTE Got_Input = -1;

struct DBufInfo *DBufInfo;

#ifdef CD32
BYTE Got_CD = -1;
#endif

int Start_Sp;

int Got_NVRAM;

BOOL Even_Frame;

struct View View;
extern struct ColorMap *ColorMap;
extern struct ViewPort ViewPort;
extern LPBITMAP BitMap_Front;
extern LPBITMAP BitMap_Back;
extern struct RasInfo RasInfo;
extern struct cprlist *CopperList_Front;
extern struct RastPort RastPort;
LPBITMAP BitMap_Ptr_Front;
LPBITMAP BitMap_Ptr_Back;

extern const char *PROGNAME;
extern const char *INPUTNAME;

extern int MemLeak;

extern PALETTE32 Main_Palette;

int SCREEN_BYTES;
int SCREEN_PLANE_SIZE;
int SCREEN_SIZE;

ULONG Black_Palette[256];
ULONG White_Palette[256];

short Mouse_X;
short Mouse_Y;
UBYTE Mouse_Buttons;
short Key;

struct Interrupt VBlank_Int_Vector =
{
	{ NULL, NULL, NT_INTERRUPT, (BYTE) 196, NULL },
	NULL, NULL
};

struct Interrupt Audio_Int_Vector =
{
	{ NULL, NULL, NT_INTERRUPT, (BYTE) 196, NULL },
	NULL, NULL
};

struct Interrupt Input_Int_Vector =
{
    { NULL, NULL, NT_UNKNOWN, 100, NULL },
    NULL, &Handler_Stub
};

/* --------------------------------
   VBlank Interrupt
   -------------------------------- */
void Start_VBlank_Interrupt()
{
    Stop_VBlank_Interrupt();
    VBlank_Int_Vector.is_Code = &VBlank_Interrupt;
    AddIntServer(INTB_VERTB, &VBlank_Int_Vector);
}

void Stop_VBlank_Interrupt()
{
    if(VBlank_Int_Vector.is_Code)
    {
        RemIntServer(INTB_VERTB, &VBlank_Int_Vector);
        VBlank_Int_Vector.is_Code = NULL;
    }
}

/* --------------------------------
   Audio Interrupt
   -------------------------------- */
void Start_Audio_Interrupt(void (*Interrupt)())
{
    if(Audio_Int_Vector.is_Code != Interrupt)
    {
        Stop_Audio_Interrupt();
        Audio_Int_Vector.is_Code = Interrupt;
        AddIntServer(INTB_AUD3, &Audio_Int_Vector);
    }
}

void Stop_Audio_Interrupt()
{
    if(Audio_Int_Vector.is_Code)
    {
        RemIntServer(INTB_AUD3, &Audio_Int_Vector);
        Audio_Int_Vector.is_Code = NULL;
    }
}

/* --------------------------------
   Retrieve the mouse positions from a specified port
   -------------------------------- */
#ifndef CD32
UBYTE Get_Mouse(short *Pos_X, short *Pos_Y)
{
    short Calc_X = *Pos_X;
    short Calc_Y = *Pos_Y;

    Calc_X += Mouse_X;
    Calc_Y += Mouse_Y;
    Mouse_X = 0;
    Mouse_Y = 0;
    if(Calc_X > SCREEN_WIDTH - 1) Calc_X = SCREEN_WIDTH - 1;
    if(Calc_Y > SCREEN_HEIGHT - 1) Calc_Y = SCREEN_HEIGHT - 1;
    if(Calc_X < 0) Calc_X = 0;
    if(Calc_Y < 0) Calc_Y = 0;
    *Pos_X = Calc_X;
    *Pos_Y = Calc_Y;
    return(Mouse_Buttons);
}
#endif

/* --------------------------------
   Monitor input events
   -------------------------------- */
extern "C" struct InputEvent *Input_Handler(struct InputEvent *Input_Events,
                                            void *Data)
{
    switch(Input_Events->ie_Class)
    {
        case IECLASS_RAWMOUSE:

            Mouse_X = Input_Events->ie_position.ie_xy.ie_x;
            Mouse_Y = Input_Events->ie_position.ie_xy.ie_y;

            switch(Input_Events->ie_Code)
            {
                case IECODE_LBUTTON:

                    Mouse_Buttons |= MOUSE_LEFT_BUTTON;
                    return NULL;

                case (IECODE_LBUTTON | IECODE_UP_PREFIX):

                    Mouse_Buttons &= ~MOUSE_LEFT_BUTTON;
                    return NULL;

                case IECODE_MBUTTON:

                    Mouse_Buttons |= MOUSE_MIDDLE_BUTTON;
                    return NULL;

                case (IECODE_MBUTTON | IECODE_UP_PREFIX):

                    Mouse_Buttons &= ~MOUSE_MIDDLE_BUTTON;
                    return NULL;

                case IECODE_RBUTTON:

                    Mouse_Buttons |= MOUSE_RIGHT_BUTTON;
                    return NULL;

                case (IECODE_RBUTTON | IECODE_UP_PREFIX):

                    Mouse_Buttons &= ~MOUSE_RIGHT_BUTTON;
                    return NULL;
            }
            return NULL;

        case IECLASS_RAWKEY:

 	        Key = Input_Events->ie_Code;
            return NULL;

        default:

            return Input_Events;
    }
}

/* --------------------------------
   Report any error from LUA
   -------------------------------- */
#ifdef LUA
static void Lua_Message(const char *pname, const char *msg)
{
    if(pname)
    {
        fprintf(stderr, "%s: ", pname);
    }
    fprintf(stderr, "%s\n", msg);
    fflush(stderr);
}

static int Lua_Report_Error(lua_State *L, int status)
{
    if (status && !lua_isnil(L, -1))
    {
        const char *msg = lua_tostring(L, -1);
        if(msg == NULL)
        {
            msg = "(error object is not a string)";
        }
        Lua_Message(PROGNAME, msg);
        lua_pop(L, 1);
    }
    return status;
}
#endif

/* --------------------------------
   Free the allocated resources
   -------------------------------- */
void Free_Resources()
{
    char *Err_DOS = "Couldn't open dos.library v39.\n";
    
    if(DOSBase)
    {
        Stop_VBlank_Interrupt();
        Stop_Audio_Interrupt();
        Release();

        if(!Got_Input)
        {
            InputIO->io_Data = (APTR) &Input_Int_Vector;
            InputIO->io_Command = IND_REMHANDLER;
            DoIO((struct IORequest *) InputIO);
        }

        if(InputBase) CloseDevice((struct IORequest *) InputIO);
        if(InputIO) DeleteIORequest((struct IORequest *) InputIO);
        if(InputMP) DeleteMsgPort(InputMP);


#ifdef CD32
        NVRAM_Close();
#endif
    
        if(DBufInfo) FreeDBufInfo(DBufInfo);
        if(CopperList_Front) FreeCprList(CopperList_Front);
        FreeVPortCopLists(&ViewPort);
    
        Destroy_Bitmap(BitMap_Front);
        Destroy_Bitmap(BitMap_Back);
        
        if(ColorMap) FreeColorMap(ColorMap);
    
        RemakeDisplay();

#ifdef CD32
        CD_Device_Term();
#endif

        if(LowLevelBase)
        {
            SystemControl(SCON_KillReq, FALSE,
                          SCON_CDReboot, CDReboot_Default,
                          SCON_TakeOverSys, FALSE,
#ifdef CD32
                          SCON_StopInput, FALSE,
#endif
                          TAG_DONE, 0
                         );
            SetJoyPortAttrs(0, SJA_Reinitialize, NULL,
                               TAG_DONE, 0);
        }

        if(MathIeeeDoubTransBase) CloseLibrary((struct Library *) MathIeeeDoubTransBase);
        if(MathIeeeDoubBasBase) CloseLibrary((struct Library *) MathIeeeDoubBasBase);
        if(MathIeeeSingTransBase) CloseLibrary((struct Library *) MathIeeeSingTransBase);
        if(MathIeeeSingBasBase) CloseLibrary((struct Library *) MathIeeeSingBasBase);
        if(MathBase) CloseLibrary((struct Library *) MathBase);
        if(IntuitionBase) CloseLibrary((struct Library *) IntuitionBase);
        if(__UtilityBase) CloseLibrary((struct Library *) __UtilityBase);
        if(LowLevelBase) CloseLibrary((struct Library *) LowLevelBase);
        if(IFFParseBase) CloseLibrary((struct Library *) IFFParseBase);
        else Printf("Couldn't open iffparse.library.\n");
        if(GfxBase) CloseLibrary((struct Library *) GfxBase);

#ifdef LUA
        /* Display possible LUA errors before exiting */
        if(Lua_Main)
        {
            Lua_Report_Error(Lua_Main, Lua_Main_Status);
            lua_close(Lua_Main);
        }
#endif
        if(MemLeak)
        {
            Printf("Memory leak: %ld bytes lost\n", MemLeak);
        }

        if(DOSBase) CloseLibrary((struct Library *) DOSBase);
        fflush(0);
    }
    else
    {
        /* Open any version */
        DOSBase = (struct DOSBase *) OldOpenLibrary("dos.library");
        if(DOSBase)
        {
            Write(Output(), Err_DOS, strlen(Err_DOS));
            CloseLibrary((struct Library *) DOSBase);
        }
    }
}

/* --------------------------------
   Swap front and back buffers
   -------------------------------- */
void Swap_Logical_Buffers()
{
    LPBITMAP Bitmap_Tmp;

    WaitBlit();
    Bitmap_Tmp = BitMap_Ptr_Front;
    BitMap_Ptr_Front = BitMap_Ptr_Back;
    BitMap_Ptr_Back = Bitmap_Tmp;

    RastPort.BitMap = &BitMap_Ptr_Front->Bmp;
    Even_Frame ^= TRUE;
}

/* --------------------------------
   Swap front and back buffers and switch the visual
   -------------------------------- */
void Swap_Physical_Buffers()
{
    WaitTOF();
    Swap_Logical_Buffers();
    ChangeVPBitMap(&ViewPort, &BitMap_Ptr_Back->Bmp, DBufInfo);
}

/* --------------------------------
   Entry point
   -------------------------------- */
int main(int argc, char *argv[])
{
    ULONG Looping = FALSE;
    BYTE Flip_Flop;
    int i;   

    printf("FUCK\n");

    SysBase = (struct ExecBase *) (*((int *) 4));

    atexit(Free_Resources);

	__UtilityBase = (struct UtilityBase *) OpenLibrary("utility.library", 0);
    if(!__UtilityBase)
    {
        exit(EXIT_FAILURE);
    }

    /* Set our variables according to the user's choice */
    SCREEN_BYTES = SCREEN_WIDTH >> 3;
    SCREEN_PLANE_SIZE = SCREEN_BYTES * SCREEN_HEIGHT;
    SCREEN_SIZE = SCREEN_BYTES * SCREEN_HEIGHT * SCREEN_BPP;

    /* Open the necessary libraries */
    DOSBase = (struct DOSBase *) OpenLibrary("dos.library", 39);
	GfxBase = (struct GfxBase *) OpenLibrary("graphics.library", 0);
	IFFParseBase = (struct IFFParseBase *) OpenLibrary("iffparse.library", 0);
	LowLevelBase = (struct LowLevelBase *) OpenLibrary("lowlevel.library", 0);
    IntuitionBase = (struct IntuitionBase *) OpenLibrary("intuition.library", 0);
    MathBase = (struct MathBase *) OpenLibrary("mathffp.library", 0);
    MathIeeeSingBasBase = (struct MathIeeeSingBasBase *) OpenLibrary("mathieeesingbas.library", 0);
    MathIeeeSingTransBase = (struct MathIeeeSingTransBase *) OpenLibrary("mathieeesingtrans.library", 0);
    MathIeeeDoubBasBase = (struct MathIeeeDoubBasBase *) OpenLibrary("mathieeedoubbas.library", 0);
    MathIeeeDoubTransBase = (struct MathIeeeDoubTransBase *) OpenLibrary("mathieeedoubtrans.library", 0);
    if(!DOSBase ||
       !IntuitionBase ||
       !GfxBase ||
       !IFFParseBase ||
       !MathBase ||
       !MathIeeeSingBasBase ||
       !MathIeeeSingTransBase ||
       !MathIeeeDoubBasBase ||
       !MathIeeeDoubTransBase
      )
    {
        exit(EXIT_FAILURE);
    }
    __MathIeeeDoubTransBase = MathIeeeDoubTransBase;
    __MathIeeeSingBasBase = MathIeeeSingBasBase;
    __MathIeeeSingTransBase = MathIeeeSingTransBase;
    __DOSBase = DOSBase;

#ifdef CD32
    CloseLibrary(OpenLibrary("freeanim.library", 0));
#endif

    /* Obtain input.device */
    InputMP = CreateMsgPort();
    if(!InputMP)
	{
        exit(EXIT_FAILURE);
	}
	InputIO = (struct IOStdReq *) CreateIORequest(InputMP, sizeof(struct IOStdReq));
	if(!InputIO)
	{
        exit(EXIT_FAILURE);
    }
    if(OpenDevice("input.device", 0xFFFFFFFF, (struct IORequest *) InputIO, 0))
    {
        exit(EXIT_FAILURE);
    }

#ifdef CD32
    Got_NVRAM = NVRAM_Init(99);
#endif

    InputBase = (struct InputBase *) InputIO->io_Device;

    InputIO->io_Command = IND_ADDHANDLER;
    InputIO->io_Data = (APTR) &Input_Int_Vector;
    Got_Input = DoIO((struct IORequest *) InputIO);
	if(Got_Input)
    {
        exit(EXIT_FAILURE);
    }

#ifdef CD32
    if(!CD_Device_Init())
    {
        Got_CD = FALSE;
    }
    else
    {
        Got_CD = TRUE;
    }
#endif

	InitView(&View);
	InitVPort(&ViewPort);

    ColorMap = GetColorMap(SCREEN_COLORS);
    if(!ColorMap)
    {
        exit(EXIT_FAILURE);
    }
    /* We don't use overscan so we clear it */
    ColorMap->Flags |= BORDER_BLANKING;

    /* Remove the mouse pointer */
	FreeSprite(0);

    /* Create our front and back buffers */
    BitMap_Front = Create_Bitmap(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, TRUE, NULL);
    BitMap_Back = Create_Bitmap(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, TRUE, NULL);
    BitMap_Ptr_Front = BitMap_Front;
    BitMap_Ptr_Back = BitMap_Back;

	RasInfo.RxOffset = 0;
	RasInfo.RyOffset = 0;
	RasInfo.Next = NULL;
	
	ViewPort.DWidth = SCREEN_WIDTH;
	ViewPort.DHeight = SCREEN_HEIGHT;
	ViewPort.RasInfo = &RasInfo;
	ViewPort.ColorMap = ColorMap;

	RasInfo.BitMap = &BitMap_Ptr_Front->Bmp;

	/* Let the system construct the first copperlist */
    View.ViewPort = &ViewPort;

/*    BestModeID(BIDTAG_ViewPort, &ViewPort,
               BIDTAG_DIPFMustHave, DIPF_IS_PAL,
               TAG_DONE, 0);
*/
//    GfxNew(VIEW_EXTRA_TYPE);

	if(MakeVPort(&View, &ViewPort) != MVP_OK)
    {
        exit(EXIT_FAILURE);
    }
	if(MrgCop(&View) != MCOP_OK)
    {
        exit(EXIT_FAILURE);
    }
	CopperList_Front = View.LOFCprList;

    InitRastPort(&RastPort);
    RastPort.Flags |= DBUFFER;
    RastPort.BitMap = &BitMap_Ptr_Front->Bmp;

    for(i = 0; i < SCREEN_COLORS; i++)
    {
        White_Palette[i] = 0xffffff;
    }

    /* Set all colors black at startup */
    Set_Palette(Black_Palette, 0, SCREEN_COLORS);

    DBufInfo = AllocDBufInfo(&ViewPort);
    if(DBufInfo)
    {
      	/* Load our view now */
       	LoadView(&View);
    
#ifdef LUA
        Lua_Main = lua_open();
        if(Lua_Main)
#endif

        {
            Forbid();
    
            WaitTOF();
            Swap_Physical_Buffers();

            if(LowLevelBase)
            {
                SystemControl(SCON_KillReq, TRUE,
                              SCON_CDReboot, CDReboot_Off,
                              SCON_TakeOverSys, TRUE,
#ifdef CD32
                              SCON_StopInput, TRUE,
#endif
                              TAG_DONE, 0
                             );
                SetJoyPortAttrs(0, SJA_Type, SJA_TYPE_GAMECTLR,
                                   TAG_DONE, 0);
                SetJoyPortAttrs(1, SJA_Type, SJA_TYPE_GAMECTLR,
                                   TAG_DONE, 0);
            }

            srand(time(NULL));

            Start_VBlank_Interrupt();
    
            WaitTOF();
            /* Execute user init */

#ifdef LUA
            if(Init(Lua_Main))
#else
            if(Init())
#endif

            {
                Flip_Flop = FALSE;
                while(!Looping)
                {
                    WaitTOF();
                    Swap_Logical_Buffers();
                    ChangeVPBitMap(&ViewPort, &BitMap_Ptr_Back->Bmp, DBufInfo);
                    Looping = Loop(Flip_Flop);
                    WaitBlit();
                    Flip_Flop++;
                    if(Flip_Flop == 2)
                    {
                        Flip_Flop = 0;
                        CD_Loop_Track();
                    }
                }
                Permit();
                exit(EXIT_SUCCESS);
            }
            Permit();
        }
    }
    exit(EXIT_FAILURE);
}
