/* --------------------------------
   Phantasie II CD32
   Application entry point
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

#ifndef _STARTUP_H_
#define _STARTUP_H_

/* --------------------------------
   Includes
   -------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <stdarg.h>
#include <time.h>
#include <exec/types.h>
#include <exec/interrupts.h>
#include <exec/ports.h>
#include <exec/io.h>
#include <exec/errors.h>
#include <dos/dos.h>
#include <hardware/custom.h>
#include <hardware/intbits.h>
#include <devices/inputevent.h>
#include <devices/input.h>
#include <devices/gameport.h>
#include <devices/cd.h>
#include <hardware/custom.h>
#include <graphics/gfx.h>
#include <graphics/gfxbase.h>
#include <libraries/iffparse.h>
#include <libraries/lowlevel.h>
#include <libraries/nonvolatile.h>
#include <libraries/mathieeesp.h>
#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/iffparse_protos.h>
#include <clib/lowlevel_protos.h>
#include <clib/nonvolatile_protos.h>
#include <clib/mathieeesingtrans_protos.h>

#ifdef __cplusplus

#ifdef LUA
#define lua_c
extern "C"
{
    #include "lua/lua.h"
    #include "lua/lauxlib.h"
    #include "lua/lualib.h"
}
#endif

extern "C"
{
    #include "zlib/zlib.h"
}

#endif

#include "files.h"
#include "memory.h"
#include "gfx.h"
#include "menus.h"
#include "misc.h"
#include "cd.h"
#include "nvram.h"
#include "pad.h"
#include "packer.h"

/* --------------------------------
   Constants
   -------------------------------- */
#define MOUSE_LEFT_BUTTON 1
#define MOUSE_MIDDLE_BUTTON 2
#define MOUSE_RIGHT_BUTTON 4

/* --------------------------------
   Variables
   -------------------------------- */
extern ULONG Black_Palette[256];
extern ULONG White_Palette[256];
extern int Got_NVRAM;
extern const WORD SCREEN_WIDTH;
extern const WORD SCREEN_HEIGHT;
extern const WORD SCREEN_BPP;
extern const WORD SCREEN_COLORS;

/* --------------------------------
   Functions
   -------------------------------- */
UBYTE Get_Mouse(short *Pos_X, short *Pos_Y);
ULONG Get_JoyPad(BYTE Throttle_Slow, BYTE Throttle_Fast);
void Swap_Logical_Buffers();
void Swap_Physical_Buffers();

#endif
