/* --------------------------------
   Phantasie II CD32
   Towns handling
   --------------------------------
   Copyright (C) 2010-2016 Franck Charlet.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   
       1. Redistributions of source code must retain the above copyright notice,
          this list of conditions and the following disclaimer.
       2. Redistributions in binary form must reproduce the above copyright notice,
          this list of conditions and the following disclaimer in the documentation
          and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL FRANCK CHARLET OR CONTRIBUTORS BE LIABLE
   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.
   -------------------------------- */

/* NOTE: the wilderness should look grim, bleak and dark and
         only becomes green at the end of the game (we could have an effect
         or just a palette fading).

   NOTE: maybe we should remove the distribution stuff
         and dispatch points and gold automatically at the end of each combat ?
         (it looks like that's what is done in the ST version).
         in dungeons the level of monsters should be constantly adjusted according to this
         not only when entering the dungeon.

         Combats should display the list of characters with their previous (or default) action
         at the beginning of each round and let the user modify or not the actions.

         We're going to need 2 or 3 wilderness maps (wilderness, underground 1, underground 2).

   TODO: Ask the player if a default party should be created when starting a new game
         (and create one).
*/

/* --------------------------------
   Includes
   -------------------------------- */
#include "main.h"
#include "towns.h"
#include "characters.h"
#include "characters_sheets.h"
#include "save.h"

/* --------------------------------
   Constants
   -------------------------------- */
#define TOWN_NAME_X 70
#define TOWN_NAME_Y 0
#define TOWN_NAME_WIDTH 196
#define TOWN_NAME_HEIGHT 24
#define TOWN_NAME_Y_DEST 8

#define NO_FADING 0
#define FADING_TO_SHEET 1

/* --------------------------------
   Variables
   -------------------------------- */
TOWNDAT Towns[MAX_TOWNS];
LPMENU Menu_Town_Main;
LPMENU Menu_Character_Name;
LPMENU Menu_Create_Character;
LPMENU Menu_Stats_Character;
LPMENU Menu_Withdraw;

char Old_Character_Name[MAX_CHARACTER_NAME];

char Chars_Name_List[] =
{
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K',
    'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V',
    'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
    'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r',
    's', 't', 'u', 'v', 'w', 'x', 'y', 'z', ' '
};

int Nbr_Classes = 6;
BYTE Normal_Classes[] =
{
    0, 1, 2, 3, 4, 5
};
BYTE Random_Classes[] =
{
    0, 3
};
BYTE *Ptr_Classes;

int Created_Random_Race;
int Old_Town_Nbr = 0;
int Cur_Town_Nbr;

int Create_Character_Menu_Index;
int Consult_Mystic_Menu_Index;
int Rest_At_Inn_Menu_Index;
int Withdraw_Gold_Menu_Index;
int Deposit_Gold_Menu_Index;
int Characters_Menu_Index;
int Save_Game_Menu_Index;
int Leave_Town_Menu_Index;

float Prices_Multiplier;

BOOL WithDraw_Gold;
int Temp_Gold;

BYTE Current_Fading;

LPTOWNDAT Cur_Town;

int Temp_WithDraw_Gold[MAX_CHARACTERS - 1];

int Character_Sheet_Idx;

/* --------------------------------
   Functions
   -------------------------------- */
void Display_Choose_Character_Race(LPMENU Self,
                                   LPCHARACTER Character,
                                   int Line,
                                   BOOL Refresh);
void Display_Choose_Character_Class(LPMENU Self,
                                    LPCHARACTER Character,
                                    int Line,
                                    BOOL Refresh);
void Start_Character_Creation();
void Display_Character_Base_Stats(LPCHARACTER Character, int Race);
void Display_Mystic_Message();
void Rest_At_Inn();
void Manage_Gold(BOOL WidthDraw);
void Display_Character_And_Party_Gold(LPCHARACTER Character,
                                      int Line);
int Calc_Carried_Gold(BOOL Store);
void Move_Gold(LPCHARACTER Character,
               int i,
               BOOL WithDraw,
               int Speed);
void Randomize_Town_Stock();

/* --------------------------------
   Menus events
   -------------------------------- */
void Full_Town_Menu_Events(void *Self, int Item, ULONG Pad_Dats)
{
    int i;

    switch(Pad_Dats)
    {
        case PAD_OK:

            /* If members = 6 then check the mystic entry
               as the menu will have the same position */
            if(Party.Number_Characters < 6)
            {
                if(Item == Create_Character_Menu_Index)
                {
                    Start_Character_Creation();
                }
            }
            if(Item == Consult_Mystic_Menu_Index)
            {
                Display_Mystic_Message();
            }
            else if(Item == Rest_At_Inn_Menu_Index)
            {
                Rest_At_Inn();
            }
            else if(Item == Withdraw_Gold_Menu_Index)
            {
                Manage_Gold(TRUE);
            }
            else if(Item == Deposit_Gold_Menu_Index)
            {
                Manage_Gold(FALSE);
            }
            else if(Item >= Characters_Menu_Index &&
                    Item < (Characters_Menu_Index + Party.Number_Characters))
            {
                /* Go to character sheet */
                Hide_Menu((LPMENU) Self, Pointer);

                /* Remove the town picture */
                for(i = 0; i < PARTY_COLORS; i++)
                {
                    Current_Palette[i] = 0;
                }
                for(i = 239; i < 239 + MENUS_COLORS; i++)
                {
                    Current_Palette[i] = 0;
                }
                Current_Palette[254] = 0;
                Current_Palette[255] = 0;
                Current_Palette[237] = 0;
                Current_Palette[238] = 0;

                Set_Fade(Current_Palette, 12, MOVE_NONE);
                Current_Fading = FADING_TO_SHEET;
                Character_Sheet_Idx = Item - Characters_Menu_Index;
            }
            else if(Item == Save_Game_Menu_Index)
            {
                Save_Game(Cur_Town_Nbr);
            }
            else if(Item == Leave_Town_Menu_Index)
            {
                if(Party.Number_Characters == 0)
                {
                    Display_Message(Font, Pic_Menus, -1, "YOU MUST ASSEMBLE A GROUP");
                    break;
                }
                /* TODO: Nothing yet (exit here is only used for debugging) */
                exit(0);
            }
            break;
    }
}

/* --------------------------------
   Create character
   -------------------------------- */
void Create_Menu_Events(void *Self, int Item, ULONG Pad_Dats)
{
    LPCHARACTER Character = NULL;
    int Line;
/*    int Speed = Pad_Dats & PAD_SPEED ? 1 : 0; */

    Pad_Dats = Pad_Dats & ~PAD_SPEED;

    switch(Pad_Dats)
    {
        case PAD_LEFT:

            Character = &Party.Characters[Party.Number_Characters];
            Line = Get_Selected_Menu_Line((LPMENU) Self);
            switch(Line)
            {
                case 2:

                    if(Character->Race)
                    {
                        Character->Race--;
                    }
                    Display_Choose_Character_Race((LPMENU) Self, Character, Line, TRUE);
                    break;

                case 3:

                    if(Character->Class)
                    {
                        Character->Class--;
                    }
                    Display_Choose_Character_Class((LPMENU) Self, Character, Line, TRUE);
                    break;
            }
            break;

        case PAD_RIGHT:

            Character = &Party.Characters[Party.Number_Characters];
            Line = Get_Selected_Menu_Line((LPMENU) Self);
            switch(Line)
            {
                case 2:

                    if(Character->Race < MAX_CREATE_RACES - 1)
                    {
                        Character->Race++;
                    }
                    Display_Choose_Character_Race((LPMENU) Self, Character, Line, TRUE);
                    break;

                case 3:

                    if((int) Character->Class < Nbr_Classes - 1)
                    {
                        Character->Class++;
                    }
                    Display_Choose_Character_Class((LPMENU) Self, Character, Line, TRUE);
                    break;
            }
            break;

        case PAD_OK:

            Character = &Party.Characters[Party.Number_Characters];
            Line = Get_Selected_Menu_Line((LPMENU) Self);
            switch(Line)
            {
                case 5:

                    Hide_Menu((LPMENU) Self, Pointer);
                    Hide_Menu(Menu_Create_Character, Pointer);

                    int Race;

                    if(Character->Race == RACE_RANDOM)
                    {
                        Race = Create_Random_Race();
                        /* Save it later */
                        Created_Random_Race = Race;
                    }
                    else
                    {
                        Race = Character->Race;
                    }

                    /* Base characteristics */
                    Character->Strength = Calc_Character_Base_Attribute(Character, 0);
                    /* Patch: minotaurs or ogres are supposed to be strong fellas */
                    if(Race == MINOTAUR || Race == OGRE)
                    {
                        if(Character->Strength < 14) Character->Strength = 14;
                    }
                    Character->Intelligence = Calc_Character_Base_Attribute(Character, 1);
                    Character->Dexterity = Calc_Character_Base_Attribute(Character, 2);
                    Character->Constitution = Calc_Character_Base_Attribute(Character, 3);
                    Character->Charisma = Calc_Character_Base_Attribute(Character, 4);
 
                    Display_Character_Base_Stats(Character, Race);
                    break;
            }
            break;

        case PAD_CANCEL:

            if(Party.Number_Characters == 0)
            {
                /* Return to the main menu if no character have been created */
                Hide_Menu((LPMENU) Self, Pointer);
                Destroy_Town();
                Set_Fade(Black_Palette, 16, MOVE_NONE);
                Current_Sequence = SEQUENCE_INTRO;
                /* We need to reload the logo */
                Pic_Logo = Load_Picture("gfx/logo.iff", TRUE, NULL, 0);
                if(!Pic_Logo) exit(EXIT_FAILURE);
                RawPic_Clouds = (BYTE *) Load_File("gfx/clouds_320x640.raw");
                if(!RawPic_Clouds) exit(EXIT_FAILURE);
            }
            else
            {
                /* Return to parent menu which was destroyed (normally the Main town menu) */
                Hide_Menu((LPMENU) Self, Pointer);
                Construct_Main_Town_Menu();
                Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
            }
            break;
    }
}

/* --------------------------------
   Display stats and ask for name
   -------------------------------- */
void Base_Stats_Menu_Events(void *Self, int Item, ULONG Pad_Dats)
{
    LPCHARACTER Character = NULL;
    int Line;

    switch(Pad_Dats)
    {
        case PAD_OK:

            Character = &Party.Characters[Party.Number_Characters];
            Line = Get_Selected_Menu_Line((LPMENU) Self);
            switch(Line)
            {
                case 6:

                    /* Enter a name */
                    memcpy(Old_Character_Name, Character->Name, MAX_CHARACTER_NAME);
                    Show_Menu(Menu_Character_Name, Pointer, -1, 185, 0);
                    break;

                case 8:

                    /* Character is accepted: finalize it */
                    if(strlen(Character->Name))
                    {
                        Hide_Menu((LPMENU) Self, Pointer);
                        Destroy_Menu(Menu_Stats_Character);

                        /* Fix the attributes */
                        if(Character->Race == RACE_RANDOM)
                        {
                            /* Restore saved value as we can't randomize twice */
                            Character->Race = Created_Random_Race;
                        }

                        /* Now we can fix the class */
                        Character->Class = Ptr_Classes[Character->Class];

                        /* (Starts a level 0) */
                        Calc_Character_Modifiable_Attributes(Character, 0);

                        Character->Status = OKAY;
                        /* [150..256] gold pieces
                           (In the original game all characters started with 256 gold pieces)
                        */
                        Character->Gold = (int) (rnd() * 256.0f) + 150;
                        if(Character->Gold > 256) Character->Gold = 256;
                        Character->Current_Town = Cur_Town_Nbr;
                        Character->Origin = 1 + (Cur_Town_Nbr > FERRON);

                        /* Character starts with these objects */
                        Character->Items[0] = SMALL_SHIELD;
                        Character->Items[1] = CLUB;
                        Character->Items[2] = ROBES;
                        Reset_Character_Shield_Armor_Weapon_Points(Character);

                        /* Monks & wizard have FIREFLASH1 learned by default */
                        if(Character->Class == MONK ||
                           Character->Class == WIZARD)
                        {
                            Character->Spells[FIREFLASH_1] = LEARNED;
                        }

                        if(Character->Class == RANGER ||
                           Character->Class == PRIEST)
                        {
                            Character->Spells[HEALING_1] = LEARNED;
                        }

                        Character->Luck = (int) (8.0f + rnd() * 6.0f);
                        Character->Current_Hit_Points = Character->Max_Hit_Points;
                        Character->Max_Magic = (int) (0.75f * Character->Max_Class_Magic);
                        /* Magic to max */
                        Character->Current_Magic = Character->Max_Magic;

                        Calc_Character_Life_Span(Character);

                        float Age = 260 + (Character->LifeSpan << 8) * (0.1f + rnd() / 40.0f);
                        Character->Age += (int) Age;

                        /* Welcome to the party */
                        Party.Number_Characters++;
                        /* Reconstruct it */
                        Construct_Main_Town_Menu();
                        Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
                    }
                    else
                    {
                        Set_Selected_Menu_Line((LPMENU) Self, 6, FALSE);
                    }
                    break;
            }
            break;

        case PAD_CANCEL:

            Hide_Menu((LPMENU) Self, Pointer);
            Destroy_Menu(Menu_Stats_Character);
            Show_Menu(Menu_Create_Character, Pointer, -1, -1, 0);
            break;
    }
}

/* --------------------------------
   Enter character name
   -------------------------------- */
void Name_Menu_Events(void *Self, int Item, ULONG Pad_Dats)
{
    int i;
    int Line;
    int Column;
    int Len;
    char Name[21];
    char Text_Line[32];
    LPCHARACTER Character;

    switch(Pad_Dats)
    {
        case PAD_OK:

            Line = Get_Selected_Menu_Line((LPMENU) Self);
            Column = Get_Selected_Menu_Column((LPMENU) Self);
            Character = &Party.Characters[Party.Number_Characters];
            if(Line == 4 && (Column == 9 || Column == 10))
            {
                switch(Column)
                {
                    case 9:

                        /* Delete */
                        Len = strlen(Character->Name);
                        if(Len > 0)
                        {
                            memset(Name, 0, MAX_CHARACTER_NAME);
                            Len--;
                            Character->Name[Len] = '\0';
                            strcpy(Name, Character->Name);
                            for(i = (MAX_CHARACTER_NAME - 1); i >= Len; i--)
                            {
                                Name[i] = '_';
                            }
                            sprintf(Text_Line, "NAME: %s", Name);
                            Set_Menu_Line_Text(Menu_Stats_Character, Text_Line, 6, '_', TRUE);
                        }
                        break;

                    case 10:

                        Hide_Menu((LPMENU) Self, Pointer);
                        break;
                }
            }
            else
            {
                /* Any other char */
                Len = strlen(Character->Name);
                if(Len < (MAX_CHARACTER_NAME - 1))
                {
                    memset(Name, 0, MAX_CHARACTER_NAME);
                    strcpy(Name, Character->Name);
                    Name[Len] = Chars_Name_List[(Line * 11) + Column];
                    strcpy(Character->Name, Name);
                    Len++;
                    for(i = (MAX_CHARACTER_NAME - 1); i >= Len; i--)
                    {
                        Name[i] = '_';
                    }
                    sprintf(Text_Line, "NAME: %s", Name);
                    Set_Menu_Line_Text(Menu_Stats_Character, Text_Line, 6, '_', TRUE);
                }
            }
            break;

        case PAD_CANCEL:

            Character = &Party.Characters[Party.Number_Characters];
            memcpy(Character->Name, Old_Character_Name, MAX_CHARACTER_NAME);
            sprintf(Text_Line, "NAME: %s", Character->Name);
            Set_Menu_Line_Text(Menu_Stats_Character, Text_Line, 6, '_', TRUE);
            Hide_Menu((LPMENU) Self, Pointer);
            break;
    }
}

/* --------------------------------
   Withdraw gold
   -------------------------------- */
void WithDraw_Menu_Events(void *Self, int Item, ULONG Pad_Dats)
{
    LPCHARACTER Character = NULL;
    int Line;
    int i;

    int Speed = Pad_Dats & PAD_SPEED ? 10 : 1;

    Pad_Dats = Pad_Dats & ~PAD_SPEED;

    switch(Pad_Dats)
    {
        case PAD_LEFT:

            Line = Get_Selected_Menu_Line((LPMENU) Self) - 4;
            if(Line < Party.Number_Characters)
            {
                Character = &Party.Characters[Line];
                Move_Gold(Character, Line, WithDraw_Gold, Speed);
                Display_Character_And_Party_Gold(Character, Line);
            }

            break;

        case PAD_RIGHT:

            Line = Get_Selected_Menu_Line((LPMENU) Self) - 4;
            if(Line < Party.Number_Characters)
            {
                Character = &Party.Characters[Line];
                Move_Gold(Character, Line, WithDraw_Gold, -Speed);
                Display_Character_And_Party_Gold(Character, Line);
            }
            break;

        case PAD_OK:

            Line = Get_Selected_Menu_Line((LPMENU) Self) - 4;
            if(Line == Party.Number_Characters + 1)
            {
                /* Record the gold movements */
                Party.Gold = Temp_Gold;
                for(i = 0; i < Party.Number_Characters; i++)
                {
                    Party.Characters[i].Gold = Temp_WithDraw_Gold[i];
                }
                Hide_Menu((LPMENU) Self, Pointer);
                Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
            }
            break;

        case PAD_CANCEL:

            Hide_Menu((LPMENU) Self, Pointer);
            Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
            break;
    }
}

/* --------------------------------
   Init the town context
   -------------------------------- */
BOOL Init_Town(int Town_Nbr)
{
    Fade_Town_Pic();
    Static_To_Screen(Pic_Towns, 0, 0, 4, 0,
                     Pic_Towns->Width, Pic_Towns->Height);

    Menu_Create_Character = Construct_Menu(Font,
                                           Pic_Menus,
                                           &Create_Menu_Events,
                                           MENU_TEXT_CENTER,
                                           6, 2,
                                           1, 0,
                                           MOVE_VERTICALLY | MOVE_HORIZONTALLY,
                                           "%s\n%s\n%s\n%s\n%s\n%s",
                                           "CREATE CHARACTER",
                                           " ",
                                           "RACE:       <          >",
                                           "CLASS:      <          >",
                                           " ",
                                           "DONE"
                                          );

    Menu_Character_Name = Construct_Menu(Font,
                                         Pic_Menus,
                                         &Name_Menu_Events,
                                         MENU_TEXT_LEFT,
                                         5, 0,
                                         11, 2,
                                         MOVE_VERTICALLY | MOVE_HORIZONTALLY,
                                         "%s\n%s\n%s\n%s\n%s",
                                         "A  B  C  D  E  F  G  H  I  J  K",
                                         "L  M  N  O  P  Q  R  S  T  U  V",
                                         "W  X  Y  Z  a  b  c  d  e  f  g",
                                         "h  i  j  k  l  m  n  o  p  q  r",
                                         "s  t  u  v  w  x  y  z     <  @"
                                        );

    Set_Current_Town(Town_Nbr);
    Fix_Current_Town();

    Current_Fading = NO_FADING;

    if(!Party.Number_Characters)
    {
        /* Clean up */
        memset(&Party, 0, sizeof(PARTY));

        /* Run create character menu directly */
        Start_Character_Creation();
    }
    else
    {
        if(!Calc_Age())
        {
            Construct_Main_Town_Menu();
            Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
        }
    }
    return TRUE;
}

/* --------------------------------
   Run in the town context
   -------------------------------- */
void Loop_Town(ULONG Pad_Dats)
{
    LPMENU Cur_Menu;

    Cur_Menu = Get_Focus_Menu();
    if(Cur_Menu)
    {
        Erase_Pointer(Pointer);
        Refresh_Menu(Cur_Menu);
        if(!(Cur_Menu->Flags & MENU_NO_POINTER))
        {
            Display_Pointer(Pointer, Cur_Menu, Get_Selected_Menu_Line(Cur_Menu), 36, 9);
        }
        Dispatch_Menu_Events(Pad_Dats);
    }
    else
    {
        if(!Doing_Fade)
        {
            if(Current_Fading == FADING_TO_SHEET)
            {
                /* Remove the town picture */
                Set_Town_Pic(TRUE);
    
                /* Background fade is done,
                   prepare the screen to display the character's sheet */
                Prepare_Character_Sheet_Context(IN_TOWN, Character_Sheet_Idx);
    
                Current_Fading = NO_FADING;
            }
        }
    }
}

/* --------------------------------
   Exit the town
   -------------------------------- */
void Destroy_Town()
{
    Destroy_Sheet();
    Destroy_Menu(Menu_Age_Notification);
    Destroy_Menu(Menu_Withdraw);
    Destroy_Menu(Menu_Stats_Character);
    Destroy_Menu(Menu_Create_Character);
    Destroy_Menu(Menu_Character_Name);
    Destroy_Menu(Menu_Town_Main);
}

/* --------------------------------
   Exit the town
   -------------------------------- */
void Construct_Main_Town_Menu()
{
    char Menu_Text[512];
    char Char_Entry[64];
    int i;
    int Index = 0;

    memset(Menu_Text, 0, sizeof(Menu_Text));

    Characters_Menu_Index = Index;
    for(i = 0; i < Party.Number_Characters; i++)
    {
        sprintf(Char_Entry, "%-12s ", Party.Characters[i].Name);
        strcat(Menu_Text, Char_Entry);
        sprintf(Char_Entry, "%10s ", Races_List[Party.Characters[i].Race]);
        strcat(Menu_Text, Char_Entry);
        if(Party.Characters[i].Status == DEAD)
        {
            sprintf(Char_Entry, "%7s\n", "*DEAD*");
            strcat(Menu_Text, Char_Entry);
        }
        else
        {
            sprintf(Char_Entry, "%7s\n", Classes_List[Party.Characters[i].Class]);
            strcat(Menu_Text, Char_Entry);
        }
        Index++;
    }

    if(Party.Number_Characters)
    {
        strcat(Menu_Text, " \n");
        Index++;
    }
    if(Party.Number_Characters < 6)
    {
        Create_Character_Menu_Index = Index;
        strcat(Menu_Text, "CREATE CHARACTER\n");
        Index++;
    }
    Consult_Mystic_Menu_Index = Index;
    strcat(Menu_Text, "CONSULT MYSTIC\n");
    Index++;
    Rest_At_Inn_Menu_Index = Index;
    strcat(Menu_Text, "REST AT INN\n");
    Index++;
    Withdraw_Gold_Menu_Index = Index;
    strcat(Menu_Text, "WITHDRAW GOLD\n");
    Index++;
    Deposit_Gold_Menu_Index = Index;
    strcat(Menu_Text, "DEPOSIT GOLD\n");
    Index++;

    if(Can_Save)
    {
        Save_Game_Menu_Index = Index;
        strcat(Menu_Text, "SAVE GAME\n");
        Index++;
    }

    Leave_Town_Menu_Index = Index;
    strcat(Menu_Text, "LEAVE TOWN\n");
    Index++;

    if(Menu_Town_Main)
    {
        Destroy_Menu(Menu_Town_Main);
    }

    /* Construct the list of characters */
    Menu_Town_Main = Construct_Menu(Font,
                                    Pic_Menus,
                                    &Full_Town_Menu_Events,
                                    MENU_TEXT_CENTER,
                                    Index, 0,
                                    1, 0,
                                    MOVE_VERTICALLY,
                                    Menu_Text
                                   );
}

/* --------------------------------
   Display the race of a character and modify the class if necessary
   -------------------------------- */
void Display_Choose_Character_Race(LPMENU Self,
                                   LPCHARACTER Character,
                                   int Line,
                                   BOOL Refresh)
{
    char Text_Line[32];

    sprintf(Text_Line, "RACE:       < %8s >", Races_Create_List[Character->Race]);
    Set_Menu_Line_Text((LPMENU) Self, Text_Line, Line, ' ', Refresh);
    if(Character->Race == RACE_RANDOM)
    {
        Nbr_Classes = sizeof(Random_Classes) / sizeof(BYTE);
        Ptr_Classes = Random_Classes;
        Character->Class = THIEF;
        Display_Choose_Character_Class(Self, Character, Line + 1, Refresh);
    }
    else
    {
        Nbr_Classes = sizeof(Normal_Classes) / sizeof(BYTE);
        Ptr_Classes = Normal_Classes;
    }
}

/* --------------------------------
   Display the class of a character
   -------------------------------- */
void Display_Choose_Character_Class(LPMENU Self,
                                    LPCHARACTER Character,
                                    int Line,
                                    BOOL Refresh)
{
    char Text_Line[32];

    sprintf(Text_Line, "CLASS:      <  %7s >", Classes_List[Ptr_Classes[Character->Class]]);
    Set_Menu_Line_Text((LPMENU) Self, Text_Line, Line, ' ', Refresh);
}

/* --------------------------------
   Display the Race of a character and modify the class if necessary
   -------------------------------- */
void Start_Character_Creation()
{
    LPCHARACTER Character;

    if(Menu_Town_Main)
    {
        Hide_Menu(Menu_Town_Main, Pointer);
        Destroy_Menu(Menu_Town_Main);
    }

    Nbr_Classes = sizeof(Normal_Classes) / sizeof(BYTE);
    Ptr_Classes = Normal_Classes;

    Character = &Party.Characters[Party.Number_Characters];

    /* Reset character slot */
    memset(Character, 0, sizeof(CHARACTER));

    Display_Choose_Character_Race(Menu_Create_Character,
                                  Character,
                                  2, FALSE);
    Display_Choose_Character_Class(Menu_Create_Character,
                                   Character,
                                   3, FALSE);
    Show_Menu(Menu_Create_Character, Pointer, -1, -1, 0);
}

/* --------------------------------
   Display the base stats of a character
   -------------------------------- */
void Display_Character_Base_Stats(LPCHARACTER Character, int Race)
{
    if(Menu_Stats_Character)
    {
        Destroy_Menu(Menu_Stats_Character);
    }

    Menu_Stats_Character = Construct_Menu(Font,
                                          Pic_Menus,
                                          &Base_Stats_Menu_Events,
                                          MENU_TEXT_LAST_LINE_CENTER,
                                          9, 6,
                                          1, 0,
                                          MOVE_VERTICALLY,
                                          "RACE:  %10s STRENGTH:     %2d\n"
                                          "CLASS:    %7s INTELLIGENCE: %2d\n"
                                          "                  DEXTERITY:    %2d\n"
                                          "                  CONSTITUTION: %2d\n"
                                          "                  CHARISMA:     %2d\n"
                                          " \n"
                                          "NAME: ____________\n"
                                          " \n"
                                          "DONE",
                                          Races_List[Race],
                                          Character->Strength,
                                          Classes_List[Ptr_Classes[Character->Class]], Character->Intelligence,
                                          Character->Dexterity,
                                          Character->Constitution,
                                          Character->Charisma
                                         );
    Show_Menu(Menu_Stats_Character, Pointer, -1, -1, 0);
    /* Enter a name */
    memcpy(Old_Character_Name, Character->Name, MAX_CHARACTER_NAME);
    Show_Menu(Menu_Character_Name, Pointer, -1, 185, 0);
}

/* --------------------------------
   Load all the towns so they're always present in memory
   for the saving process and the teleportation
   -------------------------------- */
int Load_Towns(void)
{
    char Town_Dat_Name[32];
    BYTE *Town_File;
    int i;

    for(i = 0; i < MAX_TOWNS; i++)
    {
        sprintf(Town_Dat_Name, "dat/TWN%d.BIN", i + 1);
        Town_File = (BYTE *) Load_File(Town_Dat_Name);
        if(Town_File == NULL) return FALSE;
        memcpy(&Towns[i], Town_File, sizeof(TOWNDAT) - sizeof(BOOL));
        Towns[i].Loaded = TRUE;
        Destroy_File(Town_File);
    }
    return TRUE;
}

/* --------------------------------
   Display the message from the mystic
   -------------------------------- */
void Display_Mystic_Message()
{
    int i;
    int j;
    LPCHARACTER Character = NULL;
    int Have_Orb = 0;
    int Have_Wand = 0;
    int Exp_Idx = 0;
    int Party_Score = 0;
    int Have_Runes = 0;
    int CScore;
    int Item;
    BOOL Decrypted_Some_Amulets = FALSE;
    char Mystic_Message[512];    
    char Heroes_Message[64];
    LPMENU Menu_Mystic;

    memset(Mystic_Message, 0, sizeof(Mystic_Message));

    for(i = 0; i < Party.Number_Characters; i++)
    {
        Character = &Party.Characters[i];

        for(j = 0; j < MAX_CHARACTER_ITEMS; j++)
        {
            Item = Character->Items[j];

            /* Check if any of the character have an amulet which isn't decrypted yet */
            if(Item >= AMULET_1 &&
               Item <= AMULET_8)
            {
                /* Decrypt it if necessary */
                if(Party.Beast_Runes[Item - AMULET_1] != DECRYPTED)
                {
                    Party.Beast_Runes[Item - AMULET_1] = DECRYPTED;
                    Decrypted_Some_Amulets = TRUE;
                }
            }

            /* Does any of the character have the ORB ? */
            Have_Orb |= Item == ORB;
        }
        
        /* Does any of the character have the wand of nikademus ? */
        Have_Wand |= Character->Wand_Of_Nikademus > 99;
        
        /* Avoid a GCC warning bug */
        CScore = Character->Score;
        if(CScore > 150)
        {
            Exp_Idx = 5;
        }
    }
 
    if(Decrypted_Some_Amulets)
    {
        Display_Message(Font, Pic_Menus, -1, "THE MYSTIC HELPED YOU TO SHED MORE LIGHT ON THE AMULETS...");
    }

    for(i = 0; i < MAX_BEAST_RUNES; i++)
    {
        Have_Runes |= Party.Beast_Runes[i] == DECRYPTED;
        Party_Score += (10 * (Party.Beast_Runes[i] == DECRYPTED));
    }
    Party_Score += (20 * Have_Orb);

    Exp_Idx += (Party_Score > 10) + (Party_Score > 60) + (Party_Score > 90);
    if(Party_Score > 50 && Have_Orb == 0)
    {
        Exp_Idx = 4;
    }
    if(Exp_Idx > 5) Exp_Idx = 5;

    /* Update characters scores */
    for(i = 0; i < Party.Number_Characters; i++)
    {
        Character = &Party.Characters[i];
        CScore = Character->Score;
        if(CScore < Party_Score)
        {
            Character->Score = Party_Score;
        }
    }

    strcpy(Mystic_Message, "%s MYSTIC\n"
                           " \n"
                           "THE WISE OLD MAN PEERS INTO\n"
                           "HIS CRYSTAL BALL AND SPEAKS...\n"
                           " \n"
                           "YOUR SCORE IS %d\n"
                           " \n"
                           "YOU ARE %s");

    if(Have_Wand)
    {
        sprintf(Heroes_Message, "\nAND ALSO HEROES OF %s", Origin_Land[0]);
        strcat(Mystic_Message, Heroes_Message);
    }

    if(Have_Runes)
    {
        strcat(Mystic_Message, "\nYOU POSSESS THE FOLLOWING BEAST RUNES:\n ");
        for(i = 0 ; i < MAX_BEAST_RUNES; i++)
        {
            if(Party.Beast_Runes[i] == DECRYPTED)
            {
                strcat(Mystic_Message, "\n");
                strcat(Mystic_Message, Runes_List[i]);
            }
        }
    }

    Menu_Mystic = Construct_Menu(Font,
                                 Pic_Menus,
                                 NULL,
                                 MENU_TEXT_CENTER | MENU_NO_POINTER,
                                 9, 6,
                                 1, 0,
                                 MOVE_NONE,
                                 Mystic_Message,
                                 Cur_Town->Name,
                                 Party_Score,
                                 Party.Number_Characters > 1 ? Adventurers[Exp_Idx] : Adventurer[Exp_Idx]
                                );
    Hide_Menu(Menu_Town_Main, Pointer);
    Show_Menu(Menu_Mystic, NULL, -1, -1, 0);
    Wait_Pad_Ok_Cancel();
    Hide_Menu(Menu_Mystic, NULL);
    Destroy_Menu(Menu_Mystic);
    Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
}

/* --------------------------------
   Display the inn message and replenish characters
   hit points and magic
   -------------------------------- */
void Rest_At_Inn()
{
    char Message[256];
    int i;
    LPCHARACTER Character = NULL;
    LPMENU Menu_Inn;

    strcpy(Message, "AT %s INN...\n \n");
 
    for(i = 0; i < Party.Number_Characters; i++)
    {
        Character = &Party.Characters[i];
        if(Character->Status != DEAD)
        {
            strcat(Message, "\n");
            strcat(Message, Character->Name);
            strcat(Message, " IS RESTING");

            /* Did character use magic ? */
            if(Character->Used_Magic)
            {
                Character->Used_Magic = 0;
                if((rnd() * 100) <= (20.0f * Character->Max_Class_Magic - Character->Max_Magic))
                {
                    Character->Max_Magic++;
                    strcat(Message, "\n");
                    strcat(Message, Character->Name);
                    strcat(Message, " MAGIC INCREASES");
                }
            }

            Character->Current_Magic = Character->Max_Magic;
            Character->Current_Hit_Points = Character->Max_Hit_Points;
        }
    }

    Menu_Inn = Construct_Menu(Font,
                              Pic_Menus,
                              NULL,
                              MENU_TEXT_CENTER | MENU_NO_POINTER,
                              9, 6,
                              1, 0,
                              MOVE_NONE,
                              Message,
                              Cur_Town->Name
                             );
    Hide_Menu(Menu_Town_Main, Pointer);
    Show_Menu(Menu_Inn, NULL, -1, -1, 0);
    Wait_Pad_Ok_Cancel();
    Hide_Menu(Menu_Inn, NULL);
    Destroy_Menu(Menu_Inn);
    Show_Menu(Menu_Town_Main, Pointer, -1, -1, 0);
}

/* --------------------------------
   Remove gold from the bank
   -------------------------------- */
void Manage_Gold(BOOL WithDraw)
{
    int i;
    char Message[256];
    char Line[64];

    if(Menu_Withdraw)
    {
        Destroy_Menu(Menu_Withdraw);
    }

    WithDraw_Gold = WithDraw;

    if(WithDraw_Gold)
    {
        strcpy(Message, "WITHDRAW GOLD FROM %s BANK\n \n");
    }
    else
    {
        strcpy(Message, "DEPOSIT GOLD AT %s BANK\n \n");
    }
    strcat(Message, "GOLD PIECES CARRIED: %5d\n \n");

    Temp_Gold = Party.Gold;

    /* Construct the menu */
    for(i = 0; i < Party.Number_Characters; i++)
    {
        Temp_WithDraw_Gold[i] = Party.Characters[i].Gold;

        sprintf(Line, "%12s:    < %5d >\n", Party.Characters[i].Name, Temp_WithDraw_Gold[i]);
        strcat(Message, Line);
    }

    strcat(Message, " \nDONE");

    Menu_Withdraw = Construct_Menu(Font,
                                   Pic_Menus,
                                   &WithDraw_Menu_Events,
                                   MENU_TEXT_CENTER,
                                   9, 4,
                                   1, 0,
                                   MOVE_VERTICALLY | MOVE_HORIZONTALLY,
                                   Message,
                                   Cur_Town->Name,
                                   Temp_Gold
                                  );

    Hide_Menu(Menu_Town_Main, Pointer);
    Show_Menu(Menu_Withdraw, Pointer, -1, -1, 0);
}

/* --------------------------------
   Display the gold of a character and the party gold
   -------------------------------- */
void Display_Character_And_Party_Gold(LPCHARACTER Character, int Line)
{
    char Text_Line[48];

    sprintf(Text_Line, "%12s:    < %5d >\n", Character->Name, Temp_WithDraw_Gold[Line]);
    Set_Menu_Line_Text(Menu_Withdraw, Text_Line, Line + 4, ' ', TRUE);

    sprintf(Text_Line, "GOLD PIECES CARRIED: %5d\n", Temp_Gold);
    Set_Menu_Line_Text(Menu_Withdraw, Text_Line, 2, ' ', TRUE);
}

/* --------------------------------
   Get the amount of gold carried by the party
   -------------------------------- */
void Move_Gold(LPCHARACTER Character,
               int i,
               BOOL WithDraw,
               int Speed)
{
    int Old_Value;

    if(WithDraw)
    {
        Old_Value = Temp_WithDraw_Gold[i];
        if(Speed > 0)
        {
            Temp_WithDraw_Gold[i] -= Speed;
            if(Temp_WithDraw_Gold[i] < 0)
            {
                Temp_WithDraw_Gold[i] = 0;
            }
            Temp_Gold += Old_Value - Temp_WithDraw_Gold[i];
        }
        else
        {
            Temp_WithDraw_Gold[i] += -Speed;
            if(Temp_WithDraw_Gold[i] > Character->Gold)
            {
                Temp_WithDraw_Gold[i] = Character->Gold;
            }
            Temp_Gold -= Temp_WithDraw_Gold[i] - Old_Value;
        }
    }
    else
    {
        /* Deposit */
        Old_Value = Temp_Gold;
        if(Speed > 0)
        {
            Temp_Gold += Speed;
            if(Temp_Gold > Party.Gold)
            {
                Temp_Gold = Party.Gold;
            }
            Temp_WithDraw_Gold[i] -= Temp_Gold - Old_Value;
        }
        else
        {
            Temp_Gold -= -Speed;
            if(Temp_Gold < 0)
            {
                Temp_Gold = 0;
            }
            Temp_WithDraw_Gold[i] += Old_Value - Temp_Gold;
        }
    }
}

/* --------------------------------
   Clear the town pic without and redraw the town name
   (That's a special case)
   -------------------------------- */
void Set_Town_Pic(BOOL Clear)
{
    if(Clear)
    {
        Clear_Array(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
    }
    else
    {
        Copy_To_Screen(Pic_Towns, 0, 0, 4, 0, Pic_Towns->Width, Pic_Towns->Height);
        Copy_To_Screen(Pic_Menus, TOWN_NAME_X, TOWN_NAME_Y,
                       (SCREEN_WIDTH - TOWN_NAME_WIDTH) / 2, TOWN_NAME_Y_DEST, TOWN_NAME_WIDTH, TOWN_NAME_HEIGHT);
        Print(Font, 0, -1, TOWN_NAME_Y_DEST + 10, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "%s", Cur_Town->Name);
    }
    
    Swap_Physical_Buffers();

    if(Clear)
    {
        Clear_Array(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
    }
    else
    {
        Copy_To_Screen(Pic_Towns, 0, 0, 4, 0, Pic_Towns->Width, Pic_Towns->Height);
        Copy_To_Screen(Pic_Menus, TOWN_NAME_X, TOWN_NAME_Y,
                       (SCREEN_WIDTH - TOWN_NAME_WIDTH) / 2, TOWN_NAME_Y_DEST, TOWN_NAME_WIDTH, TOWN_NAME_HEIGHT);
        Print(Font, 0, -1, TOWN_NAME_Y_DEST + 10, SCREEN_WIDTH, FALSE, PRINT_BACKGROUND, "%s", Cur_Town->Name);
    }
}

/* --------------------------------
   Prepare town picture palette fading
   -------------------------------- */
void Fade_Town_Pic()
{
    int i;

    for(i = 0; i < TOWN_COLORS; i++)
    {
        Current_Palette[i] = Towns_Palette[i];
    }
    for(i = 239; i < 239 + MENUS_COLORS; i++)
    {
        Current_Palette[i] = Menus_Palette[i];
    }
    Current_Palette[254] = 0xa5a09a;
    Current_Palette[255] = 0;
    Current_Palette[237] = 0x706a60;
    Current_Palette[238] = 0xc5c0ba;
    Set_Fade(Current_Palette, 12, MOVE_HORIZONTALLY | MOVE_VERTICALLY);
    Set_Palette(Black_Palette, 0, TOWN_COLORS);
}

/* --------------------------------
   Change the stock of the town's armory a bit
   Party level is set when leaving the town
   -------------------------------- */
void Randomize_Town_Stock()
{
    int i;

    /* Generate an "out of stock" status for 17 items
       amonst first 110 ones.
     */
    for(i = 0; i < 18; i++)
    {
        Cur_Town->Stock[(int) (rnd() * 110)] = 0;
    }
    for(i = 0; i < 9; i++)
    {
        Cur_Town->Stock[Cur_Town->Stock_Modifiers[(int) ((rnd() * Party.Level))]] = 1;
    }
}

/* --------------------------------
   Set the current town number
   -------------------------------- */
void Set_Current_Town(int Town_Nbr)
{
    Old_Town_Nbr = Cur_Town_Nbr;
    Cur_Town_Nbr = Town_Nbr;
}

/* --------------------------------
   Fix the current town data and name
   -------------------------------- */
void Fix_Current_Town(void)
{
    Cur_Town = &Towns[Cur_Town_Nbr - 1];

    Prices_Multiplier = (float) Cur_Town->Multiplier / 100.0f;

    Static_To_Screen(Pic_Menus, TOWN_NAME_X, TOWN_NAME_Y,
                     (SCREEN_WIDTH - TOWN_NAME_WIDTH) / 2,
                     TOWN_NAME_Y_DEST, TOWN_NAME_WIDTH, TOWN_NAME_HEIGHT);

    Print(Font, 0, -1, TOWN_NAME_Y_DEST + 10,
          SCREEN_WIDTH, TRUE, PRINT_BACKGROUND, "%s", Cur_Town->Name);

    if(Old_Town_Nbr != Cur_Town_Nbr)
    {
        /* Restock only if we changed town */
        Randomize_Town_Stock();
    }
}
